from datetime import datetime
import os

from PySide2.QtGui import QIcon
from loguru import logger
from axipy import Plugin, Position

from PanoramaRsc.ui.EditRscDictinaryDb import EditDbRsc
from PanoramaRsc.ui.RscTools import ToolsRsc
from PanoramaRsc.ui.ToolExportToFgisGml import ToolExportFigisTpGml
from PanoramaRsc.ui.add_tools.helper_tab.add_tab import SetDefaultValueInTab
from PanoramaRsc.ui.add_tools.observers import factoryObserver
from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict
from PanoramaRsc.ui.convertRscToDb import ConvertRscToDbDict
from PanoramaRsc.ui.createAndLoadPanoramaToGpkg import ToolsCreateAndLoadPanorama


class PanoramaRscPlugin(Plugin):
    def load(self) -> None:

        self.__init_log()
        self.__observer_config=factoryObserver("PanoramaRscConfig")
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__observer_select_from_dict = factoryObserver("PanoramaRscSelect")
        ConfigRscDict.path_folder=self.get_plugin_data_dir()

        self.__configDbRsc=ConfigRscDict.load(self.get_plugin_data_dir())
        self.__observer_change_config.changed.connect(self.__reloadConfig)
        self.__clsAddDefaultValue = SetDefaultValueInTab(self.__configDbRsc)
        if self.__configDbRsc is not None:
            self.__observer_config.value=True
        self.__cls_actions=[]
        self.__cls_actions.append(ToolsRsc(self))
        self.__cls_actions.append(EditDbRsc(self.__configDbRsc,self))
        self.__cls_actions.append(ConvertRscToDbDict(self))
        self.__cls_actions.append(ToolsCreateAndLoadPanorama(self))
        self.__cls_actions.append(ToolExportFigisTpGml(self))

        position = Position('Панорама Rsc', 'Команды')
        for action_tool in self.__cls_actions:
            position.add(action_tool.action)
    @property
    def configDbRsc(self):
        return self.__configDbRsc
    def __init_log(self):
        path_data_plugin = self.get_plugin_data_dir()
        path_folder_log = path_data_plugin.joinpath("log")
        if not path_folder_log:
            os.makedirs(str(path_folder_log))
        name_log="PanoramaRscPlugin"+datetime.now().strftime("%Y_%m_%d_%H_%M_%S")+".log"
        logger.add(str(path_folder_log.joinpath(name_log)), level='DEBUG',format="{time:MMMM D, YYYY > HH:mm:ss} | {level} | {message} | {extra}")
    def unload(self) -> None:
        for action in self.__cls_actions:
            action.unload()
            action.action.remove()
    def __reloadConfig(self):
        self.__configDbRsc=ConfigRscDict.load()
        self.__observer_config.value=True


