import os

import axipy
from PySide2.QtCore import QObject
from PySide2.QtGui import QIcon
from loguru import logger

from PanoramaRsc.ui.add_tools.ObserverEditRecord import ObserverEnableEditRecord
from PanoramaRsc.ui.add_tools.observers import factoryObserver
from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict
from PanoramaRsc.ui.dlgFormEdit import FormEdit
from PanoramaRsc.ui.docWidget import AxiDocWidget, addDoc, existDocWidget, DelDocWidget


class EditDbRsc(QObject):
    def __init__(self,config:ConfigRscDict,parent_plugin):
        super().__init__()
        self.__name_doc = "Редактирование"
        self.__parent_plugin = parent_plugin
        self.__form_tool = None
        self.__docWidget = None
        self.__config=config
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__name_observer_ready="PanoramaRscSelect"
        self.__observer_select_from_dict = factoryObserver(self.__name_observer_ready)
        self.__observer_select_from_dict.changed.connect(self.__DisbaleCloseDocWidget)
        self.__local_file_icon_tools = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'icons',
                                                    'edit-database-32.png')
        self.__observer_ready_edit=ObserverEnableEditRecord(self.__config)

        self.__iconTools = QIcon(self.__local_file_icon_tools)
        self.__title = self.tr(self.__name_doc)
        self.__action = self.__parent_plugin.create_action(
            self.__title,
            icon=self.__local_file_icon_tools,
            on_click=self.__run_edit,enable_on=self.__name_observer_ready
        )
        self.__observer_ready_edit.enable_edit_select.connect(self.__change_select_editable)
        self.__observer_change_config.changed.connect(self.__change_config)
    def __run_edit(self):
        print("click run")
        if self.__form_tool is None:
            self.__form_tool=FormEdit(self.__config,axipy.mainwindow.qt_object())
        active_table,base_active=self.__observer_ready_edit.activeTable
        if active_table is None or base_active is None:
            logger.debug("Error find active table")
            return

        self.__form_tool.setActiveTable(active_table,base_active)
        if self.__docWidget is None:
            self.__docWidget = AxiDocWidget(self.__form_tool.widget, self.__name_doc, self.__name_doc,
                                                QIcon(), axipy.mainwindow.qt_object())
            self.__docWidget.close_doc.connect(self.__close_doc_widget)
            addDoc(axipy.app.mainwindow.qt_object(), self.__docWidget)

        else:
            isExist = existDocWidget(axipy.mainwindow.qt_object(), self.__docWidget)
            self.__docWidget.setVisible(True)
            self.__docWidget.setFocus()
            self.__docWidget.raise_()
    def __close_doc_widget(self,name_tool,status):
        if name_tool!=self.__name_doc:
            return

        if self.__form_tool is not None:
            if not status:
                self.__form_tool.clear_temp()
            else:
                active_table, base_active = self.__observer_ready_edit.activeTable
                self.__form_tool.setActiveTable(active_table, base_active)
            pass
    def __change_select_editable(self):
        if self.__docWidget is None:
            return
        self.__run_edit()
    @property
    def action(self):
        return self.__action
    def __DisbaleCloseDocWidget(self):
        if (not self.__observer_select_from_dict.value) and self.__docWidget is not None:

            #DelDocWidget(axipy.view_manager.global_parent, self.__docWidget)
            try:
                axipy.mainwindow.qt_object().removeDockWidget(self.__docWidget)
            except:
                jkl=0
            self.__docWidget=None
            self.__close_doc_widget(self.__name_doc,False)
            self.__form_tool =None
    def __change_config(self):
        self.__config=ConfigRscDict.load()

    def unload(self):
        if self.__docWidget is not None:
            axipy.mainwindow.qt_object().removeDockWidget(self.__docWidget)
            self.__docWidget=None