import os
from pathlib import Path

from PanoramaRsc.ui.add_tools.axi_polygon_style.xpm import buildXpmFillHatch, buildXpmHatchSquareShift, \
    factorySquareVector
from PanoramaRsc.ui.add_tools.rsc_tools.RSC.add_help.saveBrush.cls_saveBrush import writeImageBytes
from PanoramaRsc.ui.add_tools.rsc_tools.RSC.panSymbol import PanDrawSymbol, PanSquare, FillHatch, PanHatchSquareShift, \
    SquareVector
from PanoramaRsc.ui.add_tools.rsc_tools.panoramaToAxiPen.factoryAxiPen import convertPanDrawSymbolToAxiPen, getColor
from PanoramaRsc.ui.add_tools.rsc_tools.panoramaToAxiPen.factoryPen.penFactory import AxiPenPatern


class WritePenToFolder:
    def __init__(self,path_folder,start_number_pen):
        self.__path_out=Path(path_folder)
        self.__start_number=start_number_pen
        self.__index_pen=0
    def write(self,obj_trtp):
        cur_pen=self.__start_number+self.__index_pen
        curent_number_str="pen_"+"%03d" % cur_pen
        path_out_pen=self.__path_out.joinpath(curent_number_str+".trtp")
        obj_trtp.writeToFile(str(path_out_pen))
        self.__index_pen=self.__index_pen+1
        return cur_pen
    def writeByName(self,name,obj_trtp):
        cur_pen = self.__start_number + self.__index_pen
        curent_number_str = "%03d" % cur_pen
        path_out_pen = self.__path_out.joinpath(name + ".trtp")
        obj_trtp.writeToFile(str(path_out_pen))
    @property
    def curentPen(self):
        return self.__start_number + self.__index_pen-1
class WriteBrushToFolder:
    def __init__(self,path_folder,start_number_brush):
        self.__path_out=Path(path_folder)
        self.__start_number=start_number_brush
        self.__index_brush=0
    def write(self,obj_xpm):
        cur_pen=self.__start_number+self.__index_brush
        curent_number_str="pat%03d" % cur_pen
        path_out_pen=self.__path_out.joinpath(curent_number_str+".xpm")
        writeImageBytes(obj_xpm,str(path_out_pen))
        self.__index_brush=self.__index_brush+1
        return cur_pen

    @property
    def curentBrush(self):
        return self.__start_number + self.__index_brush-1

    @property
    def curentPen(self):
        return self.__start_number + self.__index_brush - 1

class TempContainerPen:
    def __init__(self,path_out_pen_folder,start_number,type="PEN"):
        self.__container=[]
        if type=="PEN":
            self.__save_cls = WritePenToFolder(str(path_out_pen_folder), start_number)
        else:
            self.__save_cls = WriteBrushToFolder(str(path_out_pen_folder), start_number)

    def add(self,obj_pen):
        if isinstance(obj_pen,AxiPenPatern):
            obj_pen_str=str(obj_pen)
            index_exist=self.__exist(obj_pen_str)
            if index_exist is not None:
                return None,self.__save_cls.curentPen
            self.__container.append(obj_pen_str)
            curent_pen=self.__save_cls.write(obj_pen)
            return len(self.__container),curent_pen
        ''' xpm'''
        index_exist=self.__exist(obj_pen)
        if index_exist is not None:
            return None, self.__save_cls.curentPen
        self.__container.append(obj_pen)
        curent_brush = self.__save_cls.write(obj_pen)
        return len(self.__container), curent_brush

    def __exist(self,obj_pen):
        for i,pen in enumerate(self.__container):
            if pen==obj_pen:
                return i
        return None
class BuildBrushStyle:
    def __init__(self,path_xpm,palette,start_id=0):
        self.__path_out=path_xpm
        self.__paltte=palette
        self.__start_id=start_id
    def build(self,object):
        for sym in object.Symbols:
            if isinstance(sym,PanSquare):
                color=getColor(sym.ColorRGBA,self.__paltte)
                color_int=color.rgb()-4278190080
                return "Brush(2,"+str(color_int)+",16777215)",None
            if isinstance(sym,FillHatch):
                xpm_bytes,color=buildXpmFillHatch(sym,self.__paltte)
                return xpm_bytes,color
            if isinstance(sym,PanHatchSquareShift):
                xpm_bytes,color=buildXpmHatchSquareShift(sym,self.__paltte)
                return xpm_bytes,color
            if isinstance(sym,SquareVector):
                xpm_bytes,color=factorySquareVector(sym,self.__paltte)
                return xpm_bytes, color

            print(sym)
        return None,None
class AxiPolygonStyle:
    def __init__(self,path_folder_temp,palette,start_number_pen,start_number_brush):
        self.__base_folder=Path(path_folder_temp)
        self.__path_xmp=self.__base_folder.joinpath("Brush")
        if not self.__path_xmp.exists():
            os.mkdir(str(self.__path_xmp))
        self.__path_pen = self.__base_folder.joinpath("Pen")
        if not self.__path_pen.exists():
            os.mkdir(str(self.__path_pen))
        self.__palette=palette
        self.__container_pen=TempContainerPen(str(self.__path_pen),start_number_pen)
        self.__container_xpm = TempContainerPen(str(self.__path_xmp),start_number_brush,"BRUSH")
        self.__brush_style=BuildBrushStyle(self.__path_xmp,self.__palette)

    def build(self,obj_cls):
        if isinstance(obj_cls,PanDrawSymbol):
            return self.__buildPanDrawSymbol(obj_cls)
        return None
    def __buildPanDrawSymbol(self,obj_cls):
        jkl=0
        mi_style_brush,color=self.__brush_style.build(obj_cls)
        if mi_style_brush is None:
            print("Not brush")
        if mi_style_brush is not None and not isinstance(mi_style_brush,str):
            ''' bytes xpm'''
            index, curent_name_pen=self.__container_xpm.add(mi_style_brush)
            color_mi= color.rgb()-4278190080
            mi_style_brush="Brush("+str(curent_name_pen)+","+str(color_mi)+")"
        axi_pen=convertPanDrawSymbolToAxiPen(obj_cls,self.__palette)
        if len(axi_pen.getMainPaterns())==0:
            if mi_style_brush is not None:
                return mi_style_brush
            return None
        mi_style=None
        if axi_pen is not None:

            useStyleColor,color_pen=axi_pen.setUseStyleColor()

            index, curent_name_pen = self.__container_pen.add(axi_pen)
            if useStyleColor:
                color_int=color_pen.rgb()-4278190080
                mi_style="Pen("+str(curent_name_pen)+",1,"+str(color_int)+")"
            else:
                mi_style="Pen("+str(curent_name_pen)+",1,0)"
        if mi_style_brush is not None and mi_style is not None:
            return mi_style+" "+mi_style_brush
        return mi_style

