import math

from PySide2.QtCore import QPoint
from PySide2.QtGui import QImage, Qt, QPen, QPainter, QColor

from PanoramaRsc.ui.add_tools.rsc_tools.RSC.image_cls.imageMask32.helper import imageToBytes
from PanoramaRsc.ui.add_tools.rsc_tools.RSC.panSymbol import PanLine, SquareVector, ImgChainType, ImgParamType
from PanoramaRsc.ui.add_tools.rsc_tools.panoramaToAxiPen.factoryAxiPen import  getColor

def calcSizeRasterFillHatch(step_mm,angle:int):

    if angle % 90 ==0:
        return step_mm, step_mm

    angle_rad=math.radians(angle)
    h=math.cos(angle_rad)*step_mm
    width=h/math.tan(angle_rad)
    return width,h
class MmtoPix:
    def __init__(self,dpi=96):
        self.__dpi=dpi
        self.__mm_in_pix=25.4/dpi
    def toPx(self,value_mm):
        value_pix=int(value_mm/self.__mm_in_pix+0.5)
        return value_pix


def createImage(width_px,height_px,dpi=96):

    image=QImage(width_px,height_px, QImage.Format_ARGB32_Premultiplied)
    image.fill(Qt.transparent)
    return image
def drawLineToRaster(image,width_px,angle,color):
    pen=QPen(color,width_px)
    width_img=image.width()
    height_img=image.height()
    paint = QPainter(image)
    paint.setPen(pen)
    if angle==0:

        x1=0
        y1=int(height_img/2)
        x2=width_img
        y2=y1

    if angle==90:
        xc=int(width_img/2)

        x1 = int(width_img/2)
        y1 = 0
        x2 = x1
        y2 = height_img
    if angle>0 and angle<90:
        x1=0
        y1=height_img
        x2=width_img
        y2=0
    if angle>90:
        x1 = 0
        y1 = 0
        x2 = width_img
        y2 = height_img
    paint.drawLine(x1,y1,x2,y2)
    paint.end()
    return





def buildXpmFillHatch(pan_obj,palette):
    cls_MmtoPix=MmtoPix()
    angle=pan_obj.AngleIsDegrees

    step=pan_obj.StepInMicrones/1000
    #step_in_px=cls_MmtoPix.toPx(step)
    width_mm,height_mm=calcSizeRasterFillHatch(step,angle)
    width_px=cls_MmtoPix.toPx(abs(width_mm))
    height_px=cls_MmtoPix.toPx(abs(height_mm))
    if width_px % 2 ==0:
        width_px=width_px+1
    if height_px % 2==0:
        height_px=height_px+1
    image=createImage(width_px,height_px)
    colors=[]
    for sub_sym in pan_obj.SymbolLineParameters:
        if isinstance(sub_sym,PanLine):
            ''' Рисуем линию в растр'''
            width_pen_px=cls_MmtoPix.toPx(sub_sym.WidthInMicrones/1000)
            color_brush=getColor(sub_sym.ColorRGBA,palette)
            colors.append(color_brush)
            color=QColor(0,0,0)
            drawLineToRaster(image,width_pen_px,angle,color)
            #image.save(r"E:\Download\Panorama\terplan7\temp\test_p1.png","PNG")
            jkl=0
    data_image = imageToBytes(image, "XPM")
    data_image = data_image.replace(b"#ffffff", b'None')
    out_color=None
    if len(colors)==1:
        out_color=colors[0]
    return data_image,out_color
def buildXpmHatchSquareShift(pan_obj,palette):
    cls_MmtoPix=MmtoPix()
    angle=pan_obj.Angle

    step=pan_obj.Step/1000
    #step_in_px=cls_MmtoPix.toPx(step)
    width_mm,height_mm=calcSizeRasterFillHatch(step,angle)
    image=createImage(cls_MmtoPix.toPx(abs(width_mm)),cls_MmtoPix.toPx(abs(height_mm)))

    color_brush=None
    if isinstance(pan_obj.Line,PanLine):
        ''' Рисуем линию в растр'''
        sub_sym=pan_obj.Line
        width_pen_px=cls_MmtoPix.toPx(sub_sym.WidthInMicrones/1000)
        color_brush=getColor(sub_sym.ColorRGBA,palette)

        color=QColor(0,0,0)
        drawLineToRaster(image,width_pen_px,angle,color)
            #image.save(r"E:\Download\Panorama\terplan7\temp\test_p1.png","PNG")

    data_image = imageToBytes(image, "XPM")
    data_image = data_image.replace(b"#ffffff", b'None')


    return data_image,color_brush
def calcSizeSquareVector(step_x_mm,step_y_mm,type):
    cls_px=MmtoPix()
    step_x_px=cls_px.toPx(step_x_mm/1000)
    step_y_px=cls_px.toPx(step_y_mm/1000)
    if step_x_px % 2 == 0:
        step_x_px = step_x_px + 1
    if step_y_px % 2 == 0:
        step_y_px = step_y_px +1
    if type==0 or type==-1:
        width_px=step_x_px
        height_px=step_y_px
    else:

        width_px = 2*step_x_px
        height_px = 2*step_y_px


    return width_px, height_px,step_x_px,step_y_px
def drawArea(image,color,type,size_obj,step_x,step_y):
    paint = QPainter(image)
    paint.setBrush(color)
    if type == 42405:
        paint.drawRect(0,0,step_x-1,step_y-1)
        paint.drawRect(step_x, step_y, step_x-1, step_y-1)
    else:
        paint.drawRect(step_x, 0, step_x-1, step_y-1)
        paint.drawRect(0, step_y, step_x-1, step_y-1)

def drawCircle(image,color,type,size_obj,step_x,step_y):
    paint = QPainter(image)
    paint.setBrush(color)
    xc = int(step_x / 2)
    yc = int(step_y / 2)
    if size_obj>int(step_x/2):
        size_obj=size_obj-1
    if type==0 or type==-1:
        xc=int(step_x/2)
        yc=int(step_y/2)
        paint.drawEllipse(QPoint(xc,yc),size_obj,size_obj)
        return
    if type==42405:
        xc = int(step_x / 2)
        yc = int(step_y / 2)
        paint.drawEllipse(QPoint(xc, yc), size_obj, size_obj)
        xc=xc+step_x
        yc=yc+step_y
        paint.drawEllipse(QPoint(xc, yc), size_obj, size_obj)
        return
    xc = int(step_x / 2)
    yc = int(step_y / 2)+step_y
    paint.drawEllipse(QPoint(xc, yc), size_obj, size_obj)
    xc=xc+step_x
    yc=int(step_y / 2)
    paint.drawEllipse(QPoint(xc, yc), size_obj, size_obj)
    return




def drawSymbolToImage(image,sym_frg,type,color,size_obj,step_x,step_y):
    color_draw=color
    if color_draw is not None:
        color_draw=QColor(0,0,0)
    if sym_frg.ChainType==ImgChainType.Circle and sym_frg.ParamType==ImgParamType.Area:
        drawCircle(image, color_draw, type, size_obj, step_x, step_y)
        return
    if sym_frg.ChainType==ImgChainType.Area and sym_frg.ParamType==ImgParamType.Area:
        drawArea(image, color_draw, type, size_obj, step_x, step_y)
        return






def factorySquareVector(sym:SquareVector,palette):
    width,height,step_x,step_y=calcSizeSquareVector(sym.SymbolStepByXinMicrone,sym.SymbolStepByYinMicrone,sym.FillGridMask)
    colors=[]
    for frag in sym.SymbolSymbol.Fragments:
        colors.append(getColor(frag.ParamSymbol.ColorRGBA,palette))
    out_color=None
    if len(colors)==1:
        out_color=colors[0]
    image=createImage(width,height)
    size_object=sym.SymbolSymbol.Border
    cls_px = MmtoPix()
    size_obj_px=cls_px.toPx(size_object/1000)

    for frag in sym.SymbolSymbol.Fragments:
        drawSymbolToImage(image,frag,sym.FillGridMask,out_color,size_obj_px,step_x,step_y)
    data_image = imageToBytes(image, "XPM")
    data_image = data_image.replace(b"#ffffff", b'None')
    return data_image,out_color


