from pathlib import Path

import axipy
from PySide2.QtCore import QObject, Signal
from axipy import OpenMode, Schema, provider_manager, Feature
from loguru import logger

from PanoramaRsc.ui.add_tools.helper_features.clone_feature import TableItemsReprojectIterator
from PanoramaRsc.ui.add_tools.importGmlFgis import CalcIntProcent


def copySchema(schema_source:Schema,new_cs):
    attributes=schema_source.copy()
    new_schema=Schema(attributes,new_cs)

    return new_schema

class ExportToPanorama(QObject):
    change_name_tab=Signal(str)
    change_id_feature=Signal(int)
    def __init__(self):
        super().__init__()
        self.__isCancel=False
        self.procentCounter=None
        self.__layers_processing=[]
    def __process_features(self,feature: Feature, row: int):
        #logger.debug("process "+str(row))
        if self.__isCancel:
            return False
        val_proc=self.procentCounter.getValue(row)
        if val_proc is not None:
            self.change_id_feature.emit(val_proc)
        return True
    def setCancel(self,value):
        self.__isCancel=value
    def run(self,params_source,params_out):
        logger.debug("Start ExportToPanorama")
        logger.debug(params_source)
        logger.debug(params_out)
        path_db=params_source['file_gpkg']
        list_name_table=params_source['tables']
        path_out=params_out['out_file']
        rsc=params_out['rsc_file']
        cs_out=params_out['coordsystem']
        name_object_key=params_out['objectkey']
        if name_object_key is None:
            name_object_key="ObjectKey"
        open_mode=OpenMode.Create
        self.__layers_processing.clear()
        is_openerTab=True
        for layer_info in list_name_table:
            is_openerTab=False
            name=layer_info.name
            type_geometry=layer_info.typeGeometry
            logger.debug(name)
            if self.__isCancel:
                break
            try:
                tab_curent=axipy.data_manager.find(name)
                if tab_curent is not None:
                    ''' Проверяем таблица из тогоже источника'''
                    path_tab=tab_curent.properties['fileName']
                    if Path(path_tab)==Path(path_db):
                        is_openerTab=True
                    else:
                        tab_curent=None
                if tab_curent is None:
                    def_tab_db = {'src': path_db,
                                  'dataobject': name,
                                  'provider': 'SqliteDataProvider'
                                  }
                    tab_curent=axipy.provider_manager.open_hidden(def_tab_db)
                logger.debug("Open table:"+tab_curent.name)
                schema_out=tab_curent.schema
                isChangeCoordSystem=False
                if cs_out is not None and tab_curent.schema.coordsystem!=cs_out:
                    schema_out=copySchema(schema_out,cs_out)
                    isChangeCoordSystem=True
                destination = provider_manager.panorama.get_destination(path_out, rsc, schema_out, name_object_key,open_mode=open_mode)
                out_features=None
                '''
                if isChangeCoordSystem:
                    out_features=TableItemsReprojectIterator(tab_curent.items(),cs_out)
                else:
                    out_features =tab_curent.items()
                '''
                if isChangeCoordSystem:
                    out_features = TableItemsReprojectIterator(tab_curent.items(), cs_out,type_geometry)
                else:
                    out_features = TableItemsReprojectIterator(tab_curent.items(), None, type_geometry)
                self.procentCounter=CalcIntProcent(tab_curent.count())
                destination.export(out_features,func_callback=self.__process_features)
                self.__layers_processing.append(name)
                if not is_openerTab:
                    tab_curent.close()
                open_mode = OpenMode.Append
            except   Exception as ex:
                logger.exception("Error ExportToPanorama")
                print(ex)
                try:
                    tab_curent.close()
                except:
                    pass
                break
            if self.__isCancel:
                break
        if not self.__isCancel:
            self.change_id_feature.emit(100)
        logger.debug("end ExportToPanorama")
    @property
    def CountProcessingLayers(self):
        return len(self.__layers_processing)




