import os
from pathlib import Path

import axipy
from PySide2.QtCore import QObject, Signal
from axipy import Schema, Feature
from loguru import logger
from osgeo import ogr, osr
from osgeo.ogr import FieldDefn, Geometry

from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.helper_csv import saveErrorJornal
from PanoramaRsc.ui.add_tools.importGmlFgis import CalcIntProcent


def AxiSchemaToOgrFieldDefns(schema:Schema):
    fild_defs=[]
    for att in schema:
        if att.type_string=='int' or att.type_string=='short':
            fild_defs.append(FieldDefn(att.name.upper(),ogr.OFTInteger))
            continue
        if att.type_string=='large':
            fild_defs.append(FieldDefn(att.name.upper(),ogr.OFTInteger64))
            continue
        if att.type_string=='string':
            if att.name.lower()=='classid':
                fild_defs.append(FieldDefn(att.name.upper(), ogr.OFTInteger64))
                continue
            ft_def_str=FieldDefn(att.name.upper(),ogr.OFTString)
            ft_def_str.SetWidth(att.length)
            fild_defs.append(ft_def_str)
            continue
        if att.type_string == 'double' or att.type_string == 'float':
            fild_defs.append(FieldDefn(att.name.upper(), ogr.OFTReal))
            continue
        if att.type_string == 'datetime':
            fild_defs.append(FieldDefn(att.name.upper(), ogr.OFTDateTime ))
            continue
        if att.type_string == 'date':
            fild_defs.append(FieldDefn(att.name.upper(), ogr.OFTDate ))
            continue
    return fild_defs
def createLayer(ds_out,table_source,cs_out):
    att_def=AxiSchemaToOgrFieldDefns(table_source.schema)
    ogr_cs= osr.SpatialReference()
    ogr_cs.ImportFromEPSG(cs_out.epsg)
    layer_dest = ds_out.CreateLayer(table_source.name, ogr_cs, geom_type=ogr.wkbUnknown)
    for fild_def  in att_def:
        layer_dest.CreateField(fild_def)
    return layer_dest
def copyFtAxiToOgr(ft:Feature,att_names,layer_out,cs_out)->ogr.Feature:
    out_geo=ft.geometry
    if cs_out is not None:
        out_geo=ft.geometry.reproject(cs_out)
    feature_out = ogr.Feature(layer_out.GetLayerDefn())
    ogr_geomtry=None
    geo_wkb=out_geo.to_wkb()
    if geo_wkb is not None:
        feature_out.SetGeometry( ogr.CreateGeometryFromWkb(geo_wkb))
    for att_name in att_names:
        value=ft[att_name]
        if att_name.lower()=='classid' and isinstance(value,str):
            value=int(value)
        feature_out.SetField(att_name.upper(),value)
    return feature_out

class ValidationClassid:
    def __init__(self,atributes,dict_calssid):
        self.__name_classid=None
        if 'classid' in atributes:
            self.__name_classid = 'classid'
        if 'CLASSID' in atributes:
            self.__name_classid = 'CLASSID'
        self.__dict_class=dict_calssid
    def isValid(self,ft):
        if self.__name_classid is None:
            return True
        value_classid=ft[self.__name_classid]
        if isinstance(value_classid,int):
            value_classid=str(value_classid)
        if value_classid in self.__dict_class:
            return True
        return False
    @property
    def isReadyValidate(self):
        if self.__name_classid is None:
            return False
        return True

class FactoryGmlLayerFGISTPNoXSD(QObject):
    next_feature=Signal(int)
    def __init__(self,table_axi,name_fgisttp,cs_out,dict_classid,path_out):
        super().__init__()
        self.__isValidStruct=False
        self.__name_fgisttp=name_fgisttp
        self.__table_source=table_axi
        self.__name_fgisttp=name_fgisttp

        self.__isCancel=False
        self.__FeaturePropertyTypes=None
        self.__messageErrors=None
        self.__cs_out=cs_out
        self.__dict_classid=dict_classid

        self.__path_out=Path(path_out)
        self.__cls_validator=None
        if self.__dict_classid is not None:
            self.__cls_validator=ValidationClassid(self.__table_source.schema.attribute_names,self.__dict_classid)
            if not self.__cls_validator.isReadyValidate:
                self.__cls_validator=None
    def run(self):
        coordsys_source=self.__table_source.coordsystem
        isReproject=False
        if coordsys_source!=self.__cs_out:
            isReproject=True
        path_out_gml=self.__path_out.joinpath(self.__table_source.name+".gml")
        if path_out_gml.exists():
            os.remove(str(path_out_gml))
        ds_out=ogr.GetDriverByName("GML").CreateDataSource(str(path_out_gml))
        if ds_out is None:
            logger.error("Error create GML")
            return
        layer=createLayer(ds_out,self.__table_source,self.__cs_out)
        cs_out_temp=None
        if isReproject:
            cs_out_temp=self.__cs_out
        cls_proc=CalcIntProcent(self.__table_source.count())
        '''
        name_classid=None
        if 'classid' in self.__table_source.schema.attribute_names:
            name_classid = 'classid'
        if 'CLASSID' in self.__table_source.schema.attribute_names:
            name_classid = 'CLASSID'
        '''
        self.__messageErrors = {"RowID": [], "Status": []}
        for i,ft in enumerate(self.__table_source.items()):
            proc=cls_proc.getValue(i+1)
            if proc is not None:
                self.next_feature.emit(proc)
            if self.__isCancel:
                break
            if self.__cls_validator is not None:
                isValid=self.__cls_validator.isValid(ft)
                if not isValid:
                    self.__messageErrors["RowID"].append(i)
                    self.__messageErrors["Status"].append("error : not valid CLASSID")

            ogr_ft=copyFtAxiToOgr(ft,self.__table_source.schema.attribute_names,layer,cs_out_temp)
            if ogr_ft is not None:
                layer.CreateFeature(ogr_ft)
                ogr_ft=None
        ds_out.FlushCache()
        ds_out.Destroy()
        ds_out = None
    @property
    def messageErrors(self):
        return self.__messageErrors





class FactoryGmlFGISTPNoXsd(QObject):
    next_layer=Signal(str,int)
    change_procent_feature=Signal(int)
    def __init__(self,list_source_tables,out_cs,path_out_gml,separate_file=True):
        super().__init__()
        self.__tables=list_source_tables
        self.__out_cs=out_cs
        if isinstance(path_out_gml,str):
            self.__path_out=Path(path_out_gml)
        else:
            self.__path_out=path_out_gml
        self.__separate_file=separate_file
        self.__isCancel=False
        self.__cls_xml=None
    def run(self):
        count_tables = len(self.__tables)
        cls_procent = CalcIntProcent(count_tables)
        for i, table_des in enumerate(self.__tables):
            name_table = table_des['name']
            alias_gistp = table_des['alias']
            if alias_gistp is None:
                alias_gistp='None'
            dic_classid=table_des["dict"]
            logger.debug("Export table:" + name_table)
            procent = cls_procent.getValue(i+1)
            if procent is not None:
                self.next_layer.emit(name_table + "(" + alias_gistp + ")", procent)
            if self.__isCancel:
                break
            tab = axipy.data_manager.find(name_table)
            cls_run = FactoryGmlLayerFGISTPNoXSD(tab, alias_gistp, tab.coordsystem,dic_classid,self.__path_out)
            cls_run.run()
            err_jormal = cls_run.messageErrors
            if len(err_jormal["RowID"]) > 0:
                path_jornal = self.__path_out.joinpath(name_table + ".log")
                saveErrorJornal(str(path_jornal), err_jormal)
        self.next_layer.emit(name_table + "(" + alias_gistp + ")", 100)

