import io
from pathlib import Path

import axipy
from PySide2.QtCore import QObject, Signal
from loguru import logger
from osgeo import ogr, osr

from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.geometryOgrFactory import transformAxiGeometryToOgr, \
    OgrGeometryToGeometryXml
from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.helper_csv import saveErrorJornal
from PanoramaRsc.ui.add_tools.fgis_tp.fgis_tp_10_izm import *
from PanoramaRsc.ui.add_tools.importGmlFgis import CalcIntProcent

def saveGml(path_out_gml,list_featureMembers):
    if Path(path_out_gml).exists():
        os.remove(path_out_gml)
    cls_root = FeatureCollectionType()
    cls_root.original_tagname_ = "FeatureCollection"
    cls_root.featureMember = list_featureMembers
    out_gml_io = io.StringIO()
    cls_root.export(out_gml_io, 0)
    str_out_gml = out_gml_io.getvalue()
    out_gml_io.close()
    str_out_gml = str_out_gml.replace('owns="None"', '')
    new_gml_FeatureCollection = '<gml:FeatureCollection xmlns="http://fgistp" xsi:schemaLocation="http://fgistp https://fgistp.economy.gov.ru/panorama/schemas/xsddownload/fgistp-10-izm-698.xsd" xmlns:gml="http://www.opengis.net/gml/3.2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" gml:id="aFeatureCollection"'
    str_out_gml = str_out_gml.replace('<gml:FeatureCollection', new_gml_FeatureCollection)

    str_out_gml = '<?xml version="1.0" encoding="utf-8"?>' + "\n" + str_out_gml
    with open(path_out_gml, 'w', encoding='utf-8') as f:
        f.write(str_out_gml)

def validateCLASSID( cls,name_layer):
    name_function="validate_"+name_layer+"_CLASSID"
    value_classid=getattr(cls,'CLASSID')
    isValid=cls.__getattribute__(name_function)(value_classid)
    return isValid
def validateUUID( cls,name_layer):
    name_function="validate_UUID"
    value_GLOBALID = getattr(cls, 'GLOBALID')
    isValid = cls.__getattribute__(name_function)(value_GLOBALID)
    return isValid
def variables_cls(cls_type):
    if cls_type is None:
        return None
    cls=cls_type()
    variable=vars(cls)
    list_variable=[]
    geometry_variable=None
    for name in list(variable.keys()):
        if name.find("gds")==0:
            continue
        if name.find("_nsprefix")>=0:
            continue
        if name[0].islower():
            continue
        if name.find("Multi")==0:
            geometry_variable=name
            continue
        list_variable.append(name)

    del cls
    return list_variable,geometry_variable
def factoryXmlClass(name_layer:str,out_cs):
    try:
        cls_type = GDSClassesMapping[name_layer]
    except:
        logger.error("Not find class for layer:"+name_layer)
        return None
    variables,geometry_type=variables_cls(cls_type)
    return FactoryFeatureMemberTeg(name_layer,cls_type,variables,geometry_type,out_cs)
class FactoryFeatureMemberTeg:
    def __init__(self,name_cls,type_cls,variables,name_out_geometry,out_cs):
        self.__name_cls=name_cls
        self.__type_cls=type_cls
        self.__variables=variables
        self.__out_cs=out_cs
        self.__name_out_geometry=name_out_geometry
        self.__ogr_out_type_geometry=self.__getWkbTypeByName(self.__name_out_geometry)
    def __getWkbTypeByName(self,name_geometry):
        low_name=name_geometry.lower()
        if low_name=="multicurve":
            return ogr.wkbMultiCurve
        if low_name=="multisurface":
            return ogr.wkbMultiSurface
        if low_name=="multisurface":
            return ogr.wkbMultiSurface
        if low_name=="multipoint":
            return ogr.wkbMultiPoint
        logger.error("Name geometry out field:"+name_geometry+" not define type")
        return ogr.wkbUnknown
    def setCurentTable(self,table):
        self.__source_table_axi=table
        self.__source_iterator=self.__source_table_axi.items()
        self.__id=0
        self.__fields_alias,isValidStruct=self.__build_alias_fields()
        return isValidStruct
    def getNeedFields(self):
        return self.__variables
    def __build_alias_fields(self):
        schema_tab = self.__source_table_axi.schema
        table_alias = []
        for att in schema_tab:
            name = att.name
            if name.lower() in self.__variables or name.upper() in self.__variables:
                table_alias.append({"out": name.upper(), "source": name})
        isValidNameField=False
        if len(self.__variables)==len(table_alias):
            isValidNameField=True
        return table_alias,isValidNameField
    def __iter__(self):
        return self

    def __next__(self):
        cls_obj_teg=self.__create_cls()
        ft_curent=next(self.__source_iterator)
        for def_table in self.__fields_alias:
            value=ft_curent[def_table["source"]]
            if def_table['out']=='CLASSID':
                if isinstance(value,str):
                    try:
                        value = int(value)
                    except:
                        logger.error("Error CLASSID value not define. Set CLASSID=0")
                        value=0

            if isinstance(value, str) and self.__source_table_axi.schema[def_table["source"]].type_string != 'string':
                    continue
            if isinstance(value, str) and value=='':
                continue
            setattr(cls_obj_teg,def_table['out'],value)
        if ft_curent.geometry is not None :
            if self.__source_table_axi.coordsystem!=self.__out_cs:
                geo_ogr=transformAxiGeometryToOgr(ft_curent.geometry.reproject(self.__out_cs),self.__ogr_out_type_geometry)
            else:
                geo_ogr = transformAxiGeometryToOgr(ft_curent.geometry,self.__ogr_out_type_geometry)
            if geo_ogr is not None:
                srs = osr.SpatialReference()
                srs.ImportFromEPSG(self.__out_cs.epsg)
                geo_ogr.AssignSpatialReference(srs)
                obj_xml_spatial=OgrGeometryToGeometryXml(geo_ogr)
                if obj_xml_spatial is not None:
                    setattr(cls_obj_teg,self.__name_out_geometry,obj_xml_spatial)
        return cls_obj_teg


    def __create_cls(self):
        value_id=self.__name_cls+"."+str(self.__id)
        self.__id=self.__id+1
        cls_obj=self.__type_cls(value_id)
        cls_obj.id=value_id
        logger.debug("Create cls :"+str(cls_obj)+" id="+value_id)
        cls_obj.original_tagname_ = self.__name_cls
        cls_obj.gds_collector_ = GdsCollector_()
        return cls_obj

class FactoryGmlLayerFGISTP(QObject):
    next_feature=Signal(int)
    def __init__(self,table_axi,name_fgisttp,cs_out):
        super().__init__()
        self.__isValidStruct=False
        self.__name_fgisttp=name_fgisttp
        self.__table_source=table_axi
        self.__name_fgisttp=name_fgisttp

        self.__isCancel=False
        self.__FeaturePropertyTypes=None
        self.__messageErrors=None
        self.__cs_out=cs_out
    @property
    def ValidStructTable(self):
        return self.__isValidStruct

    def run(self):
        self.__cls_fetureMember = factoryXmlClass(self.__name_fgisttp, self.__cs_out)
        self.__isValidStruct = self.__cls_fetureMember.setCurentTable(self.__table_source)
        count_ft=self.__table_source.count()

        cls_procent=CalcIntProcent(count_ft)
        self.__messageErrors={"RowID":[],"Status":[]}
        self.__FeaturePropertyTypes=[]
        for i,ft_out in enumerate(self.__cls_fetureMember):
            logger.debug("ft id:"+ft_out.id)
            proc_val=cls_procent.getValue(i)
            if proc_val is not None:
                self.next_feature.emit(proc_val)
            if self.__isCancel:
                break
            if ft_out is None:
                self.__messageErrors["RowID"].append(i)
                self.__messageErrors["Status"].append("error transform geometry")
                #self.__messageErrors.append({"record":i,"error":"transform geometry"})
            else:

                is_valid_cls_id=validateCLASSID(ft_out,self.__name_fgisttp)
                if not is_valid_cls_id:
                    self.__messageErrors["RowID"].append(i)
                    self.__messageErrors["Status"].append("error : not valid CLASSID")
                    #self.__messageErrors.append({"record": i, "error": "not valid CLASSID"})
                is_valid_clobalid=validateUUID(ft_out,self.__name_fgisttp)
                if not is_valid_clobalid:
                    #self.__messageErrors.append({"record": i, "error": "not valid GLOBALID"})
                    self.__messageErrors["RowID"].append(i)
                    self.__messageErrors["Status"].append("error : not valid GLOBALID")
                self.__FeaturePropertyTypes.append(ft_out)
        self.next_feature.emit(100)

    @property
    def FeaturePropertyTypes(self):
        return self.__FeaturePropertyTypes
    @property
    def messageErrors(self):
        return self.__messageErrors
    def setCancel(self):
        self.__isCancel=True

class FactoryGmlFGISTP(QObject):
    next_layer=Signal(str,int)
    change_procent_feature=Signal(int)
    def __init__(self,list_source_tables,out_cs,path_out_gml,separate_file=False):
        super().__init__()
        self.__tables=list_source_tables
        self.__out_cs=out_cs
        self.__path_out=Path(path_out_gml)
        self.__separate_file=separate_file
        self.__isCancel=False
        self.__cls_xml=None
    def run(self):
        count_tables=len(self.__tables)
        cls_procent = CalcIntProcent(count_tables)
        collections_featureMembers=[]
        for i,table_des in enumerate(self.__tables):

            name_table=table_des['name']
            alias_gistp=table_des['alias']
            logger.debug("Export table:" +name_table)
            procent=cls_procent.getValue(i)
            if procent is not None:
                self.next_layer.emit(name_table+"("+alias_gistp+")",procent)
            if self.__isCancel:
                break
            tab=axipy.data_manager.find(name_table)
            self.__cls_xml = FactoryGmlLayerFGISTP(tab, alias_gistp,self.__out_cs)
            self.__cls_xml.next_feature.connect(self.__change_progess)
            self.__cls_xml.run()
            featureMembers= self.__cls_xml.FeaturePropertyTypes
            err_jormal=self.__cls_xml.messageErrors
            if len(err_jormal["RowID"])>0:
                path_jornal=self.__path_out.parent.joinpath(name_table+".log")
                saveErrorJornal(str(path_jornal),err_jormal)
            self.__cls_xml.next_feature.disconnect(self.__change_progess)

            if self.__separate_file:
                path_out_gml=self.__path_out.parent.joinpath(name_table+".gml")
                saveGml(str(path_out_gml),featureMembers)
            else:
                collections_featureMembers.extend(featureMembers)
        if not self.__separate_file:
            if len(collections_featureMembers)>0:
                saveGml(str(self.__path_out), collections_featureMembers)
        self.next_layer.emit(name_table + "(" + alias_gistp + ")", 100)






    def __change_progess(self,procent):
        self.change_procent_feature.emit(procent)
    def setCancel(self):
        self.__isCancel=True
        self.__cls_xml.setCancel()






