from osgeo import ogr

from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.helper_db.db_add_helper import TypeGeometry


def getTypeGeometry(val_int):
    if val_int==0:
        return TypeGeometry.PolyLine
    if val_int==1:
        return TypeGeometry.Polygon
    if val_int==2:
        return TypeGeometry.Point
    if val_int==3:
        return TypeGeometry.Text
    if val_int==4:
        return TypeGeometry.Vector
    if val_int==5:
        return TypeGeometry.Label
    if val_int==6:
        return TypeGeometry.Any
    return TypeGeometry.Any
class FieldDesc:
    def __init__(self,values):
        self.code=values[0]
        self.type=values[1]
        self.allowrepeate = values[3]
        self.is_internal=values[3]
        self.name=values[3]
        self.shortName=values[5]
        self.units=values[6]
        self.length=values[7]
        self.precisssion=values[8]
        self.flag=values[9]
    def createTxt(self):
        out_str='['+self.shortName.lower()+'] '
        type=self.__typeToString()
        return out_str+type
    def getOgrFieldDefn(self):
        if self.type==1:
            if self.precisssion == 0:
                return ogr.FieldDefn(self.shortName.lower(), ogr.OFTInteger)
            else:
                return ogr.FieldDefn(self.shortName.lower(), ogr.OFTReal)
        if self.type == 22 or self.type == 24:
            ogr_def=ogr.FieldDefn(self.shortName.lower(), ogr.OFTString)
            ogr_def.SetWidth(40)
            return ogr.FieldDefn(self.shortName.lower(), ogr.OFTString)
        if self.type == 0:
            ogr_def=ogr.FieldDefn(self.shortName.lower(), ogr.OFTString)
            ogr_def.SetWidth(self.length)
            return ogr.FieldDefn(self.shortName.lower(), ogr.OFTString)
        if self.type == 16:
            return ogr.FieldDefn(self.shortName.lower(), ogr.OFTInteger)
    def __typeToString(self):
        if self.type==1:
            ''' Число '''
            if self.precisssion==0:
                return "INTEGER"
            else:
                return "DOUBLE"
        if self.type==22:
            return "TEXT (40)"
        if self.type==24:
            return "TEXT (40)"
        if self.type==0:
            return "TEXT ("+str(self.length)+")"
        if self.type == 16:
            return "INTEGER"
        return None
class LayerInfo:
    def __init__(self,id_layer,name,short_name,order_index,full_name,type_geometru):
        self.idLayer=id_layer
        self.name=name
        self.shortName=short_name
        self.order_index=order_index
        self.fullName=full_name
        self.typeGeometry=type_geometru
class DbTools:
    __tab_layers='layers'
    __tab_full_names = 'full_names_layers'
    __table_link_key="foreign_key_layers"
    __table_spatial_type="layer_info_dict_spatial"
    __ObjectId_code=32800
    def __init__(self,base_db,dict_db=None):
        self.__initDb(base_db,dict_db)
        self.__name_dict_db="db_dict"
    def __initDb(self,base_db,dict_db=None):
        self.__db = DbRsc(base_db)
        self.__db.connect()
        if dict_db is not None:
            sql_atach = 'ATTACH DATABASE "' + dict_db + '" AS db_dict'
            self.__db.runSqlNoResult(sql_atach)
    def getLayers(self,db_name="db_dict")->list:
        name_tab=db_name+"."+self.__tab_layers
        name_full_names=db_name+"."+self.__tab_full_names
        name_full_spatial_type=db_name+"."+self.__table_spatial_type
        sql="select lyr.*,full.name as full_name,sp.type_geometry from "+name_tab+" as lyr,"+name_full_names+" as full,"+name_full_spatial_type+" as sp where lyr.id_layer==full.code_object and  lyr.id_layer==sp.id_layer"
        rows=self.__db.select(sql)
        list_layer_info=[]
        for row in rows:
            if row[2]=="SYSTEM":
                continue
            list_layer_info.append(LayerInfo(row[0],row[1],row[2],row[3],row[4],getTypeGeometry(row[5])))
        return list_layer_info
    def getLayer(self,id_layer):
        name_tab = self.__name_dict_db + "." + self.__tab_layers
        name_full_names = self.__name_dict_db + "." + self.__tab_full_names
        sql = "select lyr.*,full.name from " + name_tab + " as lyr," + name_full_names + " as full where lyr.id_layer="+str(id_layer)+ " and lyr.id_layer==full.code_object "
        rows = self.__db.select(sql)
        row=rows[0]
        return LayerInfo(row[0],row[1],row[2],row[3],row[4])
    def getStructLayer(self,id_layer):
        sql="select sem.* from "+self.__name_dict_db+".layers_attribute lyr,"+self.__name_dict_db+".semantic_attribute sem where lyr.num_layer="+str(id_layer)+" and lyr.code_att=sem.code"
        rows=self.__db.select(sql)
        fields_info=[]
        field_objectId=None
        for i,row in enumerate(rows):
            field_des=FieldDesc(row)
            if field_des.code == self.__ObjectId_code:
                field_objectId=field_des
            else:
                fields_info.append(field_des)
        if field_objectId is not None:
            fields_info.insert(0,field_objectId)
        return fields_info
    def createTable(self,id_layer,short_name=True):
        layer_info=self.getLayer(id_layer)
        field_layers_desc=self.getStructLayer(id_layer)
        sql="CREATE TABLE ["+layer_info.shortName+"] ("
        sql_field_object_id=None
        str_fields=[]
        for field_struct in field_layers_desc:
            if field_struct.code==self.__ObjectId_code:
                sql_field_object_id=field_struct.createTxt()
            else:
                str_fields.append(field_struct.createTxt())
        sql=sql+sql_field_object_id+","+",".join(str_fields)+")"
        self.__db.runSqlNoResult(sql)

    def copyTable(self,name_table):
        name_source=self.__name_dict_db+"."+name_table
        sql="CREATE TABLE ["+name_table+"]  AS   select * from "+name_source
        self.__db.runSqlNoResult(sql)
    def copyDictionaryForLayer(self,id_layer):
        sql="Select * from "+self.__table_link_key+" where id_layer="+str(id_layer)
        rows=self.__db.select(sql)
        for row in rows:
            name_dictionary=row[1]
            self.copyTable(name_dictionary)
        return
    def copyLayerExtTable(self,name_table):
        db_name=self.__name_dict_db
        name_tab = db_name + "." + self.__tab_layers
        name_full_names = db_name + "." + self.__tab_full_names
        name_full_spatial_type = db_name + "." + self.__table_spatial_type
        sql_join = "select lyr.*,full.name as full_name,sp.type_geometry from " + name_tab + " as lyr," + name_full_names + " as full," + name_full_spatial_type + " as sp where lyr.id_layer==full.code_object and  lyr.id_layer==sp.id_layer"
        sql = "CREATE TABLE [" + name_table + "]  AS   "+sql_join
        self.__db.runSqlNoResult(sql)

    def close(self,isCommit=True):
        self.__db.close(isCommit)
