from enum import Enum
from pathlib import Path

import axipy
from loguru import logger
from osgeo import ogr

from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.helper_zip.add_helper import generate_zip, decompressZipToFile

name_tab_layer="layers"
name_tab_full_name_layers="full_names_layers"
name_tab_objects="objects"
name_tab_layers_attribute='layers_attribute'
name_tab_semantic_attribute='semantic_attribute'
class TypeNameLayer(Enum):
    Short=0
    Full=1
class TypeGeometry(Enum):
    PolyLine=0
    Polygon=1
    Point=2
    Text=3
    Vector=4
    Label=5
    Any=6

class ObjectDictionary:
    def __init__(self,code,object_key,name,inner_code,type_geometry:TypeGeometry):
        self.code=code
        self.ObjectKey=object_key
        self.Name=name
        self.InnerCode=inner_code
        self.TypeGeometry=type_geometry
    @staticmethod
    def factoryDictonary(dbRsc,id_layer):
        sql="Select * from "+name_tab_objects+" where layer_number="+str(id_layer)
        result=dbRsc.select(sql)
        objects_dictionary=[]
        temp_type_geometry=[]
        for item_obj in result:
            obj_dict=ObjectDictionary(item_obj[0],item_obj[3],item_obj[4],item_obj[1],item_obj[5])
            if not(obj_dict.TypeGeometry in temp_type_geometry):
                temp_type_geometry.append(obj_dict.TypeGeometry)
            objects_dictionary.append(obj_dict)
        type_geometry=TypeGeometry.Any
        if len(temp_type_geometry)==1:
            type_geometry=temp_type_geometry[0]
        return objects_dictionary,type_geometry

class RscAttribute:
    def __init__(self,name,full_name,inner_code,type_att,length,precission):
        self.Name=name
        self.FullName=full_name
        self.InnerCode=inner_code
        self.Type=type_att
        self.length=length
        self.precission=precission
    def getOgrFieldDefn(self):
        if self.Type==0 or self.Type==24 or self.Type==22:
            ogr_def=ogr.FieldDefn(self.Name, ogr.OFTString)
            ogr_def.SetWidth(self.length)
            return ogr_def
        if (self.Type==1 and self.precission==0) or ( self.precission==0) or (self.Type==16 and self.precission==0):
            ogr_def = ogr.FieldDefn(self.Name, ogr.OFTInteger)
            return ogr_def
        if self.Type==1 and self.precission>0:
            ogr_def = ogr.FieldDefn(self.Name, ogr.OFTReal)
            return ogr_def
        return None
    @staticmethod
    def getOgrFieldDefnMiStyle(name):
        ogr_def = ogr.FieldDefn(name, ogr.OFTString)
        ogr_def.SetWidth(255)
        return ogr_def


    @staticmethod
    def factoryListRscAttribute(dbRsc,id_layer):
        sql="select * from "+name_tab_layers_attribute+" layer,"+name_tab_semantic_attribute+" att where num_layer="+str(id_layer)+" and layer.code_att=att.code "
        result=dbRsc.select(sql)
        attributes=[]
        for item_res in result:
            att=RscAttribute(item_res[7],item_res[6],item_res[1],item_res[3],item_res[9],item_res[10])
            attributes.append(att)
        return attributes



class RscLayer:
    def __init__(self,id,name,full_name,order_layer=0):
        self.__id_layer=id
        self.__short_name=name
        self.__full_name=full_name
        self.__attribute=[]
        self.__typeGeometry=None
        self.__order=order_layer
        self.__dictionaryObject=None
        self.__name=None
    def setDictionaryObjects(self,objs_dict):
        self.__dictionaryObject=objs_dict
    def setTypeGeometry(self,type_geo:TypeGeometry):
        self.__typeGeometry=type_geo
    @property
    def ObjectIdDictionary(self):
        return self.__dictionaryObject
    @property
    def OrderDisplay(self):
        return self.__order
    @property
    def TypeGeometry(self):
        return self.__typeGeometry
    @property
    def Name(self):
        return self.__name
    @property
    def IdLayer(self):
        return self.__id_layer
    @property
    def FullName(self):
        return self.__full_name
    @property
    def ShortName(self):
        return self.__short_name
    def setName(self,name):
        self.__name=name

    def addAttribute(self,att_def):
        self.__attribute=att_def
    @property
    def Attribute(self):
        return self.__attribute
    @staticmethod
    def factory(dbRsc,id_layer):
        sql="select * from "+name_tab_layer+" layer,"+name_tab_full_name_layers+" names where layer.id_layer="+str(id_layer)+" and layer.id_layer=names.code_object"
        result=dbRsc.select(sql)
        if result is None or len(result)==0:
            return None
        info_layer=result[0]
        rscLayer=RscLayer(info_layer[0],info_layer[1],info_layer[5],info_layer[3])
        objects_dict,type_geometry=ObjectDictionary.factoryDictonary(dbRsc,id_layer)
        if objects_dict is  None:
            return None
        rscLayer.setDictionaryObjects(objects_dict)
        rscLayer.setTypeGeometry(type_geometry)
        attributes_layer=RscAttribute.factoryListRscAttribute(dbRsc,id_layer)
        '''
        attObjectKey=RscAttribute("ObjectKey","ObjectKey",0,0,32,0)
        attributes_layer.insert(0,attObjectKey)
        '''
        index_classid=None
        for i,att in enumerate(attributes_layer):
            att.Name=att.Name.lower()
            if att.Name.lower()=='classid':
                index_classid=i
        if index_classid is None:
            classid_att=RscAttribute('classid',"Код объекта",32800,22,256,0)
        else:
            classid_att=attributes_layer[index_classid]
            attributes_layer.pop(index_classid)
        attributes_layer.insert(0, classid_att)
        mi_style=RscAttribute("mi_style","mi_style",0,0,255,0)
        rscLayer.addAttribute(attributes_layer)
        return rscLayer
def writeFileToBlob(db:DbRsc,name_tab,path_file,zipTo=True):
    name_file = str(Path(path_file).stem)
    name=str(Path(path_file).name)
    byte_file_source=None
    with open(path_file, "rb") as input_file:
        byte_file_source = input_file.read()
    if zipTo:
        byte_file_source=generate_zip((name,byte_file_source))

    if byte_file_source is None:
        return False
    name_file=str(Path(path_file).stem)
    sql_create="CREATE TABLE ["+name_tab+"](  [id] INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,  [name] TEXT,  [rsc] BLOB)"
    db.runSqlNoResult(sql_create)
    sql_insert="insert into "+name_tab+" (name,rsc) values(?,?)"
    values=(name_file,byte_file_source)
    db.insert(sql_insert,values)
    db.commit()
    return True
def extractRscToFolder(path_db_dictionary,path_folder):
    db=DbRsc(str(path_db_dictionary))

    sql_read = "select * from table_rsc"
    rows = db.select(sql_read)
    db.close(False)
    if len(rows)==0:
        return None
    name=rows[0][1]+".rsc"
    #name = rows[0][1]
    path_file_rsc=path_folder.joinpath(name)
    #path_file_rsc = path_folder
    if not path_file_rsc.exists():
        decompressZipToFile(rows[0][2],str(path_folder))
    return str(path_file_rsc)
def getIdLayer(db:DbRsc,name_layer):
    sql = "Select id_layer from layers where name='" + name_layer +"'"
    logger.debug("Get id_layer:"+sql)
    rows=db.select(sql)
    if len(rows)==0:
        return None
    return rows[0][0]
def getKeysFieldLayer(db:DbRsc,id_layer,name_field):
    #sql='Select * from foreign_key_layers where id_layer='+str(id_layer)+' and name_field_base="'+name_field+'"'
    '''
    id_layer=getIdLayer(db,name_layer)
    if id_layer is None:
        return None
    '''
    key_info=None

    if name_field.lower()=="classid":
        sql = "Select * from foreign_key_layers where id_layer=" + str(
            id_layer) + " and name_field_base='objectkey'"
    else:
        sql = "Select * from foreign_key_layers where id_layer=" + str(id_layer) + " and name_field_base='" + name_field + "' order by code"
    rows=db.select(sql)
    if len(rows)==0:
        return None
    row=rows[0]
    key_info={"tab_dict":row[1],"name_field_key":row[3],"name_fiedl_value":row[4]}

    return key_info
def getPropertyVolumns(db,name_table):
    id_layer=getIdLayer(db,name_table)
    sql="pragma table_info(["+name_table+"])"
    rows=db.select(sql)

    fields_info=[]
    for row in rows:
        jkl=0
        name_field=row(1)
        key_info=getKeysFieldLayer(db,id_layer,name_field)
        #type_str=sql_query.value(1)
        '''
        type=0
        if type_str.find("INTEGER")>=0 or type_str.find("DOUBLE"):
            type=1
        fields_info.append({'name':name_field,"type":type,"key_inf":key_info})
        '''
    return fields_info
def getPropertyVolumnsAxipy(db:DbRsc,name_table):
    id_layer=getIdLayer(db,name_table)
    tab_axipy=axipy.data_manager.find(name_table)
    schema=tab_axipy.schema
    attribute_names=schema.attribute_names

    fields_info=[]
    for name_field_x in attribute_names:
        name_field=name_field_x.lower()
        if name_field=='classid':
            key_info = getKeysFieldLayer(db, id_layer, "objectkey")
        else:
            key_info=getKeysFieldLayer(db,id_layer,name_field)
        #if key_info is None:
        #    continue
        att=schema.by_name(name_field)
        type=0
        if att.type_string=='int' or att.type_string=='integer' or att.type_string=='float' or att.type_string=='double':
            type=1
        fields_info.append({'name_origin': name_field_x,'name': name_field, "type": type, "key_inf": key_info,'attribute':att})
        '''
        if type_str.find("INTEGER")>=0 or type_str.find("DOUBLE"):
            type=1
        fields_info.append({'name':name_field,"type":type,"key_inf":key_info})
        '''
    return fields_info

class GenerateStyleFromClassId:
    def __init__(self,db:DbRsc,name_base_field,second_key):
        self.__db=db
        self.__style_table='style_object'
        self.__main_key=name_base_field
        self.__second_key=second_key
        self.__isNotSyle=False
        self.__existStyleTable()
    def getStyle(self,value_base_key,value_second):
        if self.__isNotSyle:
            return None
        result_code=str(value_base_key)
        if value_second>1:
            result_code=result_code+str(value_second)
        sql_style_and_type="select name,location_type,mi_style from "+self.__style_table+" where object_key='"+result_code+"'"
        rows=self.__db.select(sql_style_and_type)
        if len(rows)==0:
            return None
        type_geometry=TypeGeometry(rows[0][1])
        return {'name':rows[0][0],'geometry_type':type_geometry,'style':rows[0][2]}
    @property
    def NameBaseKey(self):
        return self.__main_key
    @property
    def NameSecondKey(self):
        return self.__second_key
    def __existStyleTable(self):
        sql="select name from  sqlite_master where name='"+self.__style_table+"'"
        rows=self.__db.select(sql)
        if len(rows)==0:
            self.__isNotSyle=True
        return
def factoryGenerateStyleFromClassId(db,keys_info):
    base_name_key=None
    second_key_name=None
    for item_info in keys_info:
        if item_info['key_inf'] is None:
            continue
        if item_info['type']==0:
            base_name_key=item_info['name']
        if item_info['type'] == 1 and second_key_name is None:
            second_key_name=item_info['name']
            break
    if base_name_key is not None and second_key_name is not None:
        return GenerateStyleFromClassId(db,base_name_key,second_key_name)
    return None





