import uuid
from pathlib import Path

import axipy
from PySide2.QtCore import QObject
from axipy import SelectionTable, Table, Style, Geometry, LineString, Polygon, Point, Notifications

from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.helper_db.db_add_helper import TypeGeometry
from PanoramaRsc.ui.add_tools.observers import factoryObserver
from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict

def maybeTransfomGeometry(geo_object:Geometry,type_available:TypeGeometry):
    if type_available==type_available.Any:
        return 2
    if isinstance(geo_object,LineString):
        if type_available==TypeGeometry.PolyLine:
            return 0
        if type_available==TypeGeometry.Polygon:
            return 1
        if type_available==TypeGeometry.Point:
            return 2
    if isinstance(geo_object, Polygon):
        if type_available==TypeGeometry.Polygon:
            return 0
        if type_available==TypeGeometry.PolyLine:
            return 1

    if isinstance(geo_object,Point):
        if type_available==TypeGeometry.Point:
            return 0


    return 2
def getDefaultValue(path_db,name_base_layer):
    db=DbRsc(path_db)
    sql="select fk_key.* from foreign_key_layers fk_key,(select id_layer from layers where name='"+name_base_layer+"') layer where layer.id_layer=fk_key.id_layer"
    rows=db.select(sql)
    dict_default={}
    for row in rows:
        name_field=row[2]
        if row[6]==1:
            ''' base value'''
            sql_def_value="select "+row[3]+",type_geometry,mi_style from "+row[1]+" ORDER BY "+row[3]
        else:
            sql_def_value = "select " + row[3] + " from " + row[1] + " ORDER BY " + row[3]
        rows_def=db.select(sql_def_value)
        if name_field=='objectkey':
            name_field='classid'
        def_value=rows_def[0][0]
        if row[6]==1:
            mi_style=rows_def[0][2]
            type_geometry=TypeGeometry(rows_def[0][1])
            dict_default[name_field]={'value':def_value,'type_geometry':type_geometry,"style":mi_style}
        else:
            dict_default[name_field] = {'value': def_value}

    db.close(False)
    return dict_default




def find_base_table(tab):
    if isinstance(tab,SelectionTable):
        return find_base_table(tab.base_table)
    return tab
def TableInDictionary(table,config_dict:ConfigRscDict)->str:
    base_table = find_base_table(table)
    if base_table.provider != 'SqliteDataProvider':

        return None
    try:
        path_file_db = base_table.properties['fileName']
    except:
        return None
    if Path(path_file_db) == config_dict.PathDb:
        return base_table.name
    return None
class SetDefaultValueInTab(QObject):
    def __init__(self,config_dictionary):
        super().__init__()
        self.__name_global_id="globalid"
        self.__config_dict=config_dictionary
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__observer_change_config.changed.connect(self.__update_config)
        #self.__getNameFieldGlobalId()
        self.__info_tables_in_dictionary={}
        self.__initTables()
        axipy.data_manager.added.connect(self.__add_object)
        axipy.data_manager.removed.connect(self.__close_table)
        jkl=0
    def __initTables(self):
        open_tables=axipy.data_manager.tables
        if len(open_tables)==0:
            return
        for tab in open_tables:
            name_base_table=TableInDictionary(tab,self.__config_dict)
            if name_base_table is None:
                continue
            self.__info_tables_in_dictionary[tab.name]={'tab_in_dict':{'name_table':name_base_table}}
    def __add_object(self,name):
        obj_data=axipy.data_manager.find(name)
        if not isinstance(obj_data,Table):
            return
        if (not obj_data.is_editable) or (not obj_data.is_spatial):
            return
        name_base_table = TableInDictionary(obj_data, self.__config_dict)
        if name_base_table is not None:
            '''
            def change_table(name,info):
                jkl=0
            '''
            self.__info_tables_in_dictionary[obj_data.name] = {'tab_in_dict': {'name_table': name_base_table,'table':obj_data,'default_value':None}}
            self.__active_table_name=obj_data.name
            #obj_data.data_changed.connect(lambda info: self.__change_table(obj_data.name,info)  )
            obj_data.data_changed.connect(self.change_table)
    def change_table(self,info):
        if not (info['operation']=='insert'):
            return
        table_name=self.sender().name()
        info_table=self.__info_tables_in_dictionary[ table_name]
        if info_table['tab_in_dict']['default_value'] is None:
            ''' Загружаем значения по умолчанию'''
            info_table['tab_in_dict']['default_value']=getDefaultValue(str(self.__config_dict.PathRscDb),info_table['tab_in_dict']['name_table'])
        table_edit=axipy.data_manager.find(table_name)
        default_value=info_table['tab_in_dict']['default_value']
        ft_iterator=table_edit.itemsByIds(info['ids'])
        ft_edits=[]
        for ft in ft_iterator:
            if ft.geometry is not None:
                type_geometry =default_value['classid']['type_geometry']
                maybe_transform=maybeTransfomGeometry(ft.geometry,type_geometry)
                if maybe_transform<2:
                    if maybe_transform==1:
                        new_geo=None
                        if type_geometry==TypeGeometry.PolyLine:
                            new_geo=ft.geometry.to_linestring()
                        if type_geometry==TypeGeometry.Polygon:
                            new_geo=ft.geometry.to_polygon()
                        if new_geo is not None:
                            axipy.Notifications.push("Справочник", 'Геометрия преобразована в соотвестии справочнику', Notifications.Warning)

                            ft.geometry=new_geo


                    mi_style=default_value['classid']['style']

                    if mi_style is not None:
                        try:
                            axi_style=Style.from_mapinfo(mi_style)
                            ft.style = axi_style
                        except:
                            pass

            for name_field in list(default_value.keys()):
                ft[name_field]=default_value[name_field]['value']
            try:
                value_guid=ft[self.__name_global_id]
                if value_guid=='':
                    ft[self.__name_global_id]=str(uuid.uuid4())
            except:
                pass
            ft_edits.append(ft)
        table_edit.update(ft_edits)



        jkl=0

    def __close_table(self,name):
        if name=='Выборка':
            return
        table=None
        if name in self.__info_tables_in_dictionary.keys():
            if 'table' in self.__info_tables_in_dictionary[name]['tab_in_dict'].keys():
                table=self.__info_tables_in_dictionary[name]['tab_in_dict']['table']

            self.__info_tables_in_dictionary.pop(name)
        #table=axipy.data_manager.find(name)
            if table is not None:
                table.data_changed.disconnect(self.change_table)


    def __update_config(self):
        self.__config_dict=ConfigRscDict.load()
    def __getNameFieldGlobalId(self):
        db_temp=DbRsc(str(self.__config_dict.PathRscDb))
        try:
            sql="select * from base_field_name"
            rows=db_temp.select(sql)
            name_field_guid=rows[0][0]
            if len(name_field_guid)>0:
                self.__name_global_id=name_field_guid.replace(' ','_')
        except:
            return self.__name_global_id
        finally:
            db_temp.close()
