import io
import time

from PySide2.QtCore import QObject, Signal
from axipy import Task
from loguru import logger
from osgeo import ogr

from PanoramaRsc.ui.add_tools.fgis_tp.fgis_tp_10_izm import parse
from PanoramaRsc.ui.add_tools.helper_db.SqliteDataSource import DbSqliteDataSource
class CalcIntProcent:
    def __init__(self,count):
        self.__last_proc=-1
        self.__count=count
    def getValue(self,curent):
        cur_val=int((curent/self.__count)*100+0.5)
        if cur_val==self.__last_proc:
            return None
        self.__last_proc=cur_val
        return cur_val

def getGeometry(obj):
    temp_file=io.StringIO()
    obj.export(temp_file,0)
    string_geo_obj=temp_file.getvalue()
    temp_file.close()
    ogr_geometry=ogr.CreateGeometryFromGML(string_geo_obj)
    srs_name=obj.srsName
    return ogr_geometry,srs_name
def getNameAtt(obj):
    name_exception_words=['gds_collector_', 'gds_elementtree_node_', 'original_tagname_', 'parent_object_', 'ns_prefix_', 'id', 'id_nsprefix_', 'metaDataProperty', 'metaDataProperty_nsprefix_', 'description', 'description_nsprefix_', 'descriptionReference', 'descriptionReference_nsprefix_', 'identifier', 'identifier_nsprefix_', 'name', 'name_nsprefix_', 'extensiontype_', 'boundedBy', 'boundedBy_nsprefix_', 'location', 'location_nsprefix_']
    list_att_name_value= {}
    list_obj_att=(obj.__dict__.keys())
    for att_name_obj in list_obj_att:
        if att_name_obj in name_exception_words:
            continue
        if att_name_obj.find('_nsprefix_')>0:
            continue

        att_obj=getattr(obj,att_name_obj)
        class_obj=None
        try:
            class_obj=getattr(att_obj,'__dict__')
        except:
            pass
        #type_obj=type(att_obj)
        #list_att_obj=list(att_obj.__dict__.keys())
        if class_obj is not None:
            type_pref=class_obj['ns_prefix_']
            geo_obj,srs_name=getGeometry(att_obj)
            list_att_name_value['geometry']=geo_obj
            list_att_name_value['srs_name']=srs_name
        else:
            list_att_name_value[att_name_obj]=att_obj
    return list_att_name_value
class ImportGmlGisTp(QObject):
    value_changed=Signal(int)
    message_changed=Signal(str)

    def __init__(self,db_dictionary,db_spatial,name_fields_to_lower=True):
        super().__init__()
        self.__db_dict=db_dictionary
        self.__db_sp=db_spatial
        self.__count = 0
        self.__isCancel = False
    def setCancel(self):
        self.__isCancel=True
    def run(self,t:Task,path_gml):
        #print("run:"+path_gml)
        logger.debug("Start import:"+path_gml)
        #self.__db_dict.reOpen()
        self.__isCancel = False
        objGml=parse(path_gml,True)
        featureMember=objGml.featureMember
        count=len(featureMember)
        #print("count:"+str(count))
        logger.debug("count features:" + str(count))
        cls_proc=CalcIntProcent(count)
        self.__layer_name=[]
        self.__layer_ogr=None
        self.__count=0
        for i,ftMember in enumerate(featureMember):
            logger.debug(i)
            if self.__isCancel:
                break
            cur_procent=cls_proc.getValue(i)
            logger.debug(cur_procent)
            if cur_procent is not None:
                self.value_changed.emit(cur_procent)
            #time.sleep(1)
            self.__count=i
            ft = ftMember.AbstractFeature
            name_layer = ft.original_tagname_
            logger.debug("Name layer:"+name_layer)
            att_values = getNameAtt(ft)
            if not (name_layer in self.__layer_name):
                self.message_changed.emit(name_layer)
                self.__layer_name.append(name_layer)
                self.__createLayer(name_layer,att_values['srs_name'])
            self.__createFeature(att_values)
        if self.__layer_ogr is not None:
            self.__layer_ogr.CommitTransaction()
            self.__layer_ogr=None

        self.__db_dict.close()
    def __createLayer(self,name_layer,str_epsg):
        logger.debug("Srs layer:" + str_epsg)
        srs_wkt=DbSqliteDataSource.strEpsgToWkt(str_epsg)
        logger.debug(srs_wkt)
        logger.debug(self.__db_dict.isOk)
        #logger.debug(self.__db_dict.getLayers())
        self.__db_dict.reOpen()
        logger.debug(self.__db_dict.isOk)
        layer_info=self.__db_dict.getLayerByName(name_layer)
        logger.debug("Ok layerRsc att")
        if self.__layer_ogr is not None:
            self.__layer_ogr.CommitTransaction()
            self.__layer_ogr=None
        name_exist_layer=self.__db_sp.getLayer(name_layer)
        if name_exist_layer is not None:
            name_exist_layer=None
        else:
            isOkLayer=self.__db_sp.createLayer(layer_info, srs_wkt)
            logger.debug(isOkLayer)

        self.__layer_ogr=self.__db_sp.getLayer(name_layer)
        self.__layer_ogr.StartTransaction()
    def __createFeature(self,att_values):
        list_name_att=list(att_values.keys())
        new_feature = ogr.Feature(self.__layer_ogr.GetLayerDefn())
        for name in list_name_att:
            if name=="geometry" or name=="srs_name":
                continue
            value=att_values[name]
            if name=='CLASSID':
                value=str(value)
                #new_feature.SetField("ObjectKey", value)
            new_feature.SetField(name, value)
        new_feature.SetGeometry(att_values["geometry"])
        self.__layer_ogr.CreateFeature(new_feature)
        return
    @property
    def layers(self):
        return self.__layer_name
    @property
    def countFeature(self):
        return self.__count
    @property
    def isCancel(self):
        return self.__isCancel


