import struct
from pathlib import Path
from zipfile import ZipFile

from loguru import logger

from PanoramaRsc.ui.add_tools.rsc_tools.RSC.add_help.dbRsc import factoryDb
#from MyTask.Panorama.RSC.add_help.dbRsc import DbRsc, factoryDb
#from MyTask.Panorama.RSC.readFullName import InfoByFullNameTable
#from PanoramaRsc.ui.add_tools.helper_db.dbRsc import factoryDb
#from PanoramaRsc.ui.add_tools.rsc_tools.RSC.add_help.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.rsc_tools.RSC.readFullName import InfoByFullNameTable


#from MyTask.Panorama.RSC.rscPalettes import factoryRscPrintPrametrs


def getString(byte_str):
    try:

        out_str=byte_str.decode('cp1251')
    except:
        index=byte_str.find(b'\00')

        #o.decode('latin-1').encode("utf-8")
        str_out=None

            #str_out=byte_str[0:index].decode("utf-8")
        str_out=byte_str[0:index].decode('cp1251')

        return str_out
    index=out_str.find('\00')
    if index>0:
        return out_str[0:index]
    return out_str.replace('\00','')
def getDescriptorTable(bytes,start_byte):
    struct_descriptor_header_obj = struct.Struct('< I I I')
    buffer=bytes[start_byte:start_byte+3*4]
    data=struct_descriptor_header_obj.unpack(buffer)
    return {'offset':data[0],'len':data[1],'count_record':data[2]}
def readDoubleArray(bytes,offset,count):
    __structure = struct.Struct('< d')
    double_values=[]
    for i in range(count):
        buffer=bytes[offset+i*8:offset+(i+1)*8]
        data = __structure.unpack(buffer)
        double_values.append(data[0])
    return double_values
def readIntArray(bytes,offset,count):
    __structure = struct.Struct('< i')
    int_values=[]
    for i in range(count):
        buffer = bytes[offset + i * 4:offset + (i + 1) * 4]
        data = __structure.unpack(buffer)
        int_values.append(data[0])
    return int_values
class DefaultValue:
    __structure = struct.Struct('< I I d d d')
    def __init__(self):
        pass
    @staticmethod
    def Size():
        return 32
    def load(self, bytes, offset):
        buffer = bytes[offset:offset + DefaultValue.Size()]
        data = self.__structure.unpack(buffer)
        self.Number=data[0]
        self.Code=data[1]
        self.MinValue=data[2]
        self.DefaultValue = data[3]
        self.MaxValue= data[4]
    @staticmethod
    def factory(bytes,offset):
        cls=DefaultValue()
        cls.load(bytes,offset)
        return cls

    def recordToDb(self):
        out_data=(self.Code,self.MinValue,self.DefaultValue,self.MaxValue)
        return out_data
class ClassificationValue:
    __structure = struct.Struct('< I 32s')
    def __init__(self):
        pass
    def load(self,bytes,offset):
        buffer=bytes[offset:offset+36]
        data = self.__structure.unpack(buffer)
        self.Code=data[0]
        self.Name=getString(data[1])
    def recordToDb(self,parentCode):
        out_dat=(parentCode,self.Code,self.Name)
        return out_dat

class Attribute:
    '''Структура записи таблицы семантик классификатора'''
    __structure=struct.Struct('< I h b b 32s 16s 8s h b b I I I I')
    sizeRecord=84
    def __init__(self):
        self.__classificationValues=[]
        self.__defaultValues=[]
    @staticmethod
    def getSizeRecord():
        return 84
    @property
    def listDefaultValues(self):
        return self.__defaultValues
    @property
    def listClassificationValues(self):
        return self.__classificationValues
    def load(self,bytes,start_index):
        buffer=bytes[start_index:start_index+84]
        data = self.__structure.unpack(buffer)
        self.Code=data[0]
        self.Type=data[1]
        self.AllowRepeate=data[2]
        self.IsInternal=data[3]
        self.Name=getString(data[4])
        self.ShortName=getString(data[5])
        self.Units=getString(data[6])
        self.Length=data[7]
        self.Precisssion=data[8]
        self.Flag = data[9]
        self.__num1=data[10]
        self.__num2 = data[11]
        self.__num3 = data[12]
        self.__num4 = data[13]
        self.__defaultValues=None
        self.__classificationValues=None
        if self.__num1 >0 and self.__num2>0:
            ''' Читаем ClassificationValue '''
            self.__classificationValues=[]
            for i in range(self.__num2):
                offset=self.__num1+i*36
                cls_ClassificationValue=ClassificationValue()
                cls_ClassificationValue.load(bytes,offset)
                self.__classificationValues.append(cls_ClassificationValue)
        if self.__num3<=0 and self.__num4<=0:
            return
        offset=self.__num3
        self.__defaultValues=[]
        for i in range(self.__num4):
            cls_default=DefaultValue.factory(bytes,offset+i*DefaultValue.Size())
            self.__defaultValues.append(cls_default)

    def recordToDb(self):
        out_tuple=(self.Code,self.Type,self.AllowRepeate,self.IsInternal,self.Name,self.ShortName,self.Units,self.Length,self.Precisssion,self.Flag)
        return out_tuple







class TableAttribute:
    def __init__(self):
        self.__list_attribute=[]
    def loadTable(self,desc_att_table,bytes_source):
        start_byte=desc_att_table['offset']
        count_record=desc_att_table['count_record']

        for i in range(count_record):
            offset=start_byte+i*Attribute.getSizeRecord()
            cls_att=Attribute()
            cls_att.load(bytes_source,offset)
            self.__list_attribute.append(cls_att)
        jkl=0

    @property
    def Attributes(self):
        return self.__list_attribute
    def saveToDb(self,clsDb):
        sql_insert="Insert Into semantic_attribute (code,type,allowrepeate,is_internal,name,short_name,units,length,precisssion,flag) VALUES(?,?,?,?,?,?,?,?,?,?)"
        sql_insert_default="Insert Into default_values (code,min_value,default_value,max_value) VALUES(?,?,?,?)"
        sql_insert_class = "Insert Into classificator_values (id,code,name) VALUES(?,?,?)"
        for record in self.__list_attribute:
            values=record.recordToDb()
            clsDb.insert(sql_insert,values)
            if record.listDefaultValues is not None:
                for item in record.listDefaultValues:
                    value=item.recordToDb()
                    clsDb.insert(sql_insert_default,value)
            if record.listClassificationValues is not None:
                for item in record.listClassificationValues:
                    code_parent=record.Code
                    values_class=item.recordToDb(code_parent)
                    clsDb.insert(sql_insert_class,values_class)


class RecordThresholds:
    __structure = struct.Struct('< I I b 7s I h h I h h')
    def __init__(self):
        pass
    def load(self,bytes,offset):
        size=self.__structure.size
        buffer = bytes[offset:offset + 32]
        data = self.__structure.unpack(buffer)
        ''' Длина записи'''
        self.len_record=data[0]
        self.Code=data[1]
        self.Locale=data[2]
        self.Reserv=data[3]
        self.FirstAttibuteCode=data[4]
        self.FirstAttibuteConstraints=data[5]
        self.FirstAttributeDefauldsNumber=data[6]
        self.SecondAttibuteCode=data[7]
        self.SecondAttibuteConstraints=data[8]
        self.SecondAttributeDefauldsNumber=data[9]
        self.FirstConstraints=readDoubleArray(bytes,offset+size,self.FirstAttibuteConstraints)
        offset1=offset+size+self.FirstAttibuteConstraints*8
        self.SecondConstraints=readDoubleArray(bytes,offset1,self.SecondAttibuteConstraints)
        if self.SecondAttibuteConstraints>0:
            count=self.FirstAttibuteConstraints*self.SecondAttibuteConstraints
        else:
            count = self.FirstAttibuteConstraints
        offset2=offset1+self.SecondAttibuteConstraints*8
        self.MatrixExtensions=bytes[offset2:offset2+count]
        return
    @staticmethod
    def factory(bytes,offset):
        cls=RecordThresholds()
        cls.load(bytes,offset)
        return cls




class Thresholds:
    ''' Таблица порогов '''
    def __init__(self,desc_tab):
        self.__des_tab=desc_tab
    def load(self,bytes):
        self.__records=[]
        offset=self.__des_tab['offset']
        count=self.__des_tab['count_record']
        offset_cur=offset
        for i in range(count):
            cls_rec=RecordThresholds.factory(bytes,offset_cur)
            offset_cur=offset_cur+cls_rec.len_record
            self.__records.append(cls_rec)
    @property
    def Records(self):
        return self.__records
class RecordLayer:
    __structure = struct.Struct('< I 32s 16s b B h')
    def load(self,bytes,offset):
        size=self.__structure.size
        buffer=bytes[offset:offset+size]
        data = self.__structure.unpack(buffer)
        ''' Длина записи'''
        self.len_record = data[0]
        self.Name=getString(data[1])
        self.ShortName=getString(data[2])
        self.LayerNumber =data[3]
        self.DisplayOrderIndex=data[4]
        ''' num2 количество кодов семантики'''
        num2=data[5]
        self.AttributeCodes=None
        if num2>0:
            self.AttributeCodes=readIntArray(bytes,offset+size,num2)
        else:
            self.AttributeCodes = readIntArray(bytes, offset + size, 1)
    @staticmethod
    def factory(bytes, offset):
        cls = RecordLayer()
        cls.load(bytes, offset)
        return cls
    def recordToDb(self):
        out_data=(self.LayerNumber,self.Name,self.ShortName,self.DisplayOrderIndex)
        return out_data
class TableLayers:
    def __init__(self,desc_tab):
        self.__desc_tab=desc_tab
    def load(self,bytes):
        self.__records = []
        offset = self.__desc_tab['offset']
        count = self.__desc_tab['count_record']
        offset_cur = offset
        for i in range(count):
            cls_rec = RecordLayer.factory(bytes, offset_cur)
            offset_cur = offset_cur + cls_rec.len_record
            self.__records.append(cls_rec)


    def saveToDb(self, clsDb):
        sql_inser_layers=  "Insert Into layers (id_layer,name,short_name,display_orderIndex) VALUES(?,?,?,?)"
        sql_inser_layers_att = "Insert Into layers_attribute (num_layer,code_att) VALUES(?,?)"
        for record in self.__records:
            values=record.recordToDb()
            clsDb.insert(sql_inser_layers,values)
            if record.AttributeCodes is not None:
                for item in record.AttributeCodes:
                    values=(record.LayerNumber,item)
                    clsDb.insert(sql_inser_layers_att,values)

class RecordObjectClassification:
    __structure = struct.Struct('< I I I I 32s 32s b B b b b b b b h B B B B B B')
    def load(self,bytes,offset):
        size=self.__structure.size
        buffer=bytes[offset:offset+size]
        data = self.__structure.unpack(buffer)
        ''' Длина записи'''
        self.len_record = data[0]
        self.Code = data[1]
        self.InnerCode = data[2]
        self.IdentifyCode = data[3]
        try:
            self.ShortName=getString(data[4])
        except:
            self.ShortName=''
            jkl=0
        self.Name=getString(data[5])
        self.LocationType=data[6]
        self.LayerNumber=data[7]
        self.IsScalable=data[8]
        self.LevelMin=data[9]
        self.LevelMax=data[10]
        self.LevelMax=15-self.LevelMax
        self.ExtendedLocationType=data[11]
        self.DirectionDigitization=data[12]
        self.DisplayWithAttributes=data[13]
        self.ExtensionNumber=data[14]
        self.LinkedLabelsNumber=data[15]
        self.CanCompress=data[16]
        self.MaxZoom=data[17]/10.0
        self.MinZoom=data[18]/10.0
        self.IncludeBorder=data[19]
        self.Reserv=data[20]
    def recordToDb(self):
        out_data=(self.Code,self.InnerCode,self.IdentifyCode,self.ShortName,self.Name,self.LocationType,self.LayerNumber,self.IsScalable,self.LevelMin,self.LevelMax,self.ExtendedLocationType,self.DirectionDigitization,self.DisplayWithAttributes,self.ExtensionNumber,self.LinkedLabelsNumber,self.CanCompress,self.MaxZoom,self.MinZoom,self.IncludeBorder)
        sql_insert_object = "Insert Into objects (code,inner_code,identify_code,object_key,name,location_type,layer_number,is_scalable,level_min,lavel_max,extended_location_type,direction_digitization,display_with_attributes,extension_number,linked_labels_number,can_compress,max_zoom,min_zoom,include_border) "
        return out_data
    @staticmethod
    def factory(bytes, offset):
        cls = RecordObjectClassification()
        cls.load(bytes, offset)
        return cls
class TableObjectClassification:
    def __init__(self,desc_tab):
        self.__desc_tab=desc_tab
    def load(self,bytes):
        self.__records = []
        offset = self.__desc_tab['offset']
        count = self.__desc_tab['count_record']
        offset_cur = offset
        for i in range(count):

            cls_rec = RecordObjectClassification.factory(bytes, offset_cur)
            offset_cur = offset_cur + cls_rec.len_record
            self.__records.append(cls_rec)
    def saveToDb(self, clsDb):
        sql_insert_object=  "Insert Into objects (code,inner_code,identify_code,object_key,name,location_type,layer_number,is_scalable,level_min,lavel_max,extended_location_type,direction_digitization,display_with_attributes,extension_number,linked_labels_number,can_compress,max_zoom,min_zoom,include_border) "
        sql_insert_object=sql_insert_object+" VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
        #sql_insert_object = sql_insert_object + " VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
        for record in self.__records:
            values=record.recordToDb()
            clsDb.insert(sql_insert_object,values)
class RecordDisplayParameter:
    __structure = struct.Struct('< I H H')
    def load(self,bytes,offset):
        size=self.__structure.size
        buffer=bytes[offset:offset+size]
        data = self.__structure.unpack(buffer)
        ''' Длина записи'''
        self.len_record = data[0]
        self.InnerCode=data[1]
        self.__numfunctionDisplay=data[2]
        len_blob=self.len_record-size
        self.__blob=bytes[offset+size:offset+size+len_blob]
        return
    def recordToDb(self):
        out_tuple=(self.InnerCode,self.__numfunctionDisplay,self.__blob)
        return out_tuple
    @staticmethod
    def factory(bytes, offset):
        cls = RecordDisplayParameter()
        cls.load(bytes, offset)
        return cls
class RecordPossibleAttributes:
    __structure = struct.Struct('< I I B 3s H H')
    __struct_add= struct.Struct('< I')
    def load(self,bytes,offset):
        size=self.__structure.size
        buffer=bytes[offset:offset+size]
        data = self.__structure.unpack(buffer)
        self.len_record = data[0]
        self.class_code=data[1]
        self.localization=data[2]
        self.count_required_params=data[4]
        self.count_possible_params=data[5]
        self.__required_possible_semantic=[]
        offset_curent=offset+size
        for i in range(self.count_required_params):
            buf=bytes[offset_curent:offset_curent+self.__struct_add.size]
            offset_curent=offset_curent + self.__struct_add.size
            data = self.__struct_add.unpack(buf)
            self.__required_possible_semantic.append({'code_obj':self.class_code,'code_semantic':data[0],'is_required':1})

        for i in range(self.count_possible_params):
            buf=bytes[offset_curent:offset_curent+self.__struct_add.size]
            offset_curent=offset_curent + self.__struct_add.size
            data = self.__struct_add.unpack(buf)
            self.__required_possible_semantic.append({'code_obj':self.class_code,'code_semantic':data[0],'is_required':0})
        return
    def recordToDb(self):
        out_data=[]
        for item in self.__required_possible_semantic:
            out_data.append((item['code_obj'],item['code_semantic'],item['is_required']))
        return out_data





    @staticmethod
    def factory(bytes, offset):
        cls =  RecordPossibleAttributes()
        cls.load(bytes, offset)
        return cls

class TablePossibleAttributes:

    def __init__(self,desc_tab):
        self.__desc_tab=desc_tab
    def load(self,bytes):
        self.__records = []
        offset = self.__desc_tab['offset']
        count = self.__desc_tab['count_record']
        offset_cur = offset
        for i in range(count):
            cls_rec =  RecordPossibleAttributes.factory(bytes, offset_cur)
            offset_cur = offset_cur + cls_rec.len_record
            if cls_rec.len_record>20:
                jkl=0
            self.__records.append(cls_rec)
    def saveToDb(self,clsDb):
        sql_insert="Insert Into object_semantic_required_possible (code_object,code_semantic,is_required) VALUES(?,?,?)"
        for record in self.__records:
            values=record.recordToDb()
            clsDb.insertMany(sql_insert,values)


class TableDisplayParameters:
    def __init__(self,desc_tab):
        self.__desc_tab=desc_tab
    def load(self,bytes):
        self.__records = []
        offset = self.__desc_tab['offset']
        count = self.__desc_tab['count_record']
        offset_cur = offset
        for i in range(count):

            cls_rec = RecordDisplayParameter.factory(bytes, offset_cur)
            offset_cur = offset_cur + cls_rec.len_record
            self.__records.append(cls_rec)
    def saveToDb(self,clsDb):
        sql_insert="Insert Into displayparams (inner_cod,type,descript_bytes) VALUES(?,?,?)"
        for record in self.__records:
            values=record.recordToDb()
            clsDb.insert(sql_insert,values)




class Rsc:
    def __init__(self,header,bytes_source):
        self.__header=header
        self.__title=getString(header[10])
        self.__scale=header[12]
        self.__byte_source_rsc=bytes_source
        self.__tableSemantic=None
        self.__tableThresholds=None
        self.__printParams=None
    def setPrintParams(self,cls_print):
        self.__printParams=cls_print
    @property
    def PrintParams(self):
        return self.__printParams
    @property
    def title(self):
        return self.__title
    @property
    def scale(self):
        return self.__scale
    def loadTableSemantic(self,des_table):
        self.__tableSemantic=TableAttribute()
        self.__tableSemantic.loadTable(des_table,self.__byte_source_rsc)
    @property
    def TableSematic(self):
        return self.__tableSemantic
    @property
    def TablePossibleAttributes(self):
        return self.__tablePossibleAttributes
    def loadTableThresholds(self,des_table):
        self.__tableThresholds=Thresholds(des_table)
        self.__tableThresholds.load(self.__byte_source_rsc)
    def loadTablePossibleAttributes(self,des_table):
        self.__tablePossibleAttributes=TablePossibleAttributes(des_table)
        self.__tablePossibleAttributes.load(self.__byte_source_rsc)
    @property
    def TableThresholds(self):
        return self.__tableThresholds
    def loadTableLayer(self,des_tab):
        self.__tableLayers = TableLayers(des_tab)
        self.__tableLayers.load(self.__byte_source_rsc)
    @property
    def TableLayers(self):
        return self.__tableLayers
    def loadTableObjectClassification(self,des_tab):
        self.__tableObjectClassification=TableObjectClassification(des_tab)
        self.__tableObjectClassification.load(self.__byte_source_rsc)
    @property
    def TableObjectClassification(self):
        return self.__tableObjectClassification
    def loadTableDisplayParameters(self,des_tab):
        self.__tableDisplayParameters=TableDisplayParameters(des_tab)
        self.__tableDisplayParameters.load(self.__byte_source_rsc)
    @property
    def TableDisplayParameters(self):
        return self.__tableDisplayParameters
    def createClsFullNames(self):
        self.__cls_fullNames=InfoByFullNameTable.factory(self.__byte_source_rsc)
    def saveToDbFullNames(self,dbCls):
        self.__cls_fullNames.saveToDb(dbCls)
def getRscNames(names_file):
    rsc_files=[]
    for name in names_file:
        if name.lower().find(".rsc")>0:
            rsc_files.append(name)
    return rsc_files
def readFile(path_file):
    f_rsc=None
    cls_path_file=Path(path_file)
    if cls_path_file.suffix.lower()==".rsc":
        with open(path_file, "rb") as f:
            f_rsc = f.read()
        return f_rsc
    if cls_path_file.suffix.lower() == ".rscz":
        with ZipFile(path_file, 'r') as f:
            name_list=f.namelist()
            file_rsc=getRscNames(name_list)[0]
            file_zip_rsc=f.open(file_rsc,"r")
            try:
                f_rsc = file_zip_rsc.read()
            finally:
                file_zip_rsc.close()
    return f_rsc


def factoryRsc(parent,path_file):
    f_rsc=None
    f_rsc=readFile(path_file)
    '''
    with open(path_file, "rb") as f:
        f_rsc=f.read()
    '''
    if f_rsc is None:
        logger.error("Error read rsc from:"+path_file)
        return None
    header=f_rsc[0:120]
    struct_descriptor=struct.Struct('< 4s I I I I I I I 8s 32s 32s 8s I I')
    data=struct_descriptor.unpack(header)
    bt_title=data[10]
    title=getString(bt_title)
    cod_clas=getString(data[11])
    cls_rsc=Rsc(data,f_rsc)
    #from MyTask.Panorama.RSC.rscPalettes import factoryRscPrintPrametrs
    from PanoramaRsc.ui.add_tools.rsc_tools.RSC.rscPalettes import factoryRscPrintPrametrs
    csl_printPrams=factoryRscPrintPrametrs(f_rsc)
    cls_rsc.setPrintParams(csl_printPrams)
    ''' Заголовок таблиц объектов, семантик, сегментов '''
    #struct_descriptor_header_obj = struct.Struct('< I I I I I I I')
    #struct_descriptor_header_obj = struct.Struct('< I I I I I I I')
    #head_table_object=f_rsc[116:144]
    #data_struct_tab_obj=struct_descriptor_header_obj.unpack(head_table_object)
    '''таблица объектов '''
    desc_tab_obj=getDescriptorTable(f_rsc,120)
    cls_rsc.loadTableObjectClassification(desc_tab_obj)
    '''таблица семантики '''
    desc_tab_semantic = getDescriptorTable(f_rsc, 132)
    cls_rsc.loadTableSemantic(desc_tab_semantic)
    ''' Дескриптор таблицы классификатор семантики'''
    desc_clas_semantic=getDescriptorTable(f_rsc, 144)
    ''' таблица умолчаний'''
    desc_tab_defaul = getDescriptorTable(f_rsc, 156)
    ''' таблица возможных семантик '''
    desc_tab_posible_semantic = getDescriptorTable(f_rsc, 168)
    cls_rsc.loadTablePossibleAttributes(desc_tab_posible_semantic)
    ''' таблица сегментов(слоев) '''
    desc_tab_layer = getDescriptorTable(f_rsc, 180)

    cls_rsc.loadTableLayer(desc_tab_layer)
    desc_Thresholds=getDescriptorTable(f_rsc, 192)
    cls_rsc.loadTableThresholds(desc_Thresholds)

    desc_DisplayParameters=getDescriptorTable(f_rsc, 204)
    cls_rsc.loadTableDisplayParameters(desc_DisplayParameters)
    cls_rsc.createClsFullNames()


    return cls_rsc
def loadRscToDb(parent,path_source,path_dest):
    ''' загрузка классификатора Панорамы в справоник sqlite'''
    #logger.add("rsc.log", rotation="500 MB", backtrace=True, diagnose=True)    # Automatically rotate too big file
    logger.info("Start import "+path_source)
    if Path(path_dest).exists():
        print("справочник "+path_dest+" существует")
        return
    cls_rsc=factoryRsc(parent,path_source)

    dbCls = factoryDb(path_dest)
    logger.info("run load display parametrs")
    if parent is not None:
        parent.send_status.emit("load display parametrs")
    cls_rsc.TableDisplayParameters.saveToDb(dbCls)
    cls_rsc.PrintParams.saveToDb(parent,dbCls)
    logger.info("run load semantic ")
    if parent is not None:
        parent.send_status.emit("load semantic")
    cls_rsc.TableSematic.saveToDb(dbCls)
    logger.info("run load layers ")
    if parent is not None:
        parent.send_status.emit("load layers")
    cls_rsc.TableLayers.saveToDb(dbCls)
    logger.info("run load object classification  ")
    if parent is not None:
        parent.send_status.emit("load object classification")
    cls_rsc.TableObjectClassification.saveToDb(dbCls)
    cls_rsc.saveToDbFullNames(dbCls)
    logger.info("run load Posiisible and Required Attributes ")
    if parent is not None:
        parent.send_status.emit("load Attributes")
    cls_rsc.TablePossibleAttributes.saveToDb(dbCls)

    dbCls.close()
    logger.info("End import "+path_source)
    if parent is not None:
        parent.send_status.emit("end prepare rsc")

