import abc
import struct

from PySide2 import QtGui
from PySide2.QtCore import QSize, QPointF, QByteArray, QBuffer
from PySide2.QtGui import QImage, QPainter, QPen, Qt, QColor
from loguru import logger

from MyTask.Panorama.RSC.add_help.saveBrush.cls_saveBrush import brush_style_container
from MyTask.Panorama.RSC.add_help.style_helper import getColor, sizePt, getColorFromDb, sizePan
from MyTask.Panorama.RSC.image_cls.imageMask32.helper import bytesToListValueBit, createImage, imageToBytes
from MyTask.Panorama.RSC.image_cls.image_draw_helper import drawBrushFillHatch
import enum

from PanoramaRsc.ui.add_tools.rsc_tools.RSC.add_help.VectorPath import PointPath


class RenderFragments:
    def __init__(self,width,height):
        self.__width=width
        self.__height=height
        self.__qimage=QImage(width,height,QImage.Format_ARGB32)
        self.__size=QSize(width,height)
        self.__painter = QPainter(self.__qimage)
    def addFragment(self,fragment):
        width=fragment.ChainCoordsSize
        scale=self.__width/width
        self.__painter.resetTransform()
        self.__painter.translate(self.__size.width() / 2, self.__size.height() / 2)
        self.__painter.scale(scale, scale)
        self.__painter.setPen(QPen(fragment.pen_color, 16))
        self.__painter.setBrush(fragment.brush_color)
        polygon = QtGui.QPolygonF()
        for pnt in fragment.ChainCoordsXY:
            polygon.append(QPointF(pnt[0],pnt[1]))

        self.__painter.drawPolygon(polygon)
        #self.__painter.end()
    def save(self,path_file=None):
        self.__painter.end()
        if isinstance(path_file,str):
            self.__qimage.save(path_file)
            return None
        else:
            data = QByteArray()
            buffer = QBuffer(data)
            buffer.open(QBuffer.WriteOnly)
            self.__qimage.save(buffer, 'PNG')
            buffer.close()
            return bytearray(data)



class ImgChainType(enum.Enum):
    Area = 0
    Line = 1
    Circle = 6
    CircleOutline = 7
    EllipseOutline = 64 #// 0x00000040
    Ellipse = 65  # 0x00000041
    ArcCircle = 66 #0x00000042
    SectorPie = 67 # 0x00000043
    TextConst = 68 #0x00000044
    TextAttribute = 69 # 0x00000045
    ArcCircleInverse = 70
class ImgParamType(enum.Enum):
    Line = 128 # 0x00000080
    Dot = 129 # 0x00000081
    Area = 135 # 0x00000087
    VectorText = 160 # 0x000000A0
def VectorExToPng(vector_ex):
    fragments=vector_ex.Fragments
    png_image=QImage(vector_ex.PixelHeight,vector_ex.PixelHeight,QImage.Format_ARGB32)


class ImgSquare:
    __struct = struct.Struct('< I')
    def __init__(self,byte):
        self.load(byte)
    def load(self,byte):
        #buffer = byte[0:self.__struct.size]
        #data = self.__struct.unpack(buffer)
        self.ColorRGBA=byte
    @property
    def needPalette(self):
        if int(self.ColorRGBA[3])==240:
            return True
        return False

class ILine(abc.ABC):
    @abc.abstractmethod
    def getStyle(self,cls_params):
        color_rgba=self.ColorRGBA
        color=getColor(color_rgba,cls_params)
        size_pt=sizePt(self.WidthInMicrones)

        mi_size_pen=10+size_pt
        temp_string="Pen("+str(mi_size_pen)+",2,"+str(color)+")"
        return temp_string
def getPng(bytes):
    index_start=bytes.find(b'PNG')-1
    index_end=bytes.find(b'IEND')
    if index_end==-1 and index_start>=0:
        index_end = bytes.find(b'IEN')
        bytes_png=bytes[index_start:index_end+3]
        bytes_png =bytes_png+bytearray(b"D")
        bytes_png=        bytes_png+bytearray(b'\x00')
        bytes_png=        bytes_png+bytearray(b'\x00')
        bytes_png=        bytes_png+bytearray(b'\x00')
        bytes_png=        bytes_png+bytearray(b'\x00')
        return bytes_png
    bytes_png=bytes[index_start:index_end+8]
    return bytes_png
class ImgFragment:
    __struct = struct.Struct('< B B h')
    def __init__(self,buffer_byte):
        self.__bytes=buffer_byte
        self.ChainCoordsXY=[]
        self.load()
        self.pen_color=Qt.black
        self.__brush_color=None
    def load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.ChainType=ImgChainType(data[0])
        self.ParamType=ImgParamType(data[1])
        self.lenght=data[2]
        self.ParamSymbol=None
        offset=self.__struct.size
        if self.lenght>0:
            num2=offset+self.lenght
            if self.ParamType==ImgParamType.Area:
                self.ParamSymbol=ImgSquare(self.__bytes[offset:offset+4])
                offset = offset + 4
                if offset != num2:
                    offset = offset + 4
            elif self.ParamType==ImgParamType.Line:
                #logger.error(" Not exist class ImgLine")
                self.ParamSymbol = PanLine(self.__bytes[offset:len(self.__bytes)])
                offset = offset + self.ParamSymbol.lenRecord
            elif self.ParamType==ImgParamType.Dot:
                logger.error(" Not exist class ImgDot")
            else:
                if self.lenght==4:
                    self.ParamSymbol=ImgSquare(self.__bytes[offset:offset+4])
                    offset= offset + 4


                if self.lenght==8:
                    self.ParamSymbol=PanLine(self.__bytes[offset:offset+4],len(self.__bytes))
                    #logger.error(" Not exist class ImgLine")
                if self.lenght==16:
                    logger.error(" Not exist class ImgDot")
                if self.lenght>16:
                    logger.error(" Not exist class ImgVectorText")
            #offset=num2
        temp_struct=struct.Struct('< I')
        temp_buf=self.__bytes[offset:offset+4]
        data_1 = temp_struct.unpack(temp_buf)
        offset = offset + 4
        count_points=data_1[0]
        struct_point=struct.Struct('< l l')
        xmin=None
        xmax=None
        ymin=None
        ymax=None
        for i in range(count_points):
            buf_points=self.__bytes[offset:offset+struct_point.size]
            offset= offset + struct_point.size
            data_points=struct_point.unpack(buf_points)
            #x=float(data_points[0])/250.0
            #y=float(data_points[0])/250.0
            x=float(data_points[0])
            y=float(data_points[1])
            if i==0:
                xmin = x
                xmax = x
                ymin = y
                ymax = y
            else:
                xmin=min(xmin,x)
                ymin = min(ymin, y)
                xmax=max(xmax,x)
                ymax = max(ymax, y)
            self.ChainCoordsXY.append((x,y))
        delta_x=(xmax-xmin)+1
        delta_y=(ymax-ymin)+1
        self.ChainCoordsSize=delta_x
        if delta_y>delta_x:
            self.ChainCoordsSize = delta_y
        self.lenght=offset
    @property
    def brush_color(self):
        if self.__brush_color is not None:
            return self.__brush_color
        return self.ParamSymbol.brush_color
    def setBrushColor(self,color):
        self.__brush_color=color
    @property
    def pathPoints(self):
        return PointPath(self.ChainCoordsXY)





        



class VectorEx:
    __struct = struct.Struct('< I I I I I I I I I I B B B B B B B B I I')
    PixelHeight=240
    PixelWidth=240
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid = False
        self.__Fragments=[]
        self.__load()
    @property
    def Fragments(self):
        return self.__Fragments
    def __load(self):
        buffer=self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.PosV = data[1]
        self.PosH = data[2]
        self.Base = data[3]
        self.VLine1 = data[4]
        self.VLine2 = data[5]
        self.VSize = data[6]
        self.HLine1 = data[7]
        self.HLine2 = data[8]
        self.HSize = data[9]
        self.Static = data[10]
        self.Mirror = data[11]
        self.NoPress = data[12]
        self.Scale = data[13]
        self.Centre = data[14]
        self.NoChangeColor = data[15]
        self.Unicode = data[16]
        self.Reserve = data[17]
        self.Border = data[18]
        self.__count=data[19]
        offset=self.__struct.size
        size_bytes=len(self.__bytes)
        for i in range(self.__count):
            buffer_fragment=self.__bytes[offset:size_bytes]
            imgFragment=ImgFragment(buffer_fragment)
            offset=offset+imgFragment.lenght
            self.__Fragments.append(imgFragment)

        self.sizeObject=offset

    def getParams(self,db):
        result_params = {}

        result_params['pixel_h'] = self.PixelHeight
        result_params['pixel_w'] = self.PixelWidth
        #result_params['transparent_flag'] = self.TransparentFlag
        result_params['type'] = "png"
        #result_params['rotate'] = self.Rotate
        cls_render=RenderFragments(self.PixelWidth,self.PixelHeight)
        for fragment in self.__Fragments:
            if isinstance(fragment.ParamSymbol,ImgSquare) and fragment.ParamSymbol.needPalette:
                ''' получаем палитру из базы'''
                index_color=fragment.ParamSymbol.ColorRGBA[0]
                color=getColorFromDb(db,index_color)

                fragment.setBrushColor(color)

            cls_render.addFragment(fragment)
        result_params['image']=cls_render.save(None)
        return result_params


class PanSection:
    __struct = struct.Struct('< I I I')

    def __init__(self, bytes,offset):
        self.__bytes = bytes
        self.__valid = False
        self.__load(offset)

    def __load(self,offset):
        buffer = self.__bytes[offset:offset+self.__struct.size]
        data = self.__struct.unpack(buffer)
        #self.lenRecord = data[0]+self.__struct.size
        self.lenRecord = data[0]
        self.type=data[1]
        self.Base=data[2]
        buffer=self.__bytes[offset+self.__struct.size:len(self.__bytes)]
        self.Sybol=factorySymbol(buffer,offset+self.__struct.size,self.type)
        self.__valid = True

    @property
    def isValid(self):
        return self.__valid



class SquareVector:
    __struct = struct.Struct('< I H B B I I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.ColorRGBA = None
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.FillGridMask = data[1]
        self.FillOnlySymbolsIncludesInArea = 1
        self.Rezerv = data[3]
        self.SymbolStepByXinMicrone = data[4]
        self.SymbolStepByYinMicrone = data[5]
        self.RotationInAngles = data[6]
        self.SymbolSymbol=VectorEx(self.__bytes[self.__struct.size:len(self.__bytes)])
        self.__valid = True

    def getStyle(self, cls_params):
        return "Brush (7, 0)"

    @property
    def isValid(self):
        return self.__valid
class Mask32x32:
    __struct = struct.Struct('< I 128s')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid = False
        self.ColorRGBA = None
        self.__image_qt=None
        self.__load()
    def __load(self):
        data = self.__struct.unpack(self.__bytes)
        self.ColorRGBA=data[0]
        self.__mask_value=data[1]
        self.__valid=True
        self.__createImageMask()
    def __createImageMask(self):
        bit_image = []
        for byte in self.__mask_value:
            array_bit_values = bytesToListValueBit(byte)
            bit_image.extend(array_bit_values)
        self.__image_qt = createImage(bit_image)
        #self.__image_mask=imageToBytes(self.__image_qt)
    @property
    def iamgeQt(self):
        return self.__image_qt
    @property
    def isValid(self):
        return self.__valid
class MultiMark:
    __struct = struct.Struct('< I I I I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.ColorRGBA = None
        self.__load()

    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.countMask = data[1]
        self.SymbolSizeInMicrones = data[2]
        self.OffsetYInMicrones = data[3]
        self.OffsetXInMicrones = data[4]
        self.Mask=[]
        offset=self.__struct.size
        for i in range(self.countMask):
            buffer_mask=self.__bytes[offset:offset+132]
            mask=Mask32x32(buffer_mask)
            offset=offset+132
            self.Mask.append(mask)

    def getStyle(self, cls_params):
        style_brush=[]
        for i in range(self.countMask):
            color=getColor(self.Mask[i].ColorRGBA,cls_params)
            style_brush.append("ZBrush (7," +str(color)+")")
        return  ";".join(style_brush)
    @property
    def isValid(self):
        return self.__valid
class MultiSquareMark:
    __struct = struct.Struct('< H B B')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid = False

        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.FillGridMask = data[0]
        self.FillOnlySymbolsIncludesInArea = 1
        self.SymbolBorderIsThin =0
        self.Symbol=MultiMark(self.__bytes[self.__struct.size:len(self.__bytes)])
        self.__valid = True
        self.lenRecord=self.Symbol.lenRecord+self.__struct.size

    def getStyle(self, cls_params):
        return self.Symbol.getStyle(cls_params)
    @property
    def isValid(self):
        return self.__valid

class FillHatch:
    __struct = struct.Struct('< I I I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.ColorRGBA = None
        self.__load()



    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.AngleIsDegrees = data[1]
        self.StepInMicrones = data[2]
        step_line=sizePt(self.StepInMicrones)
        type_j= data[3]
        self.SymbolLineParameters=[]
        if type_j==128:
            count_line=int((self.lenRecord-16)/8)

            offset=self.__struct.size
            for i in range(count_line):
                buff_line = self.__bytes[offset:self.lenRecord]
                cls_line=PanLine(buff_line)
                offset=offset+cls_line.lenRecord+4
                width_line=sizePt(cls_line.WidthInMicrones)
                self.SymbolLineParameters.append(cls_line)
        else:
            print("err 153")
        self.__valid=True
    def getStyle(self,cls_params):
        color_line=self.SymbolLineParameters[0].ColorRGBA
        color = getColor(color_line, cls_params)
        step=int( sizePt(self.StepInMicrones))
        width_line=sizePt(self.SymbolLineParameters[0].WidthInMicrones)
        #brush_style_container
        image_brush=drawBrushFillHatch(step,self.AngleIsDegrees,width_line)
        id_brush=brush_style_container.addFile(image_brush)
        brush_style="Brush ("+str(id_brush)+","+str(color)+")"
        return brush_style
        '''
        if self.AngleIsDegrees==90:
            if step>=3 and step<=5:
                return "Brush (26,"+str(color)+")"
            if step<3:
                return "Brush (27," + str(color) + ")"
            return "Brush (4," + str(color) + ")"
        return "Brush (6,"+ str(color) + ")"
        '''
    @property
    def isValid(self):
        return self.__valid
class PanDash(ILine):
    __struct = struct.Struct('< I I I I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.ColorRGBA=None
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord =data[0]
        self.__count=data[1]
        self.last=data[2]
        self.border = data[3]
        self.__reserv=data[4]
        #buffer_section=buffer[self.__struct.size:self.__struct.size+self.lenRecord-16]
        self.sections=[]
        offset=self.__struct.size
        for i in range(self.__count):
            cls_selection=PanSection(self.__bytes,offset)
            offset=offset+cls_selection.lenRecord
            self.sections.append(cls_selection)
        self.__valid=True
    def getStyle(self,cls_params):
        if self.ColorRGBA is None:
            return "Pen (1, 40, 0)"
        color_rgba = self.ColorRGBA
        color = getColor(color_rgba, cls_params)
        #size_pt = sizePt(self.WidthInMicrones)

        #mi_size_pen = 10 + size_pt
        size_pen=size_pen=sizePan(self.WidthInMicrones,True)
        temp_string = "Pen(" + str(size_pen) + ",40," + str(color) + ")"
        return temp_string
    def getParams(self,db=None,type_obj=None):
        result_params = {}

        #result_params['type'] ="pen"
        result_params['type'] = 158
        mi_style=self.getStyle(db)
        result_params['image']=mi_style
        return result_params

    @property
    def isValid(self):
        return self.__valid
class PanDot(ILine):
    __struct = struct.Struct('< I I I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.__load()

    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = self.__struct.size
        self.ColorRGBA = data[0]
        self.WidthInMicrones = data[1]
        self.StrokeWidthInMicrones = data[2]
        self.SpaceWidthInMicrones = data[3]

        self.__valid = True
    def getStyle(self,cls_params):
        color_rgba = self.ColorRGBA
        color = getColor(color_rgba, cls_params)
        '''
        size_pt = sizePt(self.WidthInMicrones)

        mi_size_pen = 10 + size_pt
        mi_size_pen_pix = int((size_pt*72)/96)
        if mi_size_pen_pix>7:
            # Размер в pt
            size_pen=int(size_pt * 10 + 0.5)+10
        else:
            size_pen=mi_size_pen
        '''
        size_pen=sizePan(self.WidthInMicrones,True)
        mi_color = color.red() * 65536 + color.green() * 256 + color.blue()
        temp_string = "Pen(" + str(size_pen) + ",9," + str(mi_color) + ")"
        return temp_string
    def getParams(self,db=None):
        result_params = {}

        #result_params['type'] ="pen"
        result_params['type'] = 129
        mi_style=self.getStyle(db)
        result_params['image']=mi_style
        return result_params
    @property
    def isValid(self):
        return self.__valid


class PanSquare:
    __struct = struct.Struct('< I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.__load()

    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = self.__struct.size
        self.ColorRGBA = data[0]
        self.__valid = True

    @property
    def isValid(self):
        return self.__valid

class PanLine(ILine):
    ''' код 128'''
    __struct = struct.Struct('< I I')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord=self.__struct.size
        self.ColorRGBA=data[0]
        self.WidthInMicrones=data[1]
        self.__valid = True
    def getStyle(self,cls_params):
        color_rgba = self.ColorRGBA
        color = getColor(color_rgba, cls_params)
        size_pen=sizePan(self.WidthInMicrones)
        '''
        size_pt = sizePt(self.WidthInMicrones)

        mi_size_pen = 10 + size_pt

        mi_size_pen_pix = int((size_pt * 72) / 96 + 0.5)
        if mi_size_pen_pix>7:
            # Размер в pt
            size_pen=int(size_pt * 10 + 0.5)+10
        else:
            size_pen=mi_size_pen
        '''
        mi_color=color.red()*65536+color.green()* 256+color.blue()
        temp_string = "Pen(" + str(size_pen) + ",2," + str(mi_color) + ")"
        return temp_string
    def getParams(self,db=None,type_object=None):
        result_params = {}

        #result_params['type'] ="pen"
        result_params['type'] = 128
        mi_style=self.getStyle(db)
        result_params['image']=mi_style
        return result_params

    @property
    def isValid(self):
        return self.__valid
class PanImgDecorate:
    __struct = struct.Struct('< i i i i i i i i')
    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord=data[0]
        self.BeginLineMarkerLength=data[1]/250
        self.EndLineMarkerLength=data[2]/250
        self.BeginSegmentMarkerLength=data[3]/250
        self.EndSegmentMarkerLength=data[4]/250
        self.FillMarker1Length=data[5]/250
        self.FillMarker2Length=data[6]/250

        self.UnformalPlace=data[7]>0
        offset=self.__struct.size
        #bytes_line=self.__bytes[self.__struct.size:len(self.__bytes)-self.__struct.size]
        self.LineSymbol=factorySymbol(self.__bytes,offset)
        offset=offset+self.LineSymbol.lenRecord+4
        self.BeginLineMarker=factorySymbol(self.__bytes,offset)
        offset = offset+self.BeginLineMarker.lenRecord+4
        self.EndLineMarker=factorySymbol(self.__bytes,offset)
        offset = offset+self.EndLineMarker.lenRecord+4
        self.BeginSegmentMarker=factorySymbol(self.__bytes,offset)
        offset = offset + self.BeginSegmentMarker.lenRecord+4
        self.EndSegmentMarker=factorySymbol(self.__bytes,offset)
        offset = offset + self.EndSegmentMarker.lenRecord+4
        self.FillMarker1=factorySymbol(self.__bytes,offset)
        offset = offset + self.FillMarker1.lenRecord+4
        self.FillMarker2 = factorySymbol(self.__bytes, offset)
        self.__valid=True

    @property
    def isValid(self):
        return self.__valid




class PanHatchSquareShift:
    __struct = struct.Struct('< i i i i i')

    def __init__(self, bytes):
        self.__bytes = bytes
        self.__valid = False
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.Angle=data[1]
        self.Step=data[2]
        type_obj=data[3]
        self.Shift=data[4]
        self.Line=factorySymbol(self.__bytes[self.__struct.size:len(self.__bytes)],0,type_obj)
        self.__valid = True

    @property
    def isValid(self):
        return self.__valid


class PanEmpty:
    def __init__(self,bytes):
        jkl=0
        pass
    @property
    def lenRecord(self):
        return 0
class PanImage:
    ''' код 171'''
    __struct=struct.Struct('< I I I I I I I I')
    __struct_add = struct.Struct('< B B B B B 15s')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False
        self.__load()

    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord=data[0]
        self.Height=data[1]
        self.Width=data[2]
        self.PosH=data[3]
        self.PosV=data[4]
        self.Shift=data[5]
        self.PixelHeight=data[6]
        self.PixelWidth=data[7]
        self.__valid=True
        self.Contour=factorySymbol(self.__bytes,self.__struct.size)
        offse=0
        if self.Contour is not None and self.Contour.isValid:
            offse=self.Contour.lenRecord

        offse=offse+self.__struct.size
        buffer=self.__bytes[offse:offse+self.__struct_add.size]
        data = self.__struct_add.unpack(buffer)
        self.ContourFlag=data[0]
        self.ShadeFlag=data[1]
        self.TransparentFlag=data[2]
        self.Type=data[3]
        self.Rotate=data[4]
        offse=offse+self.__struct_add.size
        buffer_image=self.__bytes[offse:len(self.__bytes)]
        self.PictureData=getPng(buffer_image)
        return

    @property
    def isValid(self):
        return self.__valid
    def getParams(self,add_parm=None):
        result_params={}
        result_params['height']=self.Height
        result_params['width'] = self.Width
        result_params['pos_h'] = self.PosH
        result_params['pos_v'] = self.PosV
        result_params['shift'] = self.Shift
        result_params['pixel_h'] = self.PixelHeight
        result_params['pixel_w'] = self.PixelWidth
        result_params['transparent_flag'] = self.TransparentFlag
        result_params['type'] = self.Type
        result_params['rotate'] = self.Rotate
        if self.PixelHeight==0 or self.PixelWidth==0:
            result_params['image'] = None
        else:
            result_params['image'] = self.PictureData
        return result_params


        return
class PanLineShift:
    '''код 159 '''
    __struct = struct.Struct('< i i')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False
        self.__load()
    def __load(self):
        buffer = self.__bytes[0:self.__struct.size]
        data = self.__struct.unpack(buffer)
        self.lenRecord = data[0]
        self.shift=data[1]
        self.symbol=factorySymbol(self.__bytes,self.__struct.size)
        self.__valid = True
    @property
    def isValid(self):
        return self.__valid

class PanLibSymbol:
    ''' код 250'''
    __struct=struct.Struct('< I I I I')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False
        self.__load()
    def __load(self):
        buffer=self.__bytes[0:self.__struct.size]
        data=self.__struct.unpack(buffer)
        self.lenRecord=data[0]
        self.code=data[1]
        self.index=data[2]
        self.function=data[3]
        temp_bytes=self.__bytes[self.__struct.size:self.__struct.size+self.lenRecord]
        self.symbol=PanDrawSymbol(temp_bytes)
        self.__valid=True
        return
    @property
    def isValid(self):
        return self.__valid

    def getParams(self,params_add=None):
        if self.symbol is None:
            return None
        return self.symbol.getParams()
class PanImgSymbol:
    ''' код 147'''
    __struct_symbol = struct.Struct('< I I I')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False

        self.__libSymbols = None
        self.__load()
    def __load(self):
        size=self.__struct_symbol.size

        data = self.__struct_symbol.unpack(self.__bytes[0:size])
        self.__lenRecord = data[1]
        self.__count=data[2]
        offset=size
        self.__libSymbols=[]
        for i in range(self.__count):
            csl_sym=factorySymbol(self.__bytes,offset)
            offset=offset+csl_sym.lenRecord+4
            self.__libSymbols.append(csl_sym)
        self.__valid = True

    @property
    def Symbols(self):
        return self.__libSymbols
class PanDotShift:
    __struct_symbol = struct.Struct('< I I I I i')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False
        self.__libSymbols=None
        if bytes is not None:
            self.__load()
    def __load(self):
        size = self.__struct_symbol.size

        data = self.__struct_symbol.unpack(self.__bytes[0:size])
        self.ColorRGBA=data[0]
        self.WidthInMicrones=data[1]
        self.StrokeWidthInMicrones=data[2]
        self.SpaceWidthInMicrones=data[3]
        self.OffsetInMicrones=data[4]
        self.lenRecord=size
        self.__valid = True

    @property
    def isValid(self):
        return self.__valid
class PanDrawSymbol:
    ''' код 147'''
    __struct_symbol = struct.Struct('< 4s I I')
    def __init__(self,bytes):
        self.__bytes=bytes
        self.__valid=False
        self.__libSymbols=None
        if bytes is not None:
            self.__load()
    def setSymbols(self,symbols):
        self.__libSymbols=symbols
    def __load(self):
        size=self.__struct_symbol.size
        size_bytes=len(self.__bytes)
        if size_bytes<size_bytes:
            return
        data =self.__struct_symbol.unpack(self.__bytes[0:size])
        key_word=data[0]
        if not (key_word[0]==254 and key_word[1]==127 and key_word[2]==255  and key_word[3]==127):
            return
        self.__all_len_data=data[1]
        self.__count_primetic=data[2]
        if self.__count_primetic<=0:
            return
        self.__valid = True
        offset=size
        self.__libSymbols=[]
        for i in range(self.__count_primetic):
            cls_symbol=factorySymbol(self.__bytes,offset)
            if cls_symbol is not None and cls_symbol.isValid:
                self.__libSymbols.append(cls_symbol)
            else:
                jkl=0
                continue
            offset=offset+cls_symbol.lenRecord+4
            jkl=0
    @property
    def Symbols(self):
        return self.__libSymbols
    def getParams(self,add_params=None,type_object=None):
        result_params=[]
        for item_sym in self.__libSymbols:
            result_params.append(item_sym.getParams(add_params))
        return result_params
def factorySymbol(bytes,offset,code_symbol_source=None):
    if code_symbol_source is  None:
        __struct_symbol = struct.Struct('< H H')
        buffer=bytes[offset:offset+__struct_symbol.size]
        data=__struct_symbol.unpack(buffer)
        len_att_symbol=data[0]
        code_sumbol=data[1]
        buffer=bytes[offset+__struct_symbol.size:offset+__struct_symbol.size+len_att_symbol]
    else:
        code_sumbol=code_symbol_source
        buffer=bytes
    print("Code Symboll:"+str(code_sumbol))
    if code_sumbol == 127:
        return PanEmpty(buffer)
    if code_sumbol==128:
        ''' линия'''
        return PanLine(buffer)
    if code_sumbol==129:
        ''' линия пунктирная'''
        return PanDot(buffer)
    if code_sumbol==135:
        ''' квадрат'''
        return PanSquare(buffer)
    if code_sumbol==144:
        #return  PanImgSymbol(buffer)
        return MultiSquareMark(buffer)
    if code_sumbol==147:
        #return  PanImgSymbol(buffer)
        return PanDrawSymbol(buffer)
    if code_sumbol==148:
        #return  PanImgSymbol(buffer)
        return PanDotShift(buffer)
    if code_sumbol==149:

        return VectorEx(buffer)
    if code_sumbol == 153:
        return FillHatch(buffer)
    if code_sumbol==155:
        return SquareVector(buffer)
    if code_sumbol == 157:
        return PanImgDecorate(buffer)
    if code_sumbol==158:
        #return  PanImgSymbol(buffer)
        return PanDash(buffer)
    if code_sumbol==159:
        #return  PanImgSymbol(buffer)
        return PanLineShift(buffer)
    if code_sumbol==166:

        return PanHatchSquareShift(buffer)
    if code_sumbol==250:
        ''' пользовательский тип примитива'''
        return PanLibSymbol(buffer)
    if code_sumbol==171:
        return PanImage(buffer)
    if code_sumbol!=61440:
        print("Not find class")
    return None