import struct

from loguru import logger
def fullNamesListToListRow(list_value):
    rows=[]
    for item in list_value:
        row=(item['code'],item['name'])
        rows.append(row)
    return rows
def getString(buffer,offset):
    index_end=buffer.find(b'\x00\x00',offset)
    len=index_end-offset
    if not(len % 2 ==0):
        len=len+1

    buff_string=buffer[offset:offset+len]
    try:
        str_value=buff_string.decode("utf-16")
    except:
        jkl=0

    return str_value,len
class InfoByFullNameTable:
    __base_struct_record=struct.Struct('< I I')
    def __init__(self,bytes):
        self.__bytes_source=bytes
        self.__offset = self.__bytes_source.find(b'FNM')
        if self.__offset>=0:
            self.__offset=self.__offset+4
        else:
            return
        self.__list_info_by_table=[]
        self.__getInfoAboutTables()
    def __getInfoAboutTables(self):
        offset_start=self.__offset
        offset_by_table=64
        for i in range(4):
            buffer = self.__bytes_source[offset_start+i*8:offset_start + (i+1)*8]
            data = self.__base_struct_record.unpack(buffer)
            full_size = data[0]
            count_records = data[1]
            self.__list_info_by_table.append({'count':count_records,'size':full_size,'offset':offset_by_table})
            offset_by_table=offset_by_table+full_size
        return


    def getInfoFullNameObjects(self):
        buffer=self.__bytes_source[self.__offset:self.__offset+8]
        data=self.__base_struct_record.unpack(buffer)
        full_size=data[0]
        count_records=data[1]
        return full_size,count_records
    @property
    def isValid(self):
        if self.__offset>0:
            return True
        return False
    def getTableFullName(self,id_table):
        table_values=[]
        info_by_table=self.__list_info_by_table[id_table]
        start_offset=self.__offset+info_by_table['offset']+4
        for i in range(info_by_table['count']):

            buffer_number=self.__bytes_source[start_offset:start_offset+4]
            num_id=int.from_bytes(buffer_number, "little")

            name,len=getString(self.__bytes_source,start_offset+4)

            start_offset=start_offset+4+len+6
            table_values.append({'code':num_id,"name":name})
        return table_values
    def saveToDb(self,db_cls,number_table=None):
        count_tab=len(self.__list_info_by_table)
        if count_tab==0:
            logger.error("no information available about table full names")
            return
        if number_table is not None:
            return self.__saveTableToDb(db_cls,number_table)
        for i in range(count_tab):
            self.__saveTableToDb(db_cls, i)
    def __saveTableToDb(self,db_cls,number_table):
        sql_0 = "Insert into full_names_objects (code_object,name) VALUES(?,?)"
        sql_1 = "Insert into full_names_semantic (code_object,name) VALUES(?,?)"
        sql_2 = "Insert into full_names_layers (code_object,name) VALUES(?,?)"
        sql_3 = "Insert into full_names_classificator (code_object,name) VALUES(?,?)"
        required_number_record=0
        if number_table==0:
            sql=sql_0
            sql_count_record="select Count(*) from objects"

        if number_table==1:
            sql=sql_1
            sql_count_record = "select Count(*) from semantic_attribute"
        if number_table==2:
            sql=sql_2
            sql_count_record = "select Count(*) from layers"
        if number_table==3:
            sql=sql_3
            sql_count_record = "select Count(*) from classificator_values"
        if number_table>3:
            logger.error("Save to Db FullNames . Number table >3")
            return
        rows = db_cls.select(sql_count_record)
        required_number_record = rows[0][0]
        data_tab_obj_names = self.getTableFullName(number_table)
        if len(data_tab_obj_names)<required_number_record:
            if number_table==0:
                sql ="Insert into full_names_objects select inner_code as code_object ,name from objects"

            if number_table==1:
                sql = "Insert into full_names_semantic select code as code_object ,name from semantic_attribute"
            if number_table == 2:
                sql = "Insert into full_names_layers select id_layer,name from layers"
            if number_table == 3:
                sql = "Insert into full_names_classificator  select code,name from classificator_values"
            db_cls.runSqlNoResult(sql)
            return
        if len(data_tab_obj_names)>0:
            records = fullNamesListToListRow(data_tab_obj_names)
            logger.info("Save to Db FullNames table id:"+str(number_table))
            db_cls.insertMany(sql, records)
    @staticmethod
    def factory(bytes_rsc):
        cls_fullNameTabs=InfoByFullNameTable(bytes_rsc)
        if cls_fullNameTabs.isValid:
            return cls_fullNameTabs
        return None
