import struct

from PySide2.QtGui import QColor
from loguru import logger

#from MyTask.Panorama.RSC.cls_rsc import getString
from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.rsc_tools.RSC.cls_rsc import getString


class Palette:
    def __init__(self,name,colors):
        self.__name=name
        self.__colors=colors
    @property
    def Name(self):
        return self.__name
    @property
    def colors(self):
        return self.__colors
    def getColotByIndex(self,id_color):
        return self.__colors[id_color]
    @staticmethod
    def factory(bytes,offset):
        colors = []
        count_color = 256
        struct_color = struct.Struct('< B B B B')
        for i in range(count_color):
            curent_offset=offset+i*struct_color.size
            buf=bytes[curent_offset:curent_offset+struct_color.size]
            data=struct_color.unpack(buf)
            colors.append(data)
        curent_offset = curent_offset+ struct_color.size
        buf_name=bytes[curent_offset:curent_offset+32]
        name_pal=getString(buf_name)
        return Palette(name_pal,colors)

class Palettes:
    def __init__(self):
        self.palettes=[]
    def add(self,palette):
        self.palettes.append(palette)




    @staticmethod
    def factory(bytes,offset,count_palette):
        '''
        struct_color=struct.Struct('< B B B B')
        colors=[]
        count_color=256

        for i in range(count_color):
            curent_offset=offset+i*struct_color.size
            buf=bytes[curent_offset:curent_offset+struct_color.size]
            data=struct_color.unpack(buf)
            colors.append(data)
        curent_offset = curent_offset+ struct_color.size
        buf_name=bytes[curent_offset:curent_offset+32]
        name_pal=getString(buf_name)
        '''
        palettes=Palettes()

        for i in range(count_palette):
            cur_offset =offset+i*(256*4+32)
            palette=Palette.factory(bytes,cur_offset)
            palettes.add(palette)
        return palettes

class RscPrintPrametrs:
    def __init__(self):
        self.__pallets=None
        self.count_colors=None
    def setPalette(self,cls_pallet):

        self.__pallets=cls_pallet
    def getPallets(self,id_pal):
        return self.__pallets[id_pal]
    def getColor(self,id_pal,id_color):
        pal= self.__pallets[id_pal]
        color=pal.getColotByIndex(id_color)
        return color
    def saveToDb(self,parent_cls,db):
        sql="CREATE TABLE [palettes_dictionary](  [id] INTEGER PRIMARY KEY,  [name] TEXT)"
        db.runSqlNoResult(sql)
        sql="CREATE TABLE [palettes](  [id_pal] INT REFERENCES [palettes_dictionary]([id]),    [red] SMALLINT,  [green] SMALLINT,  [blue] SMALLINT,[alpha] SMALLINT)"
        db.runSqlNoResult(sql)
        logger.debug("Start save palette")
        if parent_cls is not None:
            parent_cls.send_status.emit("Start save palette")
        count_pal=len(self.__pallets)
        for i in range(count_pal):
            pal=self.__pallets[i]
            name_pal=pal.Name
            if parent_cls is not None:
                parent_cls.send_status.emit("Save palette:"+name_pal)
            sql_insert="Insert into palettes_dictionary (id,name) Values("+str(i)+",'"+name_pal+"')"

            db.runSqlNoResult(sql_insert)
            sql_insert = "Insert Into palettes (id_pal,red,green,blue,alpha) VALUES(?,?,?,?,?)"
            rows=[]
            count_colors=self.count_colors
            if count_colors is None:
                count_colors=len(pal.colors)
            #for color in pal.colors:
            for index in range( count_colors):
                color=pal.colors[index]
                rows.append((i,color[0],color[1],color[2],color[3]))
            db.insertMany(sql_insert, rows)


def factoryRscPrintPrametrs(bytes):
    #offset = bytes.find(b'PAL')
    __struct=struct.Struct('< I I I')
    offset=216
    ''' Число таблиц'''
    count_table=6
    list_tables_info=[]
    for i in range(count_table):

        buf=bytes[offset+__struct.size*i:offset+__struct.size*(i+1)]
        data =__struct.unpack(buf)
        info_tab={'offset':data[0],'size':data[1],'count':data[2]}
        list_tables_info.append(info_tab)
    __struct_add = struct.Struct('< B B 30s I I')
    offset=offset+__struct.size*count_table
    buf=bytes[offset:offset+__struct_add.size]
    data=__struct_add.unpack(buf)
    pallet_colors=data[4]
    palettes=Palettes.factory(bytes,list_tables_info[1]['offset'],list_tables_info[1]['count'])
    cls_printParams=RscPrintPrametrs()
    cls_printParams.count_colors=pallet_colors
    cls_printParams.setPalette(palettes.palettes)
    return cls_printParams
def readPaletteFromDb(path_db,id_pal=0):
    db=DbRsc(path_db)
    sql="select * from palettes where id_pal="+str(id_pal)+" order by rowid"
    rows=db.select(sql)
    palette=[]
    for row in rows:
        color=QColor(row[1],row[2],row[3])
        palette.append(color)
    db.close(False)
    return palette

