from PySide2.QtGui import QColor
from loguru import logger

from PanoramaRsc.ui.add_tools.rsc_tools.RSC.panSymbol import PanLineShift, PanDash, VectorEx, ImgChainType, PanLine, \
    PanDot, PanDrawSymbol, PanDotShift
from PanoramaRsc.ui.add_tools.rsc_tools.panoramaToAxiPen.factoryPen.penFactory import MainLine, WidthElement, TypeUnit, \
    AxiPenPatern, ContinuousMarker, numberToString, factoryСircle

def polygonInPath(points,coef=1000):
    str_pth=""
    count=len(points)
    for i,pnt in enumerate(points):
        if i==0:
            str_pth=str_pth+"M "+numberToString(pnt[0]/coef)+" "+numberToString(pnt[1]/coef)
        else:
            str_pth = str_pth + " L " + numberToString(pnt[0] / coef) + " " + numberToString(pnt[1] / coef)
    return str_pth
def getColor(color,palette):
    if isinstance(color,int):
        b_color=color.to_bytes(4, 'big')
    else:
        b_color=color[::-1]
    if b_color[0]==240:
        index_color=b_color[3]

        return palette[index_color]
    return None
def vectorLine(len_sym,symbol,palette):
    sub_item_continuousMarker = ContinuousMarker()
    sub_item_continuousMarker.defaultWidth = WidthElement(TypeUnit.mm, symbol.WidthInMicrones/1000)
    sub_item_continuousMarker.path="M 0 0 L "+numberToString(len_sym/1000)+" 0 "
    sub_item_continuousMarker.useStyleWidth=False
    sub_item_continuousMarker.bend=True
    color=getColor(symbol.ColorRGBA,palette)
    sub_item_continuousMarker.defaultColor=color
    sub_item_continuousMarker.useStyleColor=False
    return sub_item_continuousMarker,len_sym

def ImageFragmentsToArea(obj_vect,width,height,palette):
    sub_item_continuousMarker = ContinuousMarker()
    if obj_vect.ParamSymbol.needPalette:
        color = getColor(int.from_bytes(obj_vect.ParamSymbol.ColorRGBA, "little"), palette)
        sub_item_continuousMarker.defaultColor = QColor(color)
    else:
        sub_item_continuousMarker.defaultColor = QColor(obj_vect.pen_color)
    points_poly=[]
    #width_coord=obj_vect.ChainCoordsSize-1
    w_p=obj_vect.pathPoints.MaxX-obj_vect.pathPoints.MinX
    h_p=obj_vect.pathPoints.MaxY-obj_vect.pathPoints.MinY
    if width==0:
        coef_x = w_p / height
    else:
        coef_x=w_p/width
    coef_y=h_p/height
    for pt in obj_vect.ChainCoordsXY:
        new_x=(pt[0]/coef_x)
        new_y = (pt[1] / coef_y)
        points_poly.append((new_x,new_y))
    sub_item_continuousMarker.path=polygonInPath(points_poly)
    sub_item_continuousMarker.fill=True
    sub_item_continuousMarker.bend=True
    return sub_item_continuousMarker

def ImageFragmentsToCircle(obj_vect,width,height,palette):
    sub_item_continuousMarker = ContinuousMarker()
    if obj_vect.ParamSymbol.needPalette:
        color=getColor(int.from_bytes(obj_vect.ParamSymbol.ColorRGBA, "little") ,palette)
        sub_item_continuousMarker.defaultColor = QColor(color)
    else:
        sub_item_continuousMarker.defaultColor = QColor(obj_vect.pen_color)
    radius=(width/2)/1000
    sub_item_continuousMarker.path=factoryСircle(radius)
    sub_item_continuousMarker.defaultWidth=WidthElement(TypeUnit.mm,radius*2)
    sub_item_continuousMarker.bend=True
    sub_item_continuousMarker.fill=True
    return sub_item_continuousMarker
def ImageFragmentsLineToPen(obj_vect,width,height,palette):
    sub_item_continuousMarker=ContinuousMarker()
    sub_item_continuousMarker.defaultColor=QColor(obj_vect.pen_color)
    width_line=obj_vect.pathPoints.MaxX-obj_vect.pathPoints.MinX
    height_line=obj_vect.pathPoints.MaxY-obj_vect.pathPoints.MinY
    sign = None
    #if width==0 or width_line:
    if  width_line==0.0:
        coef_x=0
    else:
        coef_x=width/width_line
    if height_line==0.0:
        coef_y=0

        if obj_vect.pathPoints.MaxY<0 and obj_vect.pathPoints.MinY<0:
            sign=-1
        if obj_vect.pathPoints.MaxY > 0 and obj_vect.pathPoints.MinY > 0:
            sign = 1

    else:
        coef_y=height/height_line
    p1_x=obj_vect.ChainCoordsXY[0][0]*coef_x
    p1_y = obj_vect.ChainCoordsXY[0][1] * coef_y
    p2_x = obj_vect.ChainCoordsXY[1][0] * coef_x
    p2_y = obj_vect.ChainCoordsXY[1][1] * coef_y
    path="M "+numberToString(p1_x/1000)+" "+numberToString(p1_y/1000)+" L "+numberToString(p2_x/1000)+" "+numberToString(p2_y/1000)
    sub_item_continuousMarker.path=path
    if sign is not None:
        sub_item_continuousMarker.offset=sign*height/1000
    return sub_item_continuousMarker
def panLineToMainLine(pan_sym,shift,palette):
    obj_pen=MainLine()
    obj_pen.offset=-(shift/1000)
    obj_pen.defaultColor=getColor(pan_sym.ColorRGBA,palette)
    if obj_pen.defaultColor is None:
        logger.error('Not find color')
        return None
    obj_pen.defaultWidth=WidthElement(TypeUnit.mm,pan_sym.WidthInMicrones/1000)
    obj_pen.useStyleWidth=False
    obj_pen.useStyleColor=False
    return obj_pen
def panDot(pan_sym,shift,palette):
    sub_object=ContinuousMarker()
    sub_object.bend=True
    sub_object.enabled=True
    sub_object.defaultWidth=WidthElement(TypeUnit.mm,pan_sym.WidthInMicrones/1000)
    sub_object.defaultColor=getColor(pan_sym.ColorRGBA,palette)
    sub_object.useStyleColor=False
    sub_object.path="M 0 0 L "+numberToString(pan_sym.StrokeWidthInMicrones/1000)+" 0 "
    sub_object.dashPattern.append(pan_sym.SpaceWidthInMicrones/1000)
    sub_object.useStyleWidth=False
    return sub_object
def vectorExtToPen(symbol:VectorEx,palette):
    jkl=0
    if len(symbol.Fragments)==1 and symbol.HSize==0 and symbol.Fragments[0].ChainType==ImgChainType.Line:
        sub_item_pen=ImageFragmentsLineToPen(symbol.Fragments[0],symbol.HSize,symbol.VSize,palette)
        sub_item_pen.useStyleWidth=False
        sub_item_pen.useStyleColor=False
        sub_item_pen.bend=True
        return sub_item_pen,0
    if len(symbol.Fragments)==1 and symbol.Fragments[0].ChainType==ImgChainType.Circle:
        sub_item_pen=ImageFragmentsToCircle(symbol.Fragments[0],symbol.HSize,symbol.VSize,palette)
        sub_item_pen.useStyleWidth = False
        sub_item_pen.useStyleColor = False
        return sub_item_pen, symbol.HSize
    if len(symbol.Fragments) == 1 and symbol.Fragments[0].ChainType == ImgChainType.Area:
        sub_item_pen=ImageFragmentsToArea(symbol.Fragments[0],symbol.HSize,symbol.VSize,palette)
        sub_item_pen.useStyleWidth = False
        sub_item_pen.useStyleColor = False
        return sub_item_pen, symbol.HSize
    print("Multi Fragments")
    sub_item_pen=[]
    size_item=[]
    for fragment in symbol.Fragments:
        if fragment.ChainType == ImgChainType.Area:
            frag_item=ImageFragmentsToArea(fragment,symbol.HSize,symbol.VSize,palette)
            frag_item.useStyleWidth = False
            frag_item.useStyleColor = False
            sub_item_pen.append(frag_item)
            size_item.append(0)
            continue
        if fragment.ChainType==ImgChainType.Line and len(fragment.ChainCoordsXY)>2:
            if fragment.ParamSymbol.ColorRGBA==0:
                fragment.ParamSymbol.needPalette=False
            frag_item = ImageFragmentsToArea(fragment, symbol.HSize, symbol.VSize, palette)
            frag_item.useStyleWidth = False
            frag_item.useStyleColor = False
            frag_item.fill=False
            sub_item_pen.append(frag_item)
            size_item.append(0)
            continue
        if fragment.ChainType==ImgChainType.Line and len(fragment.ChainCoordsXY)==2:
            if fragment.ParamSymbol.ColorRGBA==0:
                fragment.ParamSymbol.needPalette=False
            frag_item = ImageFragmentsLineToPen(fragment, symbol.HSize, symbol.VSize, palette)
            frag_item.useStyleWidth = False
            frag_item.useStyleColor = False
            frag_item.fill = False
            sub_item_pen.append(frag_item)
            size_item.append(0)
            continue

        print("No find fragment")
        jkl=0
    return sub_item_pen,size_item
def dashToPen(symbol_dash:PanDash,palette):
    sub_item_pen=[]
    len_obj_section=[]
    ''' собираем размеры секций'''
    common_len=0
    for section in symbol_dash.sections:

        symbol=section.Sybol
        len_obj=section.Base
        if isinstance(symbol,VectorEx):
            len_obj=symbol.HSize
        len_obj_section.append(len_obj)
        common_len=common_len+len_obj
    len_offset=0
    feirstLine=False
    for i,section in enumerate(symbol_dash.sections):
        sub_item=None
        symbol = section.Sybol
        len_offset=len_offset+len_obj_section[i]
        if isinstance(symbol,VectorEx):
            sub_item,len_segments=vectorExtToPen(symbol,palette)

        if isinstance(symbol,PanLine):
            sub_item,len_segments=vectorLine(section.Base,symbol,palette)
            len_offset = len_offset - len_obj_section[i]
        if sub_item is None:


            continue
        if isinstance(sub_item,list):
            for index,item in enumerate(sub_item):
                size_item=len_segments[index]
                if len_offset == 0:
                    item.dashOffset = 0
                else:
                    item.dashOffset = (common_len - len_offset+len_obj_section[i]) / 1000
                item.dashPattern.append((common_len -len_obj_section[i])/ 1000)
                sub_item_pen.append(item)
            feirstLine = True
            continue
        if len_offset==0:
            sub_item.dashOffset=0
        else:
            sub_item.dashOffset=(common_len-len_offset)/1000
        if len_segments==0:
            sub_item.dashPattern.append(common_len/1000)
        else:
            sub_item.dashPattern.append((common_len -len_segments)/ 1000)
            if isinstance(symbol, PanLine):

                if len_offset == 0:
                    sub_item.dashOffset = 0
                else:
                    if feirstLine:
                        sub_item.dashOffset = (common_len - len_offset  ) / 1000
                    else:

                        sub_item.dashOffset = (common_len - len_offset - len_segments ) / 1000
                len_offset = len_offset + len_obj_section[i]
            else:
                sub_item.dashOffset = (common_len - len_offset+len_segments/2) / 1000
        sub_item_pen.append(sub_item)



        continue
        jkl=0
    return sub_item_pen
def convertPanDrawSymbolToAxiPen(panSymbol_source,palette):
    if panSymbol_source is None:
        return None
    panSymbol=panSymbol_source
    if not isinstance(panSymbol_source,PanDrawSymbol):
        panSymbol=PanDrawSymbol(None)
        panSymbol.setSymbols([panSymbol_source])
    pen_axi=AxiPenPatern()
    for symbol in panSymbol.Symbols:

        pens_item=None
        if isinstance(symbol,PanDotShift):
            pens_item = panDot(symbol, 0, palette)
            pens_item.offset=symbol.OffsetInMicrones/1000
            if pens_item is not None:
                pen_axi.add(pens_item)
            continue
        if isinstance(symbol,PanDot):
            pens_item = panDot(symbol, 0, palette)
            if pens_item is not None:
                pen_axi.add(pens_item)
            continue

        if isinstance(symbol,PanLineShift):
            shift=symbol.shift
            pens_item=panLineToMainLine(symbol.symbol,shift,palette)
            if pens_item is not None:
                pen_axi.add(pens_item)
            continue
        if isinstance(symbol, PanLine):
            pens_item = panLineToMainLine(symbol, 0, palette)
            if pens_item is not None:
                pen_axi.add(pens_item)
            continue
        if isinstance(symbol,PanDash):
            sub_item=dashToPen(symbol,palette)
            if sub_item is not None:
                pen_axi.add(sub_item)
            continue
        jkl=0
    return pen_axi