from enum import Enum
from PySide2.QtGui import QColor, Qt


def colorToString(color:QColor):
    out_str='\t'+'colorName : "'+color.name(QColor.HexArgb)+'"'
    return [out_str,'}']
def dashPatternToString(dashPattern):
    if len(dashPattern)==0:
        return None
    out_str=[]
    for dath in dashPattern:
        out_str.append('element :'+numberToString(dath))
    return out_str


def variableToLogical(value):
    if value:
        return 'true'
    return 'false'
def numberToString(value):
    if isinstance(value,int):
        return str(value)
    return "{:.2f}".format(value)
class TypeCapStyle(Enum):
    FlatCap=0
    SquareCap=16
    RoundCap=32

    def __str__(self):
        if self.value==0:
            return "FlatCap"
        if self.value==16:
            return "SquareCap"
        if self.value==32:
            return "RoundCap"

    @staticmethod
    def from_str(text):
        if text=="FlatCap":
            return TypeCapStyle.FlatCap
        if text=="SquareCap":
            return TypeCapStyle.SquareCap
        if text=="RoundCap":
            return TypeCapStyle.RoundCap
class TypeJoinStyle(Enum):
    MiterJoin=0
    BevelJoin=64
    RoundJoin=128
    def __str__(self):
        if self.value==0:
            return "MiterJoin"
        if self.value==64:
            return "BevelJoin"
        if self.value==128:
            return "RoundJoin"

    @staticmethod
    def from_str(text):
        if text == "MiterJoin":
            return TypeJoinStyle.MiterJoin
        if text == "BevelJoin":
            return TypeJoinStyle.BevelJoin
        if text == "RoundJoin":
            return TypeJoinStyle.RoundJoin
class TypeUnit(Enum):
    px=1
    mm=2
    cm=3
    pt=4
    inch=5

    def __str__(self):
        if self.value==1:
            return "px"
        if self.value==2:
            return 'mm'
        if self.value==3:
            return 'cm'
        if self.value==4:
            return 'pt'
        if self.value==5:
            return 'in'

    @staticmethod
    def from_str(text):
        if text == "mm":
            return TypeUnit.mm
        if text == "in":
            return TypeUnit.inch
        if text == "cm":
            return TypeUnit.cm
        if text == "px":
            return TypeUnit.px
def selectItemsList(list_source,start_index,end_index)->list:

    list_out=[]
    for i in range(start_index+1,end_index):
        line=list_source[i]
        if line[0]=='\t':
            line=line[1:]
        line=line.replace('\n','')
        list_out.append(line)
    return list_out
def factoryMainBlock(lines):
    pass

def factoryPenFromFile(path_file):
    line_trtp=[]
    with open(path_file, 'r') as file_pen:
        line_trtp=file_pen.readlines()
    if len(line_trtp)<=2:
        return None
    list_adr_blocks_main=[]
    start_block=None
    end_block=None
    for i,line in enumerate(line_trtp):
        if line.find('mapInfoSubPattern')==0:
            start_block=i
            continue
        if line[0]=='}':
            end_block=i
            list_adr_blocks_main.append({'start':start_block,'end':end_block})
    if len(list_adr_blocks_main)==0:
        return None
    for pos_main in list_adr_blocks_main:
        lines_main=selectItemsList(line_trtp,pos_main['start'],pos_main['end'])
        jkl=0
class WidthElement:
    def __init__(self,unit,value):
        self.unit=unit
        self.value=value
    def toString(self):
        out_str=[]
        if isinstance(self.value,int):
            out_str.append("value : "+str(self.value))
        else:
            out_str.append("value : " + "{:.2f}".format(self.value))
        out_str.append('unit : "'+str(self.unit)+'"')
        return out_str

class SubObject:
    def __init__(self):
        self.enabled=True
        self.useStyleColor=True
        self.defaultColor=QColor(0,0,0)
        self.useStyleWidth=True
        self.baseWidth=1
        self.defaultWidth=WidthElement(TypeUnit.px,1)
        self.opacity=1
        self.offset=0
        self.joinStyle=TypeJoinStyle.BevelJoin
        self.capStyle=TypeCapStyle.FlatCap
    def toString(self):
        our_string=[]
        our_string.append('enabled'+' : '+variableToLogical(self.enabled))
        our_string.append('useStyleColor' + ' : ' + variableToLogical(self.useStyleColor))
        our_string.append("defaultColor {")
        our_string.extend(colorToString(self.defaultColor))
        our_string.append('useStyleWidth' + ' : ' + variableToLogical(self.useStyleWidth))
        if isinstance(self.baseWidth,int):
            our_string.append('baseWidth' + ' : ' + str(self.baseWidth))
        else:
            our_string.append('baseWidth' + ' : ' + "{:.2f}".format(self.baseWidth))
        our_string.append('defaultWidth {')
        our_string.extend(self.defaultWidth.toString())
        our_string.append('}')
        our_string.append("opacity : "+numberToString(self.opacity))
        our_string.append("joinStyle {")
        our_string.append('"'+str(self.joinStyle)+'"')
        our_string.append("}")
        #our_string.append("opacity : " + numberToString(self.opacity))
        our_string.append("capStyle {")
        our_string.append('"' + str(self.capStyle) + '"')
        our_string.append("}")
        our_string.append('offset :' + numberToString(self.offset))
        return our_string

class ContinuousMarker(SubObject):
    def __init__(self):
        super().__init__()
        self.bend=True
        self.fill=False
        self.path=None
        self.dashPattern=[]
        self.dashOffset=0
        self.offset=0

    def toString(self):
        out_strs=[]
        out_strs.append("continuousMarker {")
        out_strs.extend(super(ContinuousMarker, self).toString())
        out_strs.append("fill : "+variableToLogical(self.fill))
        out_strs.append('path : "'+self.path+'"')
        out_strs.append('dashPattern {')
        dath_str=dashPatternToString(self.dashPattern)
        if dath_str is not None:
            out_strs.extend(dath_str)
        out_strs.append('}')
        out_strs.append('dashOffset :'+numberToString(self.dashOffset))

        out_strs.append("bend :"+variableToLogical(self.bend))
        out_strs.append("}")
        return out_strs
class MainLine(SubObject):
    def __init__(self):
        super().__init__()
        self.dashPattern=[]
        self.dashOffset = 0

    def toString(self):
        out_strs=[]
        out_strs.append("mainLine {")
        out_strs.extend(super(MainLine, self).toString())
        out_strs.append('dashPattern {')
        dath_str = dashPatternToString(self.dashPattern)
        if dath_str is not None:
            out_strs.extend(dath_str)
        out_strs.append('}')
        out_strs.append('dashOffset :' + numberToString(self.dashOffset))
        #out_strs.append('offset :' + numberToString(self.offset))
        out_strs.append("}")
        return out_strs
class MapInfoSubPattern:
    def __init__(self):
        self.__paterns=[]
    def add(self,patern_line):
        self.__paterns.append(patern_line)
    def toString(self):
        out_strings=[]
        if len(self.__paterns)==0:
            return None
        out_strings.append("mapInfoSubPattern {")
        for line_patern in self.__paterns:
            out_strings.extend(line_patern.toString())
        out_strings.append("}")
        return out_strings
    def getChildrenPaterns(self):
        return self.__paterns[0]

class AxiPenPatern:
    def __init__(self):
        #self.__mapInfoPatern=MapInfoSubPattern()
        self.__mapInfoPatern=[]
        self.__defaultWidth=[]
    def getMaxWidthMM(self):
        value_max=0
        for def_width in self.__defaultWidth:
            if def_width.unit==TypeUnit.mm:
                value_max=max(value_max,def_width.value)
            if def_width.unit==TypeUnit.px:
                value_max = max(value_max, def_width.value*0.2645833333)
        return value_max
    def add(self,patern,addmapInfoSubPattern=True):
        if isinstance(patern,list):
            for item in patern:
                self.__defaultWidth.append(item.defaultWidth)
                self.add(item)
            return
        if addmapInfoSubPattern:
            obj_mapInfo=MapInfoSubPattern()
            obj_mapInfo.add(patern)
            self.__mapInfoPatern.append(obj_mapInfo)
            self.__defaultWidth.append(patern.defaultWidth)
            return
        self.__mapInfoPatern.add(patern)
        self.__defaultWidth.append(patern.defaultWidth)
    def setUseStyleColor(self):
        colors=[]
        for main_patern in self.__mapInfoPatern:
            sub_patern=main_patern.getChildrenPaterns()
            color=sub_patern.defaultColor
            if not (color in colors):
                colors.append(color)
        if len(colors)>1:
            return False,None
        for main_patern in self.__mapInfoPatern:
            sub_patern=main_patern.getChildrenPaterns()
            sub_patern.useStyleColor=True
            sub_patern.defaultColor=QColor(0,0,0)
        return True,colors[0]





    def getMainPaterns(self):
        return self.__mapInfoPatern
    def toStrings(self):
        if isinstance(self.__mapInfoPatern,list):
            out_strs=[]
            for main_pat in self.__mapInfoPatern:
                out_strs.extend(main_pat.toString())
            return out_strs
        out_strs=self.__mapInfoPatern.toString()
        if out_strs is None:
            return None
        return out_strs
    def __str__(self):
        strings=self.toStrings()
        if strings is None:
            return None
        out_str='\n'.join(strings)
        return out_str

    def writeToFile(self,path_out):
        out_strs=self.toStrings()
        if out_strs is None:
            return
        with open(path_out, 'w') as f:
            for line in out_strs:
                f.write(line + "\n")

def factoryСircle(size):
    base_values=[(1,0),(0.81,0.59),(0.31,0.95),(-0.31,0.95),(-0.81,0.59),(-1,0),(-0.81,-0.59),(-0.31,-0.95),(0.31,-0.95),(0.81,-0.59),(1,0)]
    count_pnts=len(base_values)
    str_path=''
    str_path=str_path+"M "
    x=base_values[0][0] * size
    y=base_values[0][1] * size
    str_path=str_path+numberToString(x)+" "+numberToString(y)
    for i in range(1,count_pnts):
        x = base_values[i][0] * size
        y = base_values[i][1] * size
        str_path = str_path + " L "+numberToString(x) + " " + numberToString(y)
    return str_path
def compareColorAxiPenPatern(base_pen:AxiPenPatern,second:AxiPenPatern):
    mainSub_1=base_pen.getMainPaterns()
    mainSub_2=second.getMainPaterns()
    notEq=True
    if len(mainSub_1)!=len(mainSub_2):
        notEq = False
    count=len(mainSub_1)
    colors_segments=[]
    for i in range(count):
        sub_main_1=mainSub_1[i].getChildrenPaterns()
        sub_main_2 = mainSub_2[i].getChildrenPaterns()
        colors_segments.append({'color_1':sub_main_1.defaultColor,'color_2':sub_main_2.defaultColor})
        if notEq:
           if sub_main_1.defaultColor!= sub_main_2.defaultColor:
               notEq=False
    return notEq,colors_segments









