import parse
from PySide2.QtGui import QColor
from loguru import logger

from PanoramaRsc.ui.add_tools.rsc_tools.panoramaToAxiPen.factoryPen.penFactory import AxiPenPatern, MainLine, \
    ContinuousMarker, TypeCapStyle, TypeJoinStyle, TypeUnit, WidthElement


def stringToLogical(str_value):
    temp_val=str_value.strip().lower()
    if temp_val=='true':
        return True
    if temp_val=='false':
        return False
    return None
def getColor(str_source)->QColor:
    item_color=parse.search("defaultColor [{value}]", str_source)
    if item_color is None:
        return None
    strhex_color=parse.search('"{value}"',item_color.named['value'])
    return QColor(strhex_color.named['value'])
def getDashPattern(str_source):
    item_sub = parse.search("dashPattern [{value}]", str_source)
    if item_sub is None:
        return None
    value = item_sub.named['value']
    value=value.replace('\n','')
    values=value.split(',')
    if len(values)==0:
        return None
    number_values=[]
    for element in values:
        if element=='':
            continue
        value_elem=getNumberItem("element", element+'\n')
        number_values.append(value_elem)
    return number_values

def getDeafultWidth(str_source):
    item_sub = parse.search("defaultWidth [{value}]", str_source)
    if item_sub is None:
        return None
    str_value=item_sub.named['value']
    value_width=getNumberItem('value',str_value)
    result_unit=parse.search('unit :{value}\n', str_value)
    value_unit_name=result_unit.named['value'].replace('"','')
    value_unit_name=value_unit_name.replace(' ','')
    unit=TypeUnit.from_str(value_unit_name)
    return WidthElement(unit,value_width)


def getStyleCapJoin(name_style,str_source):
    item_sub = parse.search(name_style + " [{value}]", str_source)
    if item_sub is None:
        return None
    value=item_sub.named['value']
    value=value.replace('\n','')
    value = value.replace('"', '')

    if name_style.lower()=='capstyle':
        return TypeCapStyle.from_str(value)
    if name_style.lower()=='joinstyle':
        return TypeJoinStyle.from_str(value)
    return None
def getNumberItem(name,str_source):
    item_sub=parse.search(name+" :{value}\n", str_source)
    if item_sub is None:
        return None
    value=item_sub.named['value']
    if value.find('.')>0:
        return float(value)
    return int(value)
def setPropertySubObject(cls_base,str_sub_object):
    item_sub=parse.search("enabled :{value}\n", str_sub_object)
    if item_sub:
        cls_base.enabled=stringToLogical(item_sub.named['value'])
    item_sub = parse.search("useStyleColor :{value}\n", str_sub_object)
    if item_sub:
        cls_base.useStyleColor = stringToLogical(item_sub.named['value'])

    item_sub = parse.search("useStyleWidth :{value}\n", str_sub_object)
    if item_sub:
        cls_base.useStyleWidth = stringToLogical(item_sub.named['value'])
    defaultColor=getColor(str_sub_object)
    if defaultColor is not None:
        cls_base.defaultColor=defaultColor
    value=getNumberItem('baseWidth',str_sub_object)
    if value is not None:
        cls_base.baseWidth=value
    cls_defult_width=getDeafultWidth(str_sub_object)
    if cls_defult_width is not None:
        cls_base.defaultWidth=cls_defult_width
    value = getNumberItem('opacity', str_sub_object)
    if value is not None:
        cls_base.opacity = value
    value = getNumberItem('offset', str_sub_object)
    if value is not None:
        cls_base.offset = value
    value=getStyleCapJoin('joinStyle',str_sub_object)
    if value is not None:
        cls_base.joinStyle=value
    value = getStyleCapJoin('capStyle', str_sub_object)
    if value is not None:
        cls_base.capStyle = value
    return
def setProperyMainLine(cls_sub:MainLine,str_source):

    value = getNumberItem('dashOffset', str_source)
    if value is not None:
        cls_sub.dashOffset=value
    dashPattern=getDashPattern(str_source)
    if dashPattern is not None:
        cls_sub.dashPattern=dashPattern
    return
def setPropertyContinuousMarker(cls_sub:ContinuousMarker,str_source):
    value = getNumberItem('dashOffset', str_source)
    if value is not None:
        cls_sub.dashOffset = value
    dashPattern = getDashPattern(str_source)
    if dashPattern is not None:
        cls_sub.dashPattern = dashPattern
    item_sub = parse.search("bend :{value}\n", str_source)
    if item_sub:
        cls_sub.bend = stringToLogical(item_sub.named['value'])
    item_sub = parse.search("fill :{value}\n", str_source)
    if item_sub:
        cls_sub.fill = stringToLogical(item_sub.named['value'])
    item_sub = parse.search('path :{value}\n', str_source)
    if item_sub:
        value_path=item_sub.named['value'].replace('"','')
        cls_sub.path=value_path



def buildSubObject(name_class,str_sub_obj):
    cls_sub=None
    if name_class=='mainLine':
        cls_sub=MainLine()
    if name_class=='continuousMarker':
        cls_sub=ContinuousMarker()
    if cls_sub is None:
        logger.error("Not define class sub object ")+name_class
        return None
    setPropertySubObject(cls_sub,str_sub_obj)
    if name_class=='mainLine':
        setProperyMainLine(cls_sub,str_sub_obj)
    if name_class=='continuousMarker':
        setPropertyContinuousMarker(cls_sub,str_sub_obj)
    return cls_sub
def getSubObject(contain_main:str):
    list_subObject=['mainLine','continuousMarker']
    temp_sub_item = contain_main + "]\n "
    for name_sub in list_subObject:
        result=parse.search(name_sub+" [\n{}]\n ", temp_sub_item)
        if result is None:
            continue
        cls_sub=buildSubObject(name_sub,result.fixed[0])
        return cls_sub
    return None
def AxiPenfromTrtp(str_data:str):
    com_str = str_data.replace('{', '[')
    com_str = com_str.replace('}', ']')
    cls_axiPen=AxiPenPatern()
    for r in parse.findall("mapInfoSubPattern [\n{}]\n]", com_str):
        if r is None:
            continue
        sub_obj=getSubObject(r.fixed[0])
        if sub_obj is not None:
            cls_axiPen.add(sub_obj)
    return cls_axiPen
