import os
import uuid

import axipy
from PySide2 import QtWidgets, QtCore
from PySide2.QtCore import QFile, Qt
from PySide2.QtSql import QSqlRelationalDelegate, QSqlRelationalTableModel, QSqlTableModel, QSqlRelation
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QDataWidgetMapper, QLineEdit, QComboBox
from axipy import Style, data_manager
from loguru import logger

#from PanoramaRsc.ui.add_tools.ObserverEditRecord import ObserverEnableEditRecord
from PanoramaRsc.ui.add_tools.observers import factoryObserver
from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.helper_db.db_add_helper import getPropertyVolumnsAxipy, factoryGenerateStyleFromClassId


def createModel(db,name_table,fiels_info):
    model = QSqlRelationalTableModel(None,db)
    model.setTable(name_table)
    model.setEditStrategy(QSqlTableModel.OnManualSubmit)
    count_index=0
    for info_field in fiels_info:
        key_info=info_field['key_inf']
        if key_info is None:
            continue
        count_index=count_index+1
        id_field=model.fieldIndex(info_field['name'])
        tab_dict=key_info['tab_dict']
        field_key=key_info['name_field_key']
        field_value=key_info["name_fiedl_value"]
        model.setRelation(id_field, QSqlRelation(tab_dict, field_key, field_value))
    if count_index>0:
        return model
    model=None
    return None
class Delegate(QSqlRelationalDelegate):
    def setModelData(self, editor, model, index):
        if isinstance(editor, QtWidgets.QComboBox):

            value = editor.currentText()
            if not value:
                return
            childModel = model.relationModel(index.column())
            for column in range(2):
                match = childModel.match(childModel.index(0, column),
                    QtCore.Qt.DisplayRole, value, QtCore.Qt.MatchStartsWith)
                if match:
                    match = match[0]
                    displayValue = match.sibling(match.row(), 1).data()
                    editValue = match.sibling(match.row(), 0).data()
                    model.setData(index, displayValue, QtCore.Qt.DisplayRole)
                    model.setData(index, editValue, QtCore.Qt.EditRole)

                    return
        super(Delegate, self).setModelData(editor, model, index)
    '''
    def setEditorData(self, editor, index):
        if isinstance(editor, QtWidgets.QComboBox):
            editor.blockSignals(True)
            value=index.model().data(index, QtCore.Qt.EditRole)
            if isinstance(value,str):
                index=editor.findText(value, Qt.MatchExactly)
                if index>=0:
                    editor.setCurrentIndex(index)
            editor.blockSignals(False)
            return
        super(Delegate, self).setEditorData(editor,  index)
    '''
class FormEdit(QWidget):
    def __init__(self,config_db_dictinory,parent=None):
        super().__init__(parent)
        self.__temp_db=None
        self.__field_guid="globalid"
        self.__tab_query=None
        self.__active_table=None
        #self.__active_tab=table
        self.__config_db_dict=config_db_dictinory
        #self.__getNameFieldGlobalId()
        self.ui=self.load_ui("DlgFormEdit.ui",self)
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__observer_change_config.changed.connect(self.__change_db_config)
        self.mapper=None

        self.__open_dict_tab = []

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def setActiveTable(self,table,base_table):
        sql="Select * from "+table.name
        if self.__tab_query is not None:
            try:
                self.__tab_query.close()
            except:
                pass
        if self.__active_table is not None:
            try:
                self.__active_table.close()
                self.__count_ft_active = 0
            except:
                pass
        self.__name_base_active_table=table.name
        self.__tab_query=axipy.data_manager.query(sql)
        self.__active_table=self.__tab_query
        self.__name_active=self.__active_table.name
        self.__count_ft_active=self.__active_table.count()
        self.__base_table_active=base_table
        self.ui.tableWidget.clear()
        self.ui.label_base_table.setText(base_table.name)
        statusEnabled=table.count()>1
        '''
        if table.count()>1:
            self.ui.pb_prev.setEnabled(True)
            self.ui.pb_next.setEnabled(True)
        else:
            self.ui.pb_prev.setEnabled(True)
            self.ui.pb_next.setEnabled(True)
        '''
        #self.ui.pb_prev.setEnabled(statusEnabled)
        self.ui.pb_next.setEnabled(statusEnabled)
        self.__initDataModel()

    def __initDataModel(self):
        self.__dict_widget_data = {}
        self.__relationModels = []
        self.__temp_db=DbRsc(self.__config_db_dict.PathRscDb)
        self.__db_axioma = axipy.sql.get_database()
        self.__fields_info = getPropertyVolumnsAxipy(self.__temp_db, self.__base_table_active.name)
        self.__cls_style=factoryGenerateStyleFromClassId(self.__temp_db,self.__fields_info)
        #self.__temp_db.close(False)
        #self.__temp_db=None
        self.__openDictionaryTable(self.__fields_info,self.__config_db_dict.PathRscDb)
        self.createDataWidget(self.__fields_info)
        # setup the model based off of an sqlite table
        # self.model = createModel(self.db,"AdmBorder",self.__fields_info)
        self.model = createModel(self.__db_axioma, self.__active_table.name, self.__fields_info)
        self.mapper=None
        initSignal=False
        if self.mapper is None:
            initSignal=True
        self.createMapper(self.__fields_info)
        if initSignal:
            #self.ui.pb_prev.clicked.connect(self.mapper.toPrevious)
            #self.ui.pb_next.clicked.connect(self.mapper.toNext)
            self.ui.pb_prev.clicked.connect(self.__previous)
            self.ui.pb_next.clicked.connect(self.__toNext)
            self.ui.pb_submit.clicked.connect(self.__updateTable)
        #self.mapper.currentIndexChanged.connect(self.__change_mapper)
        self.model.select()

        self.mapper.toFirst()
    def createMapper(self,fields_info):
        self.mapper = QDataWidgetMapper()
        #self.mapper.setItemDelegate(Delegate())
        self.mapper.setItemDelegate(QSqlRelationalDelegate(self))
        self.mapper.setModel(self.model)
        for field_info in fields_info:
            name=field_info['name']
            widget=self.__dict_widget_data[name]
            nameIndex = self.model.fieldIndex(name)
            if isinstance(widget,QLineEdit):
                self.mapper.addMapping(widget, nameIndex)
            else:
                self.mapper.addMapping(widget, nameIndex)
                self.__relationModels.append(self.model.relationModel(nameIndex))
                widget.setModel(self.__relationModels[-1])
                widget.setModelColumn(self.__relationModels[-1].fieldIndex(field_info['key_inf']["name_fiedl_value"]))
    def __change_mapper(self,index):
        jkl=0
    def createDataWidget(self,fields_info):
        self.ui.tableWidget.setColumnCount(2)
        self.ui.tableWidget.setRowCount(len(fields_info))
        for i,field_info in enumerate(fields_info):
            self.createFieldWidget(field_info,i)
        self.ui.tableWidget.resizeColumnsToContents()
    def createFieldWidget(self,field_info,id):
        label = QtWidgets.QLabel()
        label.setText(field_info['name'])
        if field_info['key_inf'] is None:
            LineEdit = QtWidgets.QLineEdit()
            if field_info['type']==1:
                LineEdit.setAlignment(Qt.AlignRight)
            if field_info['name']=='fid':
                LineEdit.setEnabled(False)
            if field_info['name']==self.__field_guid:
                LineEdit.setEnabled(False)
            self.__dict_widget_data[field_info['name']]=LineEdit
        else:
            comboBox=QtWidgets.QComboBox()
            comboBox.setFixedWidth(250)
            comboBox.adjustSize()
            #comboBox.adjustSize()
            self.__dict_widget_data[field_info['name']] = comboBox
        self.ui.tableWidget.setCellWidget(id,0,label)
        self.ui.tableWidget.setCellWidget(id, 1, self.__dict_widget_data[field_info['name']])
        #layout=QtWidgets.QHBoxLayout()
        #layout.addWidget(label)
        #layout.addWidget(self.__dict_widget_data[field_info['name']])
    def __close_dictinory_fk_tab(self):
        if len(self.__open_dict_tab)>0:
            for tab in self.__open_dict_tab:
                logger.debug("Close table:"+tab.name)
                tab.close()
            self.__open_dict_tab.clear()
    def __openDictionaryTable(self,fields_info,db_path):
        self.__close_dictinory_fk_tab()
        self.__open_dict_tab=[]
        for field_info in fields_info:
            if field_info['key_inf'] is None:
                continue
            name_tab=field_info['key_inf']['tab_dict']
            def_tab = {'src': str(db_path),
                          'dataobject': name_tab,
                          'provider': 'SqliteDataProvider'
                          }
            tab_dict_temp=axipy.provider_manager.open_hidden(def_tab)
            self.__open_dict_tab.append(tab_dict_temp)
    def __previous(self):
        curent_row = self.mapper.currentIndex()
        if curent_row==1:
            self.ui.pb_prev.setEnabled(False)
            self.ui.pb_next.setEnabled(True)
        self.mapper.toPrevious()
    def __toNext(self):
        curent_row = self.mapper.currentIndex()
        self.ui.pb_prev.setEnabled(True)
        if curent_row==(self.__count_ft_active-2):
            self.ui.pb_next.setEnabled(False)
        self.mapper.toNext()



    def __updateTable(self):
        curent_row=self.mapper.currentIndex()

        temp_update_table=None
        if self.__count_ft_active==1:
            ft=list(self.__active_table.items())[0]
        else:
            '''
            value_fid=self.model.record(curent_row).value(0)
            sql="select * from "+self.__active_table.name+" where fid="+str(value_fid)
            temp_update_table=axipy.data_manager.query_hidden(sql)
            ft=list(temp_update_table.items())[0]
            '''
            ft=self.__getCurentFeature(curent_row)
        logger.debug("style :"+ft.style.to_mapinfo())
        new_ft,update_style_need=self.__getValueEditData(ft)
        if new_ft is None:
            ''' Изменений нет '''
            return
        '''
        i=-1
        for att_info in self.__fields_info:
            i=i+1
            if att_info['name']=='fid':
                continue
            if self.model.record(curent_row).fieldName(i)==axipy.sql.geometry_uid or self.model.record(curent_row).fieldName(i)==axipy.sql.style_uid:
                i=i+1

            ft[att_info['name']]=self.model.record(curent_row).value(i)
        '''
        logger.debug("after style :" + new_ft.style.to_mapinfo())
        if temp_update_table is not None:
            temp_update_table.update(new_ft)
            temp_update_table.close()
        else:
            self.__active_table.update(new_ft)
            if update_style_need:
                self.__update_style(new_ft)
    def __getCurentFeature(self,index_ft):
        #logger.debug("Get Feature:"+str(index_ft)+" from table:"+self.__active_table.name)
        try:
            ft=list(self.__active_table.items())[index_ft]
        except:
            self.__active_table=data_manager.find(self.__name_active)
            ft = list(self.__active_table.items())[index_ft]
        return ft
    def __getValueEditData(self,ft):
        ft_update=False
        main_objectid_update=False
        for att_info in self.__fields_info:
            if att_info['name']=='fid':
                continue
            widget_data=self.__dict_widget_data[att_info['name']]
            if isinstance(widget_data,QLineEdit):
                new_value=widget_data.text().strip()
                if att_info['name'].lower()==self.__field_guid:
                    if len(new_value)==0:
                        ft[att_info['name']]=str(uuid.uuid4())
                        ft_update = True
                        continue

                if not(ft[att_info['name']]==new_value):
                    ft_update = True
                    ft[att_info['name']]=new_value
                continue
            if isinstance(widget_data, QComboBox):
                value=widget_data.currentText()
                key_value=self.__getValueFromDictionary(att_info,value)
                if not(ft[att_info['name']]==key_value):
                    ft[att_info['name']]=key_value
                    if att_info['name']==self.__cls_style.NameBaseKey:
                        main_objectid_update=True
                    if att_info['name'] == self.__cls_style.NameSecondKey:
                        main_objectid_update = True
                    ft_update=True
        if ft_update:

            return ft,main_objectid_update
        return None,main_objectid_update

    def __update_style(self,ft):
        info_style = self.__cls_style.getStyle(ft[self.__cls_style.NameBaseKey], ft[self.__cls_style.NameSecondKey])
        if info_style is not None:
            axi_style = None
            try:
                axi_style = Style.from_mapinfo(info_style['style'])
            except:
                logger.error("error parse mi_style:" + info_style['style'])
            if axi_style is not None:
                guid=ft[self.__field_guid]
                tab_temp_update=axipy.data_manager.find(self.__name_base_active_table)
                '''
                sql_select_update="select * from "+self.__name_base_active_table+" where "+self.__field_guid+"='"+guid+"'"
                tab_query_update=axipy.data_manager.query_hidden(sql_select_update)
                ft_update=list(tab_query_update.items())[0]
                ft_update.style=axi_style
                tab_query_update.update(ft_update)
                tab_query_update.close()
                '''
                for ft_update in tab_temp_update.items():
                    ft_guid = ft_update[self.__field_guid]
                    if ft_guid==guid:
                        ft_update.style = axi_style
                        tab_temp_update.update(ft_update)
                        break



    def __getValueFromDictionary(self,att_info,value):
        key_info=att_info['key_inf']
        if key_info is None:
            return None
        sql="select "+key_info['name_field_key']+" from "+key_info['tab_dict']+" where ["+key_info['name_fiedl_value']+"]="
        if isinstance(value,str):
            sql =sql+"'"+value+"'"
        else:
            sql = sql + str(value)
        try:
            query_tab=axipy.data_manager.query_hidden(sql)
        except:
            return None
        fts=query_tab.items()
        ft=list(fts)[0]
        value_key=ft[key_info['name_field_key']]
        query_tab.close()
        return value_key
    def clear_temp(self):
        if self.__temp_db is not None:
            self.__temp_db.close(False)
        if self.__open_dict_tab is not None and len(self.__open_dict_tab)>0:
            self.__close_dictinory_fk_tab()
        if self.__active_table is not None:
            try:
                self.__active_table.close()
                self.__count_ft_active=0
            except:
                pass


    def show(self):
        self.ui.show()
    @property
    def widget(self):
        return self.ui

    def __getNameFieldGlobalId(self):
        db_temp=DbRsc(str(self.__config_db_dict.PathRscDb))
        try:
            sql="select * from base_field_name"
            rows=db_temp.select(sql)
            name_field_guid=rows[0][0]
            if len(name_field_guid)>0:
                self.__field_guid=name_field_guid.replace(' ','_')
        except:
            return self.__field_guid
        finally:
            db_temp.close()
    def __change_db_config(self):
        jkl=0
