import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget
from axipy import Notifications
from loguru import logger

from PanoramaRsc.ui.add_tools.rsc_tools.RSC.cls_rsc import loadRscToDb
from PanoramaRsc.ui.add_tools.rsc_tools.convertorRscToDb import ConvertorRscToDB


class RscToDb(QWidget):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.ui=self.load_ui("formConvertRscToDb.ui",parent)
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_add_png = os.path.join(path_icon, "open-32.png")
        path_saveas_png = os.path.join(path_icon, "saveas.png")
        self.ui.tb_open_rsc.setIcon(QIcon(path_add_png))
        self.ui.tb_save_db.setIcon(QIcon(path_saveas_png))
        self.ui.tb_open_rsc.clicked.connect(self.__select_rsc)
        self.ui.tb_save_db.clicked.connect(self.__save_db)
        self.ui.pb_run.clicked.connect(self.__run_task)
        self.ui.ln_path_source_rsc.textChanged.connect(self.__change_path_rsc)
        self.__cls_run=None
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __select_rsc(self):
        path_source_rsc = axipy.open_file_dialog("Панорама классификатор (*.rsc *.rscz);;", "Выбирете файл классификатора Панорама Гис")
        if path_source_rsc is not None:

            self.ui.ln_path_source_rsc.setText(str(path_source_rsc))
            self.ui.ln_path_source_rsc.setCursorPosition(0)
        self.__isReady()
    def __save_db(self):
        path_out_db=axipy.save_file_dialog("База данных классификатора (*.db)","Выбирете файл(db) для классификатора")
        if path_out_db is not None:

            self.ui.ln_dict_db.setText(str(path_out_db))
            self.ui.ln_dict_db.setCursorPosition(0)
        self.__isReady()
    def __change_path_rsc(self):
        self.__isReady()
    def __isReady(self):
        path_rsc=self.ui.ln_path_source_rsc.text()
        if not Path( path_rsc).exists():
            self.ui.pb_run.setEnabled(False)
            return
        path_out_db=self.ui.ln_dict_db.text()
        if path_out_db=='':
            self.ui.pb_run.setEnabled(False)
            return
        if not Path(path_out_db).parent.exists():
            self.ui.pb_run.setEnabled(False)
            return
        self.ui.pb_run.setEnabled(True)
        return
    def __run(self):
        cls_run=ConvertorRscToDB(self.ui.ln_path_source_rsc.text(),self.ui.ln_dict_db.text())
        self.ui.pb_run.setEnabled(False)
        #self.ui.textBrowser.append("<span>Преобразование Панорама Rsc в базу данных</span>")
        #self.ui.textBrowser.append("<span>Start</span>")
        cls_run.send_status.connect(self.__out_status)

        try:
            cls_run.run()
        except Exception as ex:
            logger.exception(ex)
            print(ex)

        cls_run.send_status.disconnect(self.__out_status)
        self.ui.pb_run.setEnabled(True)


    def __run_task(self):
        if Path(self.ui.ln_dict_db.text()).exists():
            os.remove(self.ui.ln_dict_db.text())
        if self.__cls_run is None:
            self.__cls_run = ConvertorRscToDB(self.ui.ln_path_source_rsc.text(), self.ui.ln_dict_db.text())

        else:
            self.__cls_run.setPath(self.ui.ln_path_source_rsc.text(), self.ui.ln_dict_db.text())
        self.__cls_run.send_status.connect(self.__out_status)
        self.ui.pb_run.setEnabled(False)
        def run_task(t: axipy.Task):
            logger.debug("Start task convert")
            isError=False
            try:
                self.__cls_run.run()
            except Exception as ex:
                logger.exception(ex)
                print(ex)
                isError=True
            if isError:
                Notifications.push('Преобразование Rsc в Db', 'Ошибка (Обращайтесь к разработчику)',
                                   Notifications.Critical)
            else:
                Notifications.push('Преобразование Rsc в Db', 'Завершено', Notifications.Success)
        def finish_task():
            self.ui.pb_run.setEnabled(True)
            self.__cls_run.send_status.disconnect(self.__out_status)


        task = axipy.Task(run_task)
        task.finished.connect(finish_task)
        task.start()

    def __out_status(self,text_status):
        self.ui.textBrowser.append("<span>"+text_status+"</span>")
    @property
    def widget(self):
        return self.ui