import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile, Qt
from PySide2.QtGui import QIcon
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QListWidgetItem, QDialog
from axipy import CoordSystem, ChooseCoordSystemDialog
from loguru import logger

from PanoramaRsc import factoryObserver, ConfigRscDict
from PanoramaRsc.ui.add_tools.createDbGpkg import TaskCreateDbGpkg
from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.panoramaToGpkg import PanoramaToGpkg


class FormCreateDbData(QWidget):
    def __init__(self,config_dict,parent):
        self.__name_table_layers='[layer_info_dict_spatial]'
        super().__init__(parent)
        self.__out_cs= CoordSystem.from_epsg(4326)
        self.__config_dictionary=config_dict

        self.ui=self.load_ui("fromCreateDb1.ui",parent)
        self.ui.ln_coordsystem.setText(self.__out_cs.name)
        curent_path = str(Path(__file__).parent)
        path_icon = os.path.join(curent_path, "icons")
        path_add_png = os.path.join(path_icon, "open-32.png")
        path_saveas_png = os.path.join(path_icon, "saveas.png")
        self.ui.tb_open_dict.setIcon(QIcon(path_add_png))
        self.ui.tb_data_sit.setIcon(QIcon(path_add_png))
        self.ui.tb_gpkg.setIcon(QIcon(path_saveas_png))
        self.__path_dict=str(self.__config_dictionary.PathRscDb)
        self.ui.lnpath_dic.setText(self.__path_dict)
        self.ui.lnpath_dic.setCursorPosition(0)
        self.ui.lnpath_dic.textChanged.connect(self.__change_path_dict)
        self.ui.ch_all.stateChanged.connect(self.__change_select_all)
        self.ui.tb_open_dict.clicked.connect(self.__open_dict_db)
        self.ui.tb_data_sit.clicked.connect(self.__open_sit)
        self.ui.tb_gpkg.clicked.connect(self.__save_gpkg)
        self.ui.tb_change_cs.clicked.connect(self.__change_cs)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
        self.__initListLayer()
        #self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        #self.__observer_change_config.changed(self.__reload_config)

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __initListLayer(self):
        self.__db = DbRsc(self.__path_dict)
        self.ui.listWidget_layers.clear()
        sql="select name,full_name from "+self.__name_table_layers
        rows=self.__db.select(sql)
        statusCheck=Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck=Qt.Checked
        for row in rows:
            name_layer=row[0]+" {"+row[1]+"}"
            item = QListWidgetItem(name_layer)
            item.setCheckState(statusCheck)
            self.ui.listWidget_layers.addItem(item)
        self.__db.close(False)
        self.__db=None
    def __change_select_all(self,value):
        count=self.ui.listWidget_layers.count()
        statusCheck = Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck = Qt.Checked
        for row in range(count):
            item=self.ui.listWidget_layers.item(row)
            item.setCheckState(statusCheck)
        self.__isReady()
    def __change_path_dict(self,new_path):
        self.__initListLayer()
        self.__isReady()
    def __isReady(self):
        path_db_dict=self.ui.lnpath_dic.text()
        if not Path( path_db_dict).exists():
            self.ui.pb_run.setEnabled(False)
            return
        path_out=self.ui.ln_path_gpkg.text()
        if path_out=="":
            self.ui.pb_run.setEnabled(False)
            return
        if not Path( path_out).parent.exists():
            self.ui.pb_run.setEnabled(False)
            return
        if self.ui.grpbx_import.isChecked():
            path_source_sit=self.ui.ln_source_sit.text()
            if path_source_sit=="":
                self.ui.pb_run.setEnabled(False)
                return
            if not Path(path_source_sit).exists():
                self.ui.pb_run.setEnabled(False)
                return
        if self.__countSelectLayers()==0:
            self.ui.pb_run.setEnabled(False)
            return

        self.ui.pb_run.setEnabled(True)
        return
    def __open_dict_db(self):
        path_source_db=axipy.open_file_dialog("Sqlite (*.db);;","Выбирете базу данных справочника")
        if path_source_db is not None:
            self.__path_dict=str(path_source_db)
            self.ui.lnpath_dic.setText(self.__path_dict)
            self.ui.lnpath_dic.setCursorPosition(0)
        self.__isReady()
    def __open_sit(self):
        path_source_sit = axipy.open_file_dialog("Данные в формате Панорама Гис (*.sit *.map);;)", "Выбирете данные для импорта")
        if path_source_sit is not None:

            self.ui.ln_source_sit.setText(str(path_source_sit))
            self.ui.ln_source_sit.setCursorPosition(0)
        self.__isReady()
    def __save_gpkg(self):
        path_out_gpkg=axipy.save_file_dialog("Gpkg (*.gpkg)","Выбирете файл(gpkg) для сохранения результата")
        if path_out_gpkg is not None:

            self.ui.ln_path_gpkg.setText(str(path_out_gpkg))
            self.ui.ln_path_gpkg.setCursorPosition(0)
        self.__isReady()
    def __change_cs(self):
        dialog = ChooseCoordSystemDialog(self.__out_cs)
        if dialog.exec() == QDialog.Accepted:
            self.__out_cs=dialog.chosenCoordSystem()
            self.ui.ln_coordsystem.setText(self.__out_cs.name)
    def __countSelectLayers(self):

        count_select=0
        for index in range(self.ui.listWidget_layers.count()):
            if self.ui.listWidget_layers.item(index).checkState() == Qt.Checked:
                count_select=count_select+1
        return count_select

    def __run(self):
        list_layers=self.__getSelectLayers()
        path_db_dict=self.ui.lnpath_dic.text()
        path_out_gpkg=self.ui.ln_path_gpkg.text()
        self.__cls_runner=TaskCreateDbGpkg(list_layers,path_db_dict,path_out_gpkg,self.__out_cs)
        self.__cls_runner.change_name_layer.connect(self.__change_name_layer)
        self.__cls_runner.change_progress.connect(self.__change_progres)
        self.ui.pb_run.setEnabled(False)
        self.ui.pb_cancel.setEnabled(True)
        def run_task(t: axipy.Task):
            logger.debug("Start create db")
            try:
                self.__cls_runner.run()
            except Exception as ex:
                logger.exception(ex)
                print(ex)

        def finish_task():
            self.ui.pb_run.setEnabled(True)
            self.ui.pb_cancel.setEnabled(False)

            self.__cls_runner.change_name_layer.disconnect(self.__change_name_layer)
            self.__cls_runner.change_progress.disconnect(self.__change_progres)
            if self.ui.grpbx_import.isChecked():
                ''' Run import data from panorama'''
                self.__run_imprt_data()
        task = axipy.Task(run_task)
        task.finished.connect(finish_task)
        task.start()
    def __getSelectLayers(self):
        list_layers=[]

        for index in range(self.ui.listWidget_layers.count()):
            if self.ui.listWidget_layers.item(index).checkState() == Qt.Checked:
                name_full=self.ui.listWidget_layers.item(index).text()
                ind=name_full.find(" {")
                name=name_full[0:ind].strip()
                list_layers.append(name)
        return list_layers
    def __change_name_layer(self,name_layer):
        self.ui.progressBar.setFormat(name_layer + " .. %p%")

    def __change_progres(self, value):
        self.ui.progressBar.setValue(value)
    def __cancel(self):
        self.__cls_runner.setCancel()
    def __run_imprt_data(self):
        self.ui.pb_run.setEnabled(False)
        self.ui.pb_cancel.setEnabled(True)
        list_layers=self.__getSelectLayers()
        self.__cls_runner_imp_data=PanoramaToGpkg(list_layers,self.ui.ln_source_sit.text(),self.ui.ln_path_gpkg.text())
        self.__cls_runner_imp_data.change_name_layer.connect(self.__change_name_layer)
        self.__cls_runner_imp_data.change_proc.connect(self.__change_progres)
        def task_import(t: axipy.Task):
            logger.debug("Start import data")
            try:
                self.__cls_runner_imp_data.run()
            except Exception as ex:
                logger.exception(ex)
                print(ex)
        def finish_task_import():
            self.__cls_runner_imp_data.closeDataPanorama()
            self.ui.pb_run.setEnabled(True)
            self.ui.pb_cancel.setEnabled(False)

            self.__cls_runner_imp_data.change_name_layer.disconnect(self.__change_name_layer)
            self.__cls_runner_imp_data.change_proc.disconnect(self.__change_progres)
            self.ui.progressBar.setValue(100)

        task = axipy.Task(task_import)
        task.finished.connect(finish_task_import)
        task.start()
    @property
    def widget(self):
        return self.ui
    def __reload_config(self):
        self.__config_dictionary=ConfigRscDict.load()
        jkl=0










