import os
from pathlib import Path

import axipy
from PySide2 import QtWidgets, QtCore
from PySide2.QtCore import QFile
from PySide2.QtGui import Qt
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QListWidgetItem, QDialog, QComboBox
from axipy import CoordSystem, Notifications, ChooseCoordSystemDialog
from loguru import logger

from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.buildGmlNoXds import FactoryGmlFGISTPNoXsd
from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.builderClsXml import FactoryGmlFGISTP
from PanoramaRsc.ui.add_tools.fgis_tp.exportToXml.helper_dic_layer import DictionaryLayer
from PanoramaRsc.ui.add_tools.helper_tab.validateStructFgisTp import ValidateStructTableGisTp
from PanoramaRsc.ui.add_tools.panoramaDictionary import PanoramaDictionary

def findInDict(list_dict,name_obj,name):
    for i,item in enumerate(list_dict):
        if item[name_obj]==name:
            return i
    return None
class ExportGml(QWidget):
    def __init__(self,config_dict,parent=None):
        super().__init__(parent)
        self.__config_dict=config_dict
        self.__cls_validatr=None
        self.__out_cs=None
        self.__parent=parent
        self.__external_open_tab=[]
        self.ui=self.load_ui("formExportToGmlGisTp3.ui",parent)
        self.ui.tableWidget_tables.setHorizontalHeaderLabels(["Таблица","Алиас ФГИСТП"])
        self.ui.ch_all.stateChanged.connect(self.__change_select_all)
        self.ui.ch_xsd.stateChanged.connect(self.__change_ch_xsd)
        self.ui.tb_add_tab.clicked.connect(self.__open_tab)
        self.ui.tb_save_gml.clicked.connect(self.__save_gml)
        self.ui.pb_run.clicked.connect(self.__run)
        self.ui.pb_close.clicked.connect(self.__isCancel)
        self.ui.tb_select_coordsys.clicked.connect(self.__select_coordsys)

        self.__list_data_source = []
        self.__initListTable1()
        self.__isRun=False
        if len(self.__list_data_source)==0:
            self.__out_cs=CoordSystem. from_epsg(3857)
            self.ui.lab_cs.setText(self.__out_cs.title)
        self.ui.tableWidget_tables.itemChanged.connect(self.__change_item_layers)
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __initListTable(self):
        self.ui.listWidget_tables.clear()
        layers_dict=self.__getLayersDictinory()


        if self.__cls_validatr is None:
            self.__cls_validatr=ValidateStructTableGisTp(layers_dict)
        statusCheck = Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck = Qt.Checked
        for tab in axipy.data_manager.tables:
            if not tab.is_spatial:
                continue
            is_valid_fgistp,name_alias_fgistp=self.__cls_validatr.tableIsGisTp(tab)
            if is_valid_fgistp:
                name_fgistp=name_alias_fgistp[0]
                self.__list_data_source.append({'name':tab.name,"alias":name_fgistp})
                item = QListWidgetItem(tab.name)
                item.setCheckState(statusCheck)
                self.ui.listWidget_tables.addItem(item)
                if self.__out_cs is None:
                    self.__out_cs = tab.coordsystem
                    self.ui.lab_cs.setText(self.__out_cs.title)

    def __initListTable1(self):
        #self.ui.tableWidget_tables.clear()
        for i in range(self.ui.tableWidget_tables.rowCount()):

                self.ui.tableWidget_tables.removeRow(i)

        layers_dict=self.__getLayersDictinory()


        if self.__cls_validatr is None:
            self.__cls_validatr=ValidateStructTableGisTp(layers_dict)
        statusCheck = Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck = Qt.Checked
        for tab in axipy.data_manager.tables:
            if not tab.is_spatial:
                continue
            is_valid_fgistp,name_alias_fgistp=self.__cls_validatr.tableIsGisTp(tab)
            widget_alias=self.__createWidgetAliasTable(name_alias_fgistp)
            if is_valid_fgistp:
                widget_alias.setCurrentIndex(0)
            self.ui.tableWidget_tables.insertRow(self.ui.tableWidget_tables.rowCount())
            row_id=self.ui.tableWidget_tables.rowCount()
            item_name = QtWidgets.QTableWidgetItem(tab.name)
            item_name.setFlags(QtCore.Qt.ItemFlag.ItemIsUserCheckable |QtCore.Qt.ItemFlag.ItemIsEnabled)
            item_name.setCheckState(statusCheck)

            self.ui.tableWidget_tables.setItem(row_id-1,0,item_name)

            self.ui.tableWidget_tables.setCellWidget(row_id-1,1,widget_alias)
        self.ui.tableWidget_tables.resizeColumnsToContents()

    def __createWidgetAliasTable(self,list_alias):
        w_combo=QComboBox()
        w_combo.addItems(list_alias)
        return w_combo
    def __change_select_all(self,value):
        count=self.ui.tableWidget_tables.rowCount()
        statusCheck = Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck = Qt.Checked
        for row in range(count):
            item_name=self.ui.tableWidget_tables.item(row,0)
            item_name.setCheckState(statusCheck)
            #self.ui.tableWidget_tables.setItem(row, 0, item_name)
        self.__isReady()

    def __getLayersDictinory(self):
        db_dict = PanoramaDictionary(str(self.__config_dict.PathRscDb))
        layers = db_dict.getLayers()
        db_dict.close()
        return layers
    def __open_tab(self):
        path_new_tab=axipy.open_file_dialog('MapInfo Tab (*.tab);;Gpkg (*.gpkg)','Открыть таблицу')
        if path_new_tab is None:
            return
        sufix_file=path_new_tab.suffix
        if sufix_file.lower()=='.tab':
            tab_temp=axipy.provider_manager.openfile(path_new_tab)
        elif sufix_file.lower()==".gpkg":
            tab_temp = axipy.provider_manager.openfile(path_new_tab)
        is_valid_fgistp, name_fgistp=self.__cls_validatr.tableIsGisTp(tab_temp)
        if not is_valid_fgistp:
            axipy.Notifications.push("Структура таблица:"+tab_temp.name+" не соотвествует структуре слоев ФГИСТП")
            tab_temp.close()
            return
        self.__external_open_tab.append(tab_temp.name)
        statusCheck = Qt.Unchecked
        if self.ui.ch_all.isChecked():
            statusCheck = Qt.Checked
        self.__list_data_source.append({'name': tab_temp.name, "alias": name_fgistp})
        item = QListWidgetItem(tab_temp.name)
        item.setCheckState(statusCheck)
        self.ui.listWidget_tables.addItem(item)
        self.__isReady()
    def __countSelectLayers(self):

        count_select=0
        for index in range(self.ui.tableWidget_tables.rowCount()):
            item_name = self.ui.tableWidget_tables.itemAt(index, 0)
            if item_name.checkState() == Qt.Checked:
                count_select=count_select+1
        return count_select
    def __save_gml(self):
        path_out_gml=axipy.save_file_dialog("Gml (*.gml)","Сохранить Gml файл")
        if path_out_gml is not None:
            self.ui.ln_out_gml.setText(str(path_out_gml))
            self.ui.ln_out_gml.setCursorPosition(0)
        self.__isReady()
    def __isCancel(self):
        if self.__isRun:
            self.__cls_runner.setCancel()
        else:
            self.__close()
    def __isReady(self):
        count_sel_layer = self.__countSelectLayers()
        if count_sel_layer == 0:
            self.ui.pb_run.setEnabled(False)
            return
        espg=self.__out_cs.epsg
        if espg is None:
            axipy.Notifications.push("Экспорт в Gml","Проекция не имеет EPSG кода",Notifications.Warning)
            self.ui.pb_run.setEnabled(False)
            return
        path_out_gml=Path(self.ui.ln_out_gml.text())
        if path_out_gml.suffix.lower()!='.gml':
            self.ui.pb_run.setEnabled(False)
            return

        self.ui.pb_run.setEnabled(True)
    def __run(self):
        list_select_layers=[]
        db_dict=None
        if self.ui.ch_xsd.isChecked():
            db_dict=DictionaryLayer(self.__config_dict.PathRscDb)
        for index in range(self.ui.tableWidget_tables.rowCount()):
            item_name=self.ui.tableWidget_tables.item(index,0)
            if item_name.checkState() == Qt.Checked:
                name_full = item_name.text()
                item_alias = self.ui.tableWidget_tables.cellWidget(index, 1)
                list_keys=None
                #obj_dict= next(x for x in self.__list_data_source if x['name'] == name_full)
                if db_dict is not None:
                    list_keys=db_dict.getDictionaryLayer(name_full)
                obj_dict={'name':name_full,'alias':item_alias.currentText(),"dict":list_keys}
                list_select_layers.append(obj_dict)
        if db_dict is not None:
            db_dict.distruct()
        def run_export(task):
            logger.debug("Start process")
            self.__cls_runner.run()

        if self.ui.ch_xsd.isChecked():
            self.__cls_runner=FactoryGmlFGISTPNoXsd(list_select_layers,self.__out_cs,Path(self.ui.ln_out_gml.text()).parent)
        else:
            self.__cls_runner=FactoryGmlFGISTP(list_select_layers,self.__out_cs,self.ui.ln_out_gml.text())
        self.__cls_runner.next_layer.connect(self.__change_layer)
        self.__cls_runner.change_procent_feature.connect(self.__change_progres)
        self.__task_export = axipy.Task(run_export)
        self.__task_export.finished.connect(self.end_process)
        self.ui.pb_run.setEnabled(False)
        self.ui.pb_close.setText("Прервать")
        self.ui.pb_close.setEnabled(True)
        self.__isRun=True
        self.__task_export.start()
    def finished(self):
        if self.__isRun:
            self.__cls_runner.setCancel()
        self.__close()
    def __close(self):
        if len(self.__external_open_tab)>0:
            for tab in  self.__external_open_tab:
                tab.close()
        self.__parent.close()
    def end_process(self):
        self.ui.pb_run.setEnabled(True)
        #self.ui.pb_close.setEnabled(False)
        self.ui.pb_close.setText("Закрыть")
        self.__isRun=False
        self.__cls_runner.next_layer.disconnect(self.__change_layer)
        self.__cls_runner.change_procent_feature.disconnect(self.__change_progres)
        self.__task_export.finished.disconnect(self.end_process)
    def __change_layer(self,name_layer,proc):
        self.ui.progressBar_layers.setFormat(name_layer + " .. %p%")
        self.ui.progressBar_layers.setValue(proc)
    def __change_progres(self,value_proc):
        self.ui.progressBar_features.setValue(value_proc)




    def __change_item_layers(self,item):
        self.__isReady()
    def __select_coordsys(self):
        dialog = ChooseCoordSystemDialog(self.__out_cs)
        if dialog.exec() == QDialog.Accepted:
            self.__out_cs=dialog.chosenCoordSystem()
            self.__isReady()
    def __change_ch_xsd(self):
        if self.ui.ch_xsd.isChecked():
            self.ui.ch_multi_file.setCheckState(Qt.CheckState.Checked)
            self.ui.ch_multi_file.setEnabled(False)
        else:
            self.ui.ch_multi_file.setEnabled(True)


