import os
from pathlib import Path

import axipy
from PySide2 import QtGui, QtCore
from PySide2.QtCore import QFile
from PySide2.QtGui import QStandardItem, Qt
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QTabWidget, QDialog
from axipy import ChooseCoordSystemDialog, provider_manager, Layer
from loguru import logger

from PanoramaRsc.ui.add_tools.exportToPanorama import ExportToPanorama
from PanoramaRsc.ui.add_tools.helper_db.dbRsc import DbRsc
from PanoramaRsc.ui.add_tools.helper_db.db_add_helper import extractRscToFolder, TypeGeometry
from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict
from PanoramaRsc.ui.add_tools.observers import factoryObserver
class LayerInfoGeomtry:
    def __init__(self,name:str,type_geometry:TypeGeometry):
        self.name=name
        self.typeGeometry=type_geometry

def getTableCoordSystem(name_tab,path_gpkg):
    path_cls=Path(path_gpkg)
    tables=axipy.data_manager.tables
    if len(tables)==0:
        return None
    maybe_tab=None
    for tab in tables:
        if not('fileName' in tab.properties.keys()):
            continue
        if Path(tab.properties['fileName'])==path_cls:
            maybe_tab=tab
            if tab.name==name_tab:
                return tab.coordsystem
    if maybe_tab is not None:
        return maybe_tab.coordsystem
    return None


class WidgetExportToPanorama(QWidget):
    __name_widget = "Экспорт в формата Панорама Гис"
    def __init__(self,config_data:ConfigRscDict,parent=None):
        super().__init__(parent)
        self.__cls_expor = None
        self.__config_data=config_data
        self.__parent=parent
        self.__observerReady = factoryObserver("PanoramaRscConfig")
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__modelListView=None
        if self.__observerReady is not None:
            self.__observerReady.changed.connect(self.__change_ready)
        self.ui=self.load_ui("exportToPanorama1.ui",parent)
        self.ui.tb_save_file.clicked.connect(self.__select_dest)
        self.ui.tb_select_gpkg.clicked.connect(self.__select_source)
        self.ui.tb_change_cs.clicked.connect(self.__select_cs)
        self.ui.ch_all.stateChanged.connect(self.__change_select_all)
        self.ui.ln_path_source.textChanged.connect(self.__isReady)
        self.ui.ln_file_save.textChanged.connect(self.__isReady)
        self.ui.pb_run.clicked.connect(self.__run)
        if isinstance(parent, QTabWidget):
            self.__isTab = True
            count_tab = parent.count()
            self.__idTab = count_tab - 1
            self.__parent.setTabText(self.__idTab, self.__name_widget)
            self.__change_ready()
        if self.__config_data is not None:
            self.ui.ln_path_source.setText(str(self.__config_data.PathDb))
            self.ui.ln_path_source.setCursorPosition(0)
        self.__list_info_layer=[]
        self.__initListTable()
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def __change_ready(self):
        if self.__isTab:
            self.__parent.setTabEnabled(self.__idTab,self.__observerReady.value)
    def __select_dest(self):
        path_out=axipy.save_file_dialog("Panorama (*.sit);;","Файл Панорама Гис")
        if path_out is None:
            return
        self.ui.ln_file_save.setText(str(path_out))
        self.ui.ln_file_save.setCursorPosition(0)
    def __select_source(self):
        path_out=axipy.open_file_dialog("GeoPackage (*.gpkg);;","Выбрать исходный gpkg")
        if path_out is None:
            return
        self.ui.ln_path_source.setText(str(path_out))
        self.ui.ln_path_source.setCursorPosition(0)
        self.__initListTable()
    def __initListTable(self):
        path_source=self.ui.ln_path_source.text()
        if (not Path(path_source).exists()) or len(path_source)==0:
            return
        temp_db=DbRsc(path_source)
        if not temp_db.isConnect:
            return
        sql="select * from layer_info_spr"
        rows=temp_db.select(sql)
        temp_db.close(False)
        self.__list_info_layer.clear()
        if self.__modelListView is None:
            self.__modelListView=QtGui.QStandardItemModel()
            self.ui.listViewLayers.setModel(self.__modelListView)
        else:
            self.__modelListView.clear()
        self.ui.listViewLayers.model().clear()


        for i,row in enumerate(rows):

            self.__list_info_layer.append({'name':row[1],'short_name':row[2],'full_name':row[3],'type_geometry':row[4],'display_orderIndex':row[5]})
            if i==0:
                self.__curent_cs=self.__getLayerCoordSystem(row[1],path_source)

                self.ui.ln_name_cs.setText(self.__curent_cs.name)
            item = QStandardItem(row[1])
            item.setFlags(Qt.ItemIsUserCheckable | Qt.ItemIsEnabled)
            if self.ui.ch_all.isChecked():
                item.setData(Qt.Checked, Qt.CheckStateRole)
            else:
                pass
                #item.setData(Qt.`, Qt.CheckStateRole)
            self.__modelListView.appendRow(item)
    def __change_select_all(self):
        model = self.ui.listViewLayers.model()
        for row in range(model.rowCount()):
            #index = model.item(row, 0)
            #item = model.data(index, Qt.DisplayRole)
            item=model.item(row, 0)
            if self.ui.ch_all.isChecked():
                item.setData(Qt.Checked, Qt.CheckStateRole)
            else:
                item.setData(Qt.Unchecked, Qt.CheckStateRole)
    def __getLayerCoordSystem(self,name_layer,path_gpkg):
        cs=getTableCoordSystem(name_layer,path_gpkg)
        if cs is not None:
            return cs
        def_tab_db = {'src': path_gpkg,
                      'dataobject': name_layer,
                      'provider': 'SqliteDataProvider'
                      }
        tab_temp = axipy.provider_manager.open_hidden(def_tab_db)
        cs=tab_temp.coordsystem
        tab_temp.close()
        return cs
    def __select_cs(self):
        dlg_cs=ChooseCoordSystemDialog(self.__curent_cs)
        if dlg_cs.exec() == QDialog.Accepted:
            self.__curent_cs= dlg_cs.chosenCoordSystem()
            self.ui.ln_name_cs.setText(self.__curent_cs.name)
    def __isReady(self):
        path_source_gpkg=self.ui.ln_path_source.text()
        if not Path(path_source_gpkg).exists():
            self.ui.pb_run.setEnabled(False)
            return
        path_out_str=self.ui.ln_file_save.text()
        if len(path_out_str)==0:
            self.ui.pb_run.setEnabled(False)
            return
        path_out=Path(path_out_str)
        if not path_out.parent.exists():
            self.ui.pb_run.setEnabled(False)
            return
        self.ui.pb_run.setEnabled(True)
        return
    def __run(self):
        self.__cls_expor=None
        params_source={}
        params_source['file_gpkg']=self.ui.ln_path_source.text()
        layers_info_select=self.__getLayerInfo()
        if layers_info_select is None:
            return
        #params_source['tables']=self.__getSelectedLayer()
        params_source['tables']=layers_info_select
        params_out={}
        params_out['out_file']=self.ui.ln_file_save.text()
        '''
        path_folder_panorama_rsc=axipy.CurrentSettings.UserDataFolder.joinpath("panorama_rsc")
        if not path_folder_panorama_rsc.exists():
            os.mkdir(str(path_folder_panorama_rsc))
        '''
        path_folder_panorama_rsc=Path(params_out['out_file']).parent
        params_out['rsc_file']=extractRscToFolder(self.__config_data.PathRscDb,path_folder_panorama_rsc)
        params_out['coordsystem']=self.__curent_cs
        params_out['objectkey']="CLASSID"
        self.__cls_expor=ExportToPanorama()
        def run_export(task):
            logger.debug("Start process")
            self.__cls_expor.run(params_source,params_out)

        def finish(self):
            #self.__cls_expor.change_name_tab.disconnect(self.__change_name)
            #self.__cls_expor.change_id_feature.disconnect(self.__change_progres)
            #self.__task_export.finished.disconnect(finish)
            self.ui.pb_run.setEnabled(True)
            self.ui.pb_cancel.setEnabled(False)


        self.__cls_expor.change_name_tab.connect(self.__change_name)
        self.__cls_expor.change_id_feature.connect(self.__change_progres)
        self.__task_export = axipy.Task(run_export)
        self.__task_export.finished.connect(self.end_process)
        self.ui.pb_run.setEnabled(False)
        self.ui.pb_cancel.setEnabled(True)
        self.__task_export.start()

    def __change_name(self,name_layer):
        self.ui.progressBar.setFormat(name_layer + " .. %p%")
    def __change_progres(self,value):
        self.ui.progressBar.setValue(value)
    def end_process(self):
        self.ui.pb_run.setEnabled(True)
        self.ui.pb_cancel.setEnabled(False)
        self.__cls_expor.change_name_tab.disconnect(self.__change_name)
        self.__cls_expor.change_id_feature.disconnect(self.__change_progres)
        self.__task_export.finished.disconnect(self.end_process)
        if self.__cls_expor.CountProcessingLayers>0:
            ''' открываем файл sit'''
            tab_pan=provider_manager.panorama.open(self.ui.ln_file_save.text())
            axipy.mainwindow.add_layer_interactive(Layer.create(tab_pan))

    def __getSelectedLayer(self):
        model=self.ui.listViewLayers.model()
        selectedLayers=[]
        for index in range(model.rowCount()):
            item = model.item(index)
            if item.checkState() == QtCore.Qt.Checked:
                selectedLayers.append(item.text())
        return selectedLayers
    def __getLayerInfo(self):
        layers=self.__getSelectedLayer()
        if len(layers)==0:
            return None
        db=DbRsc(str(self.__config_data.PathRscDb))
        layers_info=[]
        for name in layers:
            sql="select type_geometry from layer_info_dict_spatial where name='"+name+"'"
            rows=db.select(sql)
            type_geometry=TypeGeometry(rows[0][0])
            layers_info.append(LayerInfoGeomtry(name,type_geometry))
        db.close(False)
        return layers_info





