import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QTabWidget, QMessageBox

from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict
from PanoramaRsc.ui.add_tools.observers import factoryObserver
from PanoramaRsc.ui.add_tools.helper_db.SqliteDataSource import DbSqliteDataSource
from PanoramaRsc.ui.add_tools.importGmlFgis import ImportGmlGisTp
from PanoramaRsc.ui.add_tools.panoramaDictionary import PanoramaDictionary
from PanoramaRsc.ui.add_tools.viewmap_db import view_map_from_db


class ImportGml(QWidget):
    __name_widget="Импорт gml ФГИС ТП"
    def __init__(self,db_dictionare:PanoramaDictionary,parent=None):
        super().__init__(parent)
        self.__parent=parent
        self.__observerReady=factoryObserver("PanoramaRscConfig")
        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        if self.__observerReady is not None:
            self.__observerReady.changed.connect(self.__change_ready)
        self.__cls_importGml=None
        self.__db_dictionary=db_dictionare
        self.ui=self.load_ui("ImportGml.ui",parent)
        self.ui.ln_source_gml.textChanged.connect(self.__change_source_gml)
        self.ui.ln_dest_gpkg.textChanged.connect(self.__change_source_dest)
        self.ui.pb_open.clicked.connect(self.__open_gml)
        self.ui.pb_save.clicked.connect(self.__save_gpkg)
        self.ui.pb_start.clicked.connect(self.__run)
        self.ui.pb_cancel.clicked.connect(self.__cancel)
        self.__isTab = False
        self.__idTab = None
        if isinstance(parent, QTabWidget):
            self.__isTab = True
            count_tab = parent.count()
            self.__idTab = count_tab - 1
            self.__parent.setTabText(self.__idTab, self.__name_widget)
            self.__change_ready()
    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui
    def show(self):
        self.ui.show()
    def __change_source_gml(self,new_text):
        self.__isReady(new_text,self.ui.ln_dest_gpkg.text())
    def __change_source_dest(self,out_path_gpk):
        self.__isReady(self.ui.ln_source_gml.text(), out_path_gpk)
        if Path(out_path_gpk).exists():
            self.ui.ch_exist_file.setEnabled(True)
        else:
            self.ui.ch_exist_file.setEnabled(False)
    def __isReady(self,path_source,path_dest):
        isOkSource = Path(path_source).exists()

        path_dest_obj=Path(path_dest)
        if path_dest=="":
            isOkDest=False
        else:
            isOkDest = path_dest_obj.exists()
            if not isOkDest:
                if path_dest_obj.parent.exists():
                    if str(path_dest_obj.suffix).lower()==".gpkg":
                        isOkDest=True
        if isOkSource and isOkDest:
            self.ui.pb_start.setEnabled(True)
        else:
            self.ui.pb_start.setEnabled(False)
    def __open_gml(self):
        path_gml=axipy.open_file_dialog("Gml fgistp (*.gml)","Выбор файла gml ФГИСТП")
        if path_gml is not None:
            self.ui.ln_source_gml.setText(str(path_gml))
            self.ui.ln_source_gml.setCursorPosition(0)
        jk=0

    def __save_gpkg(self):
        try:
            path_gpkg = axipy.save_file_dialog("GeoPackage (*.gpkg)", "Сохранить в ..")
        except Exception as ex:
            pass
        if path_gpkg is not None:
            self.ui.ln_dest_gpkg.setText(str(path_gpkg))
            self.ui.ln_dest_gpkg.setCursorPosition(0)
        jk = 0
    def __run(self):
        path_source_gml=self.ui.ln_source_gml.text()
        path_db=self.ui.ln_dest_gpkg.text()
        if Path(path_db).exists():
            if not self.ui.ch_exist_file.isChecked():
                ''' удаляем файл '''
                dlg = QMessageBox(self.ui)
                dlg.setWindowTitle("Файл базы пространственных данных существует!")
                dlg.setText("Будем удалять ??")
                dlg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
                dlg.setIcon(QMessageBox.Question)
                button = dlg.exec()
                if not (button == QMessageBox.Yes):
                    return
                os.remove(path_db)
                self.__cls_db_sp = DbSqliteDataSource()
                self.__cls_db_sp.create(path_db)
            else:
                self.__cls_db_sp = DbSqliteDataSource()
                self.__cls_db_sp.open(path_db)
        else:
            self.__cls_db_sp=DbSqliteDataSource()
            self.__cls_db_sp.create(path_db)
        def run_task(task,path_gnm):
            print("start")
            self.__cls_importGml.run(task,path_gnm)
        def finish_task():
            self.ui.progressBar.setValue(100)
            self.__task_import.finished.disconnect(finish_task)
            self.__cls_importGml.value_changed.disconnect(self.__changeProgress)
            self.__cls_importGml.message_changed.disconnect(self.__change_nameLayer)
            self.ui.pb_start.setEnabled(True)
            self.ui.pb_cancel.setEnabled(False)
            count_feature=self.__cls_importGml.countFeature
            self.__cls_db_sp.close()
            if count_feature>0:
                list_layer=self.__cls_importGml.layers
                view_map_from_db(list_layer,self.__cls_db_sp.PathDb)

           
            if not self.__cls_importGml.isCancel:
                ''' обновляем конфиг'''
                config_data=ConfigRscDict(self.__db_dictionary.pathDb,self.ui.ln_dest_gpkg.text())
                config_data.save()
                self.__observer_change_config.value=not self.__observer_change_config.value

        self.__db_dictionary.close()
        #self.__db_dictionary.reOpen()
        self.__cls_importGml=ImportGmlGisTp(self.__db_dictionary,self.__cls_db_sp)
        self.__cls_importGml.value_changed.connect(self.__changeProgress)
        self.__cls_importGml.message_changed.connect(self.__change_nameLayer)
        self.__task_import=axipy.Task(run_task)
        self.__task_import.finished.connect(finish_task)
        self.ui.pb_start.setEnabled(False)
        self.ui.pb_cancel.setEnabled(True)
        self.__task_import.start(path_source_gml)
    def __cancel(self):
        if self.__cls_importGml is not None:
            self.__cls_importGml.setCancel()
    def __changeProgress(self,value):
        self.ui.progressBar.setValue(value)
    def __change_nameLayer(self,name):
        self.ui.progressBar.setFormat(name + " .. %p%")

    def __change_ready(self):
        if self.__isTab:
            self.__parent.setTabEnabled(self.__idTab,self.__observerReady.value)


