import os
from pathlib import Path

import axipy
from PySide2.QtCore import QFile
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget, QTabWidget

from PanoramaRsc.ui.add_tools.saveConfig import ConfigRscDict
from PanoramaRsc.ui.add_tools.observers import factoryObserver

class InitRscAndData(QWidget):
    __name_widget="Настройка классификатора"
    def __init__(self,config_data,parent=None):
        super().__init__(parent)
        self.__config_data=config_data

        self.__observer_change_config = factoryObserver("PanoramaRscConfigChanged")
        self.__parent=parent
        self.ui=self.load_ui("tool_init_db.ui",parent)
        self.__isTab=False
        self.__idTab=None
        if isinstance(parent,QTabWidget):
            self.__isTab=True
            count_tab=parent.count()
            self.__idTab=count_tab-1
            self.__parent.setTabText(self.__idTab, self.__name_widget)
        self.ui.tb_select_rsc_db.clicked.connect(self.__open_db_rsc)
        self.ui.tb_data_gpkg.clicked.connect(self.__open_gpkg)
        self.ui.open_config.clicked.connect(self.__loadConfig)
        self.ui.pb_apply.clicked.connect(self.__save_config)
        self.__observer_change_config.changed.connect(self.__update_config)
        if self.__config_data is not None:
            path_db=self.__config_data.PathRscDb
            self.ui.ln_path_rsc_db.setText(str(path_db))
            self.ui.ln_path_rsc_db.setCursorPosition(0)
            path_data=self.__config_data.PathDb
            self.ui.ln_data_gpkg.setText(str(path_data))
            self.ui.ln_data_gpkg.setCursorPosition(0)

    def load_ui(self,name_resource,parent):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        ui  = loader.load(ui_file,parent)
        ui_file.close()
        return ui

    def __update_config(self):
        self.__config_data=ConfigRscDict.load()
        path_db = self.__config_data.PathRscDb
        self.ui.ln_path_rsc_db.setText(str(path_db))
        self.ui.ln_path_rsc_db.setCursorPosition(0)
        path_data = self.__config_data.PathDb
        self.ui.ln_data_gpkg.setText(str(path_data))
        self.ui.ln_data_gpkg.setCursorPosition(0)

    def __open_db_rsc(self):
        path_db=axipy.open_file_dialog("Rsc db (*.db);;","Выбрать Rsc db")
        if path_db is not None:
            self.ui.ln_path_rsc_db.setText(str(path_db))
            self.ui.ln_path_rsc_db.setCursorPosition(0)
            self.__isReady()
    def __open_gpkg(self):
        path_db = axipy.open_file_dialog("GeoPackage (*.gpkg);;", "Выбрать базу данных")
        if path_db is not None:
            self.ui.ln_data_gpkg.setText(str(path_db))
            self.ui.ln_data_gpkg.setCursorPosition(0)
            self.__isReady()
    def __isReady(self):
        path_rsc_db=self.ui.ln_path_rsc_db.text()
        path_gpkg = self.ui.ln_data_gpkg.text()
        if Path(path_rsc_db).exists() and Path(path_gpkg).exists():
            self.ui.pb_apply.setEnabled(True)
        else:
            self.ui.pb_apply.setEnabled(False)
    def __save_config(self):
        path_db_rsc=self.ui.ln_path_rsc_db.text()
        path_data=self.ui.ln_data_gpkg.text()
        config=ConfigRscDict(path_db_rsc,path_data)
        config.save()
        self.__observer_change_config.value=True
    def __loadConfig(self):
        path_new_config = axipy.open_file_dialog("Файл конфигурации (*.json);;", "Выбрать файл конфигурации",folder=ConfigRscDict.path_folder)
        if path_new_config is not None:
            self.__config_data=ConfigRscDict.loadFromFile(path_new_config)
            if self.__config_data is not None:
                self.__observer_change_config.value = False
            self.__observer_change_config.value = True



